C****************************************************************
C     PROGRAM     : BI-RCS.FOR                                  *
C     DATE        : FEBRUARY 2002                               *
C     AUTHOR      : LEVENT SEVGI                                *
C     METHOD      : FDTD + NTFF                                 *
C     PURPOSE     : ANGULAR RCS EXTRAPOLATION WITH A GIVEN      *
C                   ANGULAR STEPS                               *
C****************************************************************
      INCLUDE 'F-BIRCS.PAR'

      OPEN(UNIT=35,FILE='F-BIFAR.DAT')
      OPEN(UNIT=40,FILE='F-BINEAR.DAT')
      OPEN(UNIT=45,FILE='SOURCE.DAT')

C----------------- READ INPUT PARAMETERS ------------------------------
      OPEN(UNIT=30,FILE='BIRCS.INP')
      READ(30,*) NTYPE           !1-5  SCENARIO TYPE
      READ(30,*) O_LENGTH        !LONGEST DIMENSION 
      READ(30,*) NOX1,NOY1,NOZ1  ! OBSERVATION POINT-1
      READ(30,*) THINC,PHINC     !ANGLES OF INCIDENT FIELD
      READ(30,*) NPOLAR          ! 1 TETA, 0 PHI (POLARIZATION)
      READ(30,*) NPLANE          ! 1 VERTICAL , 0 HORIZONTAL
      READ(30,*) RCSANGLE        ! PHI (VERTICAL) OR TETA (HORIZONTAL)
      CLOSE(30)
C-----------------------------------------------------------------------

      IF (NPOLAR.EQ.1) THEN
       ETHINC=1.
       EPHINC=0.          
      ELSE
       ETHINC=0.
       EPHINC=1.
      ENDIF          

      IF (NTYPE.EQ.1) THEN
         DELX=O_LENGTH/80.
      ELSE
         DELX=O_LENGTH/40.
      ENDIF

      DELY=DELX
      DELZ=DELX

      CALL ZERO
      CALL SETUP
      CALL BUILD(NTYPE)

      IF(NPLANE.EQ.0) THEN
        TETA=RCSANGLE*PI/180.
      ELSEIF(NPLANE.EQ.1) THEN
        PHI=RCSANGLE*PI/180.
      ENDIF

      DELTA=2.5*PI/180. !STEERING ANGLE
      MAX=144
C**********************
C  MAIN FDTD LOOP     *
C**********************

      T=0.0
      DO 100 N=1,NSTOP
C
      WRITE (*,*) N

C********************
C ELECTRIC FIELDS   *
C********************
        CALL EXSFLD
        CALL EYSFLD
        CALL EZSFLD

C*******************************
C BOUNDARY CONDITIONS (MUR2)   *
C*******************************
        CALL RADEYX
        CALL RADEZX
        CALL RADEZY
        CALL RADEXY
        CALL RADEXZ
        CALL RADEYZ

        T=T+DT/2.

C***********************
C MAGNETIC FIELDS      *
C***********************
        CALL HXSFLD
        CALL HYSFLD
        CALL HZSFLD

	T=T+DT/2.

      ENXS=0.25*(EXS(NOX1,NOY1,NOZ1)+EXS(NOX1,NOY1,NOZ1+1)
     $     +EXS(NOX1,NOY1+1,NOZ1+1)+EXS(NOX1,NOY1+1,NOZ1))
      ENYS=0.25*(EYS(NOX1,NOY1,NOZ1)+EYS(NOX1+1,NOY1,NOZ1)
     $     +EYS(NOX1+1,NOY1,NOZ1+1)+EYS(NOX1,NOY1,NOZ1+1))
      ENZS=0.25*(EZS(NOX1,NOY1,NOZ1)+EZS(NOX1,NOY1+1,NOZ1)
     $     +EZS(NOX1+1,NOY1+1,NOZ1)+EZS(NOX1+1,NOY1,NOZ1))
      
	WRITE(40,*) T*1.E9,ENXS,ENYS,ENZS

C************************************
C NEAR FIELD - FAR FIELD TRANSFORM  *
C************************************
	CALL YAKUZ

      WRITE(45,*) T*1.E9,EXI(IC,JC,KC),EYI(IC,JC,KC),EZI(IC,JC,KC)
 100  CONTINUE
C**************************
C  END OF TIME LOOP       *
C**************************
      CLOSE(40)
      CLOSE(45)
C********************************
C    FAR FIELD EXTRAPOLATION    *
C********************************
C********************
C  RCS PLANE LOOP   *
C********************
      DO L=1,MAX+1
        IF(NPLANE.EQ.0) THEN
           PHI =(L-1)*DELTA
        ELSE
           TETA=(L-1)*DELTA
        ENDIF
        DO N=-MM,NSTOP+MM
           WTETA=WX(L,N)*COS(PHI)*COS(TETA)
     *        +WY(L,N)*SIN(PHI)*COS(TETA)-WZ(L,N)*SIN(TETA)
           WPHI=-WX(L,N)*SIN(PHI)+WY(L,N)*COS(PHI)

           UTETA=UX(L,N)*COS(PHI)*COS(TETA)
     *        +UY(L,N)*SIN(PHI)*COS(TETA)-UZ(L,N)*SIN(TETA)
           UPHI=-UX(L,N)*SIN(PHI)+UY(L,N)*COS(PHI)

           SABIT=12.E8*PI*DT
           FETETA = (-ETA0*WTETA-UPHI)/SABIT
           FEPHI  = (-ETA0*WPHI+UTETA)/SABIT

           WRITE(35,77) (L-1)*DELTA,N*DT*1.E9,FETETA,FEPHI
        END DO
      END DO

   77 FORMAT(2(F18.10,1X),2(F18.8,1X))
      CLOSE(35)

      STOP
      END

C*************************************************
      SUBROUTINE BUILD(NTYPE)
      INCLUDE 'F-BIRCS.PAR'

      IF(NTYPE.EQ.1) THEN
         CALL DCUBE(16,16,26,40,80,20,1)


C******** DIHEDRAL *******************
      ELSEIF(NTYPE.EQ.2) THEN
         CALL DCUBE(16,36,16,40,0,40,1)   
         CALL DCUBE(16,36,16,40,40,0,1)


C******** 4-ELEMENT ARRAY ************      
      ELSEIF(NTYPE.EQ.3) THEN
         CALL DCUBE(35,25,16,1,1,40,1)   
         CALL DCUBE(35,45,16,1,1,40,1)
         CALL DCUBE(35,65,16,1,1,40,1)
         CALL DCUBE(35,85,16,1,1,40,1)
      
       
C******** 2 * 4-ELEMENT ARRAY ************     
      ELSEIF(NTYPE.EQ.4) THEN
         CALL DCUBE(25,25,16,1,1,40,1)   
         CALL DCUBE(25,45,16,1,1,40,1)
         CALL DCUBE(25,65,16,1,1,40,1)
         CALL DCUBE(25,85,16,1,1,40,1)        
         
         CALL DCUBE(45,25,16,1,1,40,1)   
         CALL DCUBE(45,45,16,1,1,40,1)
         CALL DCUBE(45,65,16,1,1,40,1)
         CALL DCUBE(45,85,16,1,1,40,1)  

C******** PLATE (XZ) *******************
      ELSEIF(NTYPE.EQ.5) THEN
         CALL DCUBE(16,55,16,40,0,40,1)   
	END IF
      
      RETURN
      END

C********************************************************
      SUBROUTINE DCUBE (ISTART,JSTART,KSTART,NXWIDE,NYWIDE,NZWIDE,MTYPE)
      INCLUDE 'F-BIRCS.PAR'
C
      IMAX=ISTART+NXWIDE-1
      JMAX=JSTART+NYWIDE-1
      KMAX=KSTART+NZWIDE-1
C
      IF (NXWIDE.EQ.0) THEN
        DO 20 K=KSTART,KMAX
          DO 10 J=JSTART,JMAX
            IDTWO(ISTART,J,K)=MTYPE
            IDTWO(ISTART,J,K+1)=MTYPE
            IDTHRE(ISTART,J,K)=MTYPE
            IDTHRE(ISTART,J+1,K)=MTYPE
 10       CONTINUE
 20     CONTINUE
      ELSEIF (NYWIDE.EQ.0) THEN
        DO 40 K=KSTART,KMAX
          DO 30 I=ISTART,IMAX
            IDONE(I,JSTART,K)=MTYPE
            IDONE(I,JSTART,K+1)=MTYPE
            IDTHRE(I,JSTART,K)=MTYPE
            IDTHRE(I+1,JSTART,K)=MTYPE
 30       CONTINUE
 40     CONTINUE
      ELSEIF (NZWIDE.EQ.0) THEN
        DO 60 J=JSTART,JMAX
          DO 50 I=ISTART,IMAX
            IDONE(I,J,KSTART)=MTYPE
            IDONE(I,J+1,KSTART)=MTYPE
            IDTWO(I,J,KSTART)=MTYPE
            IDTWO(I+1,J,KSTART)=MTYPE
 50       CONTINUE
 60     CONTINUE
      ELSE
        DO 90 K=KSTART,KMAX
          DO 80 J=JSTART,JMAX
            DO 70 I=ISTART,IMAX
              IDONE(I,J,K)=MTYPE
              IDONE(I,J,K+1)=MTYPE
              IDONE(I,J+1,K+1)=MTYPE
              IDONE(I,J+1,K)=MTYPE
              IDTWO(I,J,K)=MTYPE
              IDTWO(I+1,J,K)=MTYPE
              IDTWO(I+1,J,K+1)=MTYPE
              IDTWO(I,J,K+1)=MTYPE
              IDTHRE(I,J,K)=MTYPE
              IDTHRE(I+1,J,K)=MTYPE
              IDTHRE(I+1,J+1,K)=MTYPE
              IDTHRE(I,J+1,K)=MTYPE
 70         CONTINUE
 80       CONTINUE
 90     CONTINUE
      ENDIF
      RETURN
      END
C*******************************************************
      SUBROUTINE SETUP
      INCLUDE 'F-BIRCS.PAR'
C
      C=3.E8
      CC1=1.0/C
C      PI=4.0*ATAN(1.0)
      BAND=1.E-9*C/(10.*DELX) 

C
C     CALCULATE DT--THE MAXIMUM TIME STEP ALLOWED BY THE
C     COURANT STABILITY CONDITION
C
      DTXI=C/DELX
      DTYI=C/DELY
      DTZI=C/DELZ
      DT=DELX/(2.*C)
C      DT=1./SQRT(DTXI**2+DTYI**2+DTZI**2)
      DT1=1.0/DT
C
C     SET OFFSET FOR COMPUTING INCIDENT FIELDS
C
      OFF=1.0
C
C     FIND DIRECTION COSINES FOR INCIDENT FIELD
C
      COSTH=COS(PI*THINC/180.)
      SINTH=SIN(PI*THINC/180.)
      COSPH=COS(PI*PHINC/180.)
      SINPH=SIN(PI*PHINC/180.)
C
C     FIND AMPLITUDE OF INCIDENT FIELD COMPONENTS
C
      AMPX=AMP*(ETHINC*COSTH*COSPH-EPHINC*SINPH)
      AMPY=AMP*(ETHINC*COSTH*SINPH+EPHINC*COSPH)
      AMPZ=AMP*(-ETHINC*SINTH)
C
C     FIND RELATIVE SPATIAL DELAY FOR X, Y, Z CELL DISPLACEMENT
C
      XDISP=-COSPH*SINTH
      YDISP=-SINPH*SINTH
      ZDISP=-COSTH
C
      DO 10 I=1,9
        EPS(I)=EPS0
        SIGMA(I)=0.0
 10   CONTINUE
C
      DTEDX=DT/(EPS0*DELX)
      DTEDY=DT/(EPS0*DELY)
      DTEDZ=DT/(EPS0*DELZ)
      DTMDX=DT/(XMU0*DELX)
      DTMDY=DT/(XMU0*DELY)
      DTMDZ=DT/(XMU0*DELZ)
C
C     LOSSY DIELECTRICS
C
      DO 20 I=2,9
        ESCTC(I)=EPS(I)/(EPS(I)+SIGMA(I) *DT)
        EINCC(I)=SIGMA(I) *DT/(EPS(I)+SIGMA(I) *DT)
        EDEVCN(I)=DT*(EPS(I)-EPS0)/(EPS(I)+SIGMA(I) *DT)
        ECRLX(I)=DT/((EPS(I)+SIGMA(I) *DT)*DELX)
        ECRLY(I)=DT/((EPS(I)+SIGMA(I) *DT)*DELY)
        ECRLZ(I)=DT/((EPS(I)+SIGMA(I) *DT)*DELZ)
 20   CONTINUE

C
C     FIND MAXIMUM SPATIAL DELAY TO MAKE SURE PULSE PROPAGATES
C     INTO SPACE PROPERLY.
C
      DELAY=0.0
      IF (XDISP.LT.0.) DELAY=DELAY-XDISP*NX1*DELX
      IF (YDISP.LT.0.) DELAY=DELAY-YDISP*NY1*DELY
      IF (ZDISP.LT.0.) DELAY=DELAY-ZDISP*NZ1*DELZ
C
C     COMPUTE OUTER RADIATION BOUNDARY CONDITION (ORBC) CONSTANTS
C
      CXD=(C*DT-DELX)/(C*DT+DELX)
      CYD=(C*DT-DELY)/(C*DT+DELY)
      CZD=(C*DT-DELZ)/(C*DT+DELZ)
C
      CXU=CXD
      CYU=CYD
      CZU=CZD
C
C     COMPUTE 2ND ORDER ORBC CONSTANTS
C
      CXX=2.*DELX/(C*DT+DELX)
      CYY=2.*DELY/(C*DT+DELY)
      CZZ=2.*DELZ/(C*DT+DELZ)
C
      CXFYD=DELX*C*DT*C*DT/(2.*DELY*DELY*(C*DT+DELX))
      CXFZD=DELX*C*DT*C*DT/(2.*DELZ*DELZ*(C*DT+DELX))
      CYFZD=DELY*C*DT*C*DT/(2.*DELZ*DELZ*(C*DT+DELY))
      CYFXD=DELY*C*DT*C*DT/(2.*DELX*DELX*(C*DT+DELY))
      CZFXD=DELZ*C*DT*C*DT/(2.*DELX*DELX*(C*DT+DELZ))
      CZFYD=DELZ*C*DT*C*DT/(2.*DELY*DELY*(C*DT+DELZ))

      RETURN
      END

C     ********************************************************
      SUBROUTINE EXSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=2,NZ1
        KK = K
        DO 20 J=2,NY1
          JJ = J
          DO 10 I=1,NX1
C
C           DETERMINE MATERIAL TYPE
C
            IF(IDONE(I,J,K).EQ.0) GO TO 100
            IF(IDONE(I,J,K).EQ.1) GO TO 200
            GO TO 300
C
C           FREE SPACE
C
 100        EXS(I,J,K)=EXS(I,J,K)+(HZS(I,J,K)-HZS(I,J-1,K))*DTEDY
     $                           -(HYS(I,J,K)-HYS(I,J,K-1))*DTEDZ
C
            GO TO 10
C
C           PERFECT CONDUCTOR
C
 200        II = I
            EXS(I,J,K)=-EXI(II,JJ,KK)
C
            GO TO 10
C
C           LOSSY DIELECTRIC
C
 300        II = I
            EXS(I,J,K)=EXS(I,J,K)*ESCTC(IDONE(I,J,K))
     $      -EINCC( IDONE(I,J,K))*EXI(II,JJ,KK)
     $      -EDEVCN(IDONE(I,J,K))*DEXI(II,JJ,KK)
     $      +(HZS(I,J,K)-HZS(I,J-1,K))*ECRLY(IDONE(I,J,K))
     $      -(HYS(I,J,K)-HYS(I,J,K-1))*ECRLZ(IDONE(I,J,K))
C
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE EYSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=2,NZ1
        KK = K
        DO 20 J=1,NY1
          JJ = J
          DO 10 I=2,NX1
C
C           DETERMINE MATERIAL TYPE
C
            IF(IDTWO(I,J,K).EQ.0) GO TO 100
            IF(IDTWO(I,J,K).EQ.1) GO TO 200
            GO TO 300
C
C           FREE SPACE
C
 100        EYS(I,J,K)=EYS(I,J,K)+(HXS(I,J,K)-HXS(I,J,K-1))*DTEDZ
     $                           -(HZS(I,J,K)-HZS(I-1,J,K))*DTEDX
C
            GO TO 10
C
C           PERFECT CONDUCTOR
C
 200        II = I
            EYS(I,J,K)=-EYI(II,JJ,KK)
C
            GO TO 10
C
C           LOSSY DIELECTRIC
C
 300        II = I
            EYS(I,J,K)=EYS(I,J,K)*ESCTC(IDTWO(I,J,K))
     $      -EINCC( IDTWO(I,J,K))*EYI(II,JJ,KK)
     $      -EDEVCN(IDTWO(I,J,K))*DEYI(II,JJ,KK)
     $      +(HXS(I,J,K)-HXS(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZS(I,J,K)-HZS(I-1,J,K))*ECRLX(IDTWO(I,J,K))
C
  10      CONTINUE
  20    CONTINUE
  30  CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE EZSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=1,NZ1
        KK = K
        DO 20 J=2,NY1
          JJ = J
          DO 10 I=2,NX1
C
C           DETERMINE MATERIAL TYPE
C
            IF(IDTHRE(I,J,K).EQ.0) GO TO 100
            IF(IDTHRE(I,J,K).EQ.1) GO TO 200
            GO TO 300
C
C           FREE SPACE
C
 100        EZS(I,J,K)=EZS(I,J,K)+(HYS(I,J,K)-HYS(I-1,J,K))*DTEDX
     $                           -(HXS(I,J,K)-HXS(I,J-1,K))*DTEDY
C
            GO TO 10
C
C           PERFECT CONDUCTOR
C
 200        II = I
            EZS(I,J,K)=-EZI(II,JJ,KK)
C
            GO TO 10
C
C           LOSSY DIELECTRIC
C
 300        II = I
            EZS(I,J,K)=EZS(I,J,K)*ESCTC(IDTHRE(I,J,K))
     $      -EINCC( IDTHRE(I,J,K))*EZI(II,JJ,KK)
     $      -EDEVCN(IDTHRE(I,J,K))*DEZI(II,JJ,KK)
     $      +(HYS(I,J,K)-HYS(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HXS(I,J,K)-HXS(I,J-1,K))*ECRLY(IDTHRE(I,J,K))
C
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEZX
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 K=1,NZ1
        J=2
        EZS(1,J,K)=EZSX1(2,J,K)+CXD*(EZS(2,J,K)-EZSX1(1,J,K))
        EZS(NX,J,K)=EZSX1(3,J,K)+CXU*(EZS(NX1,J,K)-EZSX1(4,J,K))
        J=NY1
        EZS(1,J,K)=EZSX1(2,J,K)+CXD*(EZS(2,J,K)-EZSX1(1,J,K))
        EZS(NX,J,K)=EZSX1(3,J,K)+CXU*(EZS(NX1,J,K)-EZSX1(4,J,K))
 10   CONTINUE
C
      DO 20 J=3,NY1-1
        K=1
        EZS(1,J,K)=EZSX1(2,J,K)+CXD*(EZS(2,J,K)-EZSX1(1,J,K))
        EZS(NX,J,K)=EZSX1(3,J,K)+CXU*(EZS(NX1,J,K)-EZSX1(4,J,K))
        K=NZ1
        EZS(1,J,K)=EZSX1(2,J,K)+CXD*(EZS(2,J,K)-EZSX1(1,J,K))
        EZS(NX,J,K)=EZSX1(3,J,K)+CXU*(EZS(NX1,J,K)-EZSX1(4,J,K))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 K=2,NZ1-1
        DO 30 J=3,NY1-1
          EZS(1,J,K)=-EZSX2(2,J,K)+CXD*(EZS(2,J,K)+EZSX2(1,J,K))
     $    +CXX*(EZSX1(1,J,K)+EZSX1(2,J,K))
     $    +CXFYD*(EZSX1(1,J+1,K)-2.*EZSX1(1,J,K)+EZSX1(1,J-1,K)
     $    +EZSX1(2,J+1,K)-2.*EZSX1(2,J,K)+EZSX1(2,J-1,K))
     $    +CXFZD*(EZSX1(1,J,K+1)-2.*EZSX1(1,J,K)+EZSX1(1,J,K-1)
     $    +EZSX1(2,J,K+1)-2.*EZSX1(2,J,K)+EZSX1(2,J,K-1))
          EZS(NX,J,K)=-EZSX2(3,J,K)+CXD*(EZS(NX1,J,K)+EZSX2(4,J,K))
     $    +CXX*(EZSX1(4,J,K)+EZSX1(3,J,K))
     $    +CXFYD*(EZSX1(4,J+1,K)-2.*EZSX1(4,J,K)+EZSX1(4,J-1,K)
     $    +EZSX1(3,J+1,K)-2.*EZSX1(3,J,K)+EZSX1(3,J-1,K))
     $    +CXFZD*(EZSX1(4,J,K+1)-2.*EZSX1(4,J,K)+EZSX1(4,J,K-1)
     $    +EZSX1(3,J,K+1)-2.*EZSX1(3,J,K)+EZSX1(3,J,K-1))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 K=1,NZ1
        DO 50 J=2,NY1
          EZSX2(1,J,K)=EZSX1(1,J,K)
          EZSX2(2,J,K)=EZSX1(2,J,K)
          EZSX2(3,J,K)=EZSX1(3,J,K)
          EZSX2(4,J,K)=EZSX1(4,J,K)
          EZSX1(1,J,K)=EZS(1,J,K)
          EZSX1(2,J,K)=EZS(2,J,K)
          EZSX1(3,J,K)=EZS(NX1,J,K)
          EZSX1(4,J,K)=EZS(NX,J,K)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEYX
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 K=2,NZ1
        J=1
        EYS(1,J,K)=EYSX1(2,J,K)+CXD*(EYS(2,J,K)-EYSX1(1,J,K))
        EYS(NX,J,K)=EYSX1(3,J,K)+CXU*(EYS(NX1,J,K)-EYSX1(4,J,K))
        J=NY1
        EYS(1,J,K)=EYSX1(2,J,K)+CXD*(EYS(2,J,K)-EYSX1(1,J,K))
        EYS(NX,J,K)=EYSX1(3,J,K)+CXU*(EYS(NX1,J,K)-EYSX1(4,J,K))
 10   CONTINUE
C
      DO 20 J=2,NY1-1
        K=2
        EYS(1,J,K)=EYSX1(2,J,K)+CXD*(EYS(2,J,K)-EYSX1(1,J,K))
        EYS(NX,J,K)=EYSX1(3,J,K)+CXU*(EYS(NX1,J,K)-EYSX1(4,J,K))
        K=NZ1
        EYS(1,J,K)=EYSX1(2,J,K)+CXD*(EYS(2,J,K)-EYSX1(1,J,K))
        EYS(NX,J,K)=EYSX1(3,J,K)+CXU*(EYS(NX1,J,K)-EYSX1(4,J,K))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 K=3,NZ1-1
        DO 30 J=2,NY1-1
          EYS(1,J,K)=-EYSX2(2,J,K)+CXD*(EYS(2,J,K)+EYSX2(1,J,K))
     $    +CXX*(EYSX1(1,J,K)+EYSX1(2,J,K))
     $    +CXFYD*(EYSX1(1,J+1,K)-2.*EYSX1(1,J,K)+EYSX1(1,J-1,K)
     $    +EYSX1(2,J+1,K)-2.*EYSX1(2,J,K)+EYSX1(2,J-1,K))
     $    +CXFZD*(EYSX1(1,J,K+1)-2.*EYSX1(1,J,K)+EYSX1(1,J,K-1)
     $    +EYSX1(2,J,K+1)-2.*EYSX1(2,J,K)+EYSX1(2,J,K-1))
          EYS(NX,J,K)=-EYSX2(3,J,K)+CXD*(EYS(NX1 ,J,K)+EYSX2( 4,J,K))
     $    +CXX*(EYSX1(4,J,K)+EYSX1(3,J,K))
     $    +CXFYD*(EYSX1(4,J+1,K)-2.*EYSX1(4,J,K)+EYSX1(4,J-1,K)
     $    +EYSX1(3,J+1,K)-2.*EYSX1(3,J,K)+EYSX1(3,J-1,K))
     $    +CXFZD*(EYSX1(4,J,K+1)-2.*EYSX1(4,J,K)+EYSX1(4,J,K-1)
     $    +EYSX1(3,J,K+1)-2.*EYSX1(3,J,K)+EYSX1(3,J,K-1))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 K=2,NZ1
        DO 50 J=1,NY1
          EYSX2(1,J,K)=EYSX1(1,J,K)
          EYSX2(2,J,K)=EYSX1(2,J,K)
          EYSX2(3,J,K)=EYSX1(3,J,K)
          EYSX2(4,J,K)=EYSX1(4,J,K)
          EYSX1(1,J,K)=EYS(1,J,K)
          EYSX1(2,J,K)=EYS(2,J,K)
          EYSX1(3,J,K)=EYS(NX1,J,K)
          EYSX1(4,J,K)=EYS(NX,J,K)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEZY
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 K=1,NZ1
        I=2
        EZS(I,1,K)=EZSY1(I,2,K)+CYD*(EZS(I,2,K)-EZSY1(I,1,K))
        EZS(I,NY,K)=EZSY1(I,3,K)+CYD*(EZS(I,NY1,K)-EZSY1(I,4,K))
        I=NX1
        EZS(I,1,K)=EZSY1(I,2,K)+CYD*(EZS(I,2,K)-EZSY1(I,1,K))
        EZS(I,NY,K)=EZSY1(I,3,K)+CYD*(EZS(I,NY1,K)-EZSY1(I,4,K))
 10   CONTINUE
C
      DO 20 I=3,NX1-1
        K=1
        EZS(I,1,K)=EZSY1(I,2,K)+CYD*(EZS(I,2,K)-EZSY1(I,1,K))
        EZS(I,NY,K)=EZSY1(I,3,K)+CYD*(EZS(I,NY1,K)-EZSY1(I,4,K))
        K=NZ1
        EZS(I,1,K)=EZSY1(I,2,K)+CYD*(EZS(I,2,K)-EZSY1(I,1,K))
        EZS(I,NY,K)=EZSY1(I,3,K)+CYD*(EZS(I,NY1,K)-EZSY1(I,4,K))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 K=2,NZ1-1
        DO 30 I=3,NX1-1
          EZS(I,1,K)=-EZSY2(I,2,K)+CYD*(EZS(I,2,K)+EZSY2(I,1,K))
     $    +CYY*(EZSY1(I,1,K)+EZSY1(I,2,K))
     $    +CYFXD*(EZSY1(I+1,1,K)-2.*EZSY1(I,1,K)+EZSY1(I-1,1,K)
     $    +EZSY1(I+1,2,K)-2.*EZSY1(I,2,K)+EZSY1(I-1,2,K))
     $    +CYFZD*(EZSY1(I,1,K+1)-2.*EZSY1(I,1,K)+EZSY1(I,1,K-1)
     $    +EZSY1(I,2,K+1)-2.*EZSY1(I,2,K)+EZSY1(I,2,K-1))
          EZS(I,NY,K)=-EZSY2(I,3,K)+CYD*(EZS(I,NY1,K)+EZSY2(I,4,K))
     $    +CYY*(EZSY1(I,4,K)+EZSY1(I,3,K))
     $    +CYFXD*(EZSY1(I+1,4,K)-2.*EZSY1(I,4,K)+EZSY1(I-1,4,K)
     $    +EZSY1(I+1,3,K)-2.*EZSY1(I,3,K)+EZSY1(I-1,3,K))
     $    +CYFZD*(EZSY1(I,4,K+1)-2.*EZSY1(I,4,K)+EZSY1(I,4,K-1)
     $    +EZSY1(I,3,K+1)-2.*EZSY1(I,3,K)+EZSY1(I,3,K-1))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 K=1,NZ1
        DO 50 I=2,NX1
          EZSY2(I,1,K)=EZSY1(I,1,K)
          EZSY2(I,2,K)=EZSY1(I,2,K)
          EZSY2(I,3,K)=EZSY1(I,3,K)
          EZSY2(I,4,K)=EZSY1(I,4,K)
          EZSY1(I,1,K)=EZS(I,1,K)
          EZSY1(I,2,K)=EZS(I,2,K)
          EZSY1(I,3,K)=EZS(I,NY1,K)
          EZSY1(I,4,K)=EZS(I,NY,K)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEXY
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 K=2,NZ1
        I=1
        EXS(I,1,K)=EXSY1(I,2,K)+CYD*(EXS(I,2,K)-EXSY1(I,1,K))
        EXS(I,NY,K)=EXSY1(I,3,K)+CYD*(EXS(I,NY1,K)-EXSY1(I,4,K))
        I=NX1
        EXS(I,1,K)=EXSY1(I,2,K)+CYD*(EXS(I,2,K)-EXSY1(I,1,K))
        EXS(I,NY,K)=EXSY1(I,3,K)+CYD*(EXS(I,NY1,K)-EXSY1(I,4,K))
  10  CONTINUE
C
      DO 20 I=2,NX1-1
        K=2
        EXS(I,1,K)=EXSY1(I,2,K)+CYD*(EXS(I,2,K)-EXSY1(I,1,K))
        EXS(I,NY,K)=EXSY1(I,3,K)+CYD*(EXS(I,NY1,K)-EXSY1(I,4,K))
        K=NZ1
        EXS(I,1,K)=EXSY1(I,2,K)+CYD*(EXS(I,2,K)-EXSY1(I,1,K))
        EXS(I,NY,K)=EXSY1(I,3,K)+CYD*(EXS(I,NY1,K)-EXSY1(I,4,K))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 K=3,NZ1-1
        DO 30 I=2,NX1-1
          EXS(I,1,K)=-EXSY2(I,2,K)+CYD*(EXS(I,2,K)+EXSY2(I,1,K))
     $    +CYY*(EXSY1(I,1,K)+EXSY1(I,2,K))
     $    +CYFXD*(EXSY1(I+1,1,K)-2.*EXSY1(I,1,K)+EXSY1(I-1,1,K)
     $    +EXSY1(I+1,2,K)-2.*EXSY1(I,2,K)+EXSY1(I-1,2,K))
     $    +CYFZD*(EXSY1(I,1,K+1)-2.*EXSY1(I,1,K)+EXSY1(I,1,K-1)
     $    +EXSY1(I,2,K+1)-2.*EXSY1(I,2,K)+EXSY1(I,2,K-1))
          EXS(I,NY,K)=-EXSY2(I,3,K)+CYD*(EXS(I,NY1,K)+EXSY2(I,4,K))
     $    +CYY*(EXSY1(I,4,K)+EXSY1(I,3,K))
     $    +CYFXD*(EXSY1(I+1,4,K)-2.*EXSY1(I,4,K)+EXSY1(I-1,4,K)
     $    +EXSY1(I+1,3,K)-2.*EXSY1(I,3,K)+EXSY1(I-1,3,K))
     $    +CYFZD*(EXSY1(I,4,K+1)-2.*EXSY1(I,4,K)+EXSY1(I,4,K-1)
     $    +EXSY1(I,3,K+1)-2.*EXSY1(I,3,K)+EXSY1(I,3,K-1))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 K=2,NZ1
        DO 50 I=1,NX1
          EXSY2(I,1,K)=EXSY1(I,1,K)
          EXSY2(I,2,K)=EXSY1(I,2,K)
          EXSY2(I,3,K)=EXSY1(I,3,K)
          EXSY2(I,4,K)=EXSY1(I,4,K)
          EXSY1(I,1,K)=EXS(I,1,K)
          EXSY1(I,2,K)=EXS(I,2,K)
          EXSY1(I,3,K)=EXS(I,NY1,K)
          EXSY1(I,4,K)=EXS(I,NY,K)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEXZ
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 J=2,NY1
        I=1
        EXS(I,J,1)=EXSZ1(I,J,2)+CZD*(EXS(I,J,2)-EXSZ1(I,J,1))
        EXS(I,J,NZ)=EXSZ1(I,J,3)+CZD*(EXS(I,J,NZ1)-EXSZ1(I,J,4))
        I=NX1
        EXS(I,J,1)=EXSZ1(I,J,2)+CZD*(EXS(I,J,2)-EXSZ1(I,J,1))
        EXS(I,J,NZ)=EXSZ1(I,J,3)+CZD*(EXS(I,J,NZ1)-EXSZ1(I,J,4))
 10   CONTINUE
C
      DO 20 I=2,NX1-1
        J=2
        EXS(I,J,1)=EXSZ1(I,J,2)+CZD*(EXS(I,J,2)-EXSZ1(I,J,1))
        EXS(I,J,NZ)=EXSZ1(I,J,3)+CZD*(EXS(I,J,NZ1)-EXSZ1(I,J,4))
        J=NY1
        EXS(I,J,1)=EXSZ1(I,J,2)+CZD*(EXS(I,J,2)-EXSZ1(I,J,1))
        EXS(I,J,NZ)=EXSZ1(I,J,3)+CZD*(EXS(I,J,NZ1)-EXSZ1(I,J,4))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 J=3,NY1-1
        DO 30 I=2,NX1-1
          EXS(I,J,1)=-EXSZ2(I,J,2)+CZD*(EXS(I,J,2)+EXSZ2(I,J,1))
     $   +CZZ*(EXSZ1(I,J,1)+EXSZ1(I,J,2))
     $   +CZFXD*(EXSZ1(I+1,J,1)-2.*EXSZ1(I,J,1)+EXSZ1(I-1,J,1)
     $   +EXSZ1(I+1,J,2)-2.*EXSZ1(I,J,2)+EXSZ1(I-1,J,2))
     $   +CZFYD*(EXSZ1(I,J+1,1)-2.*EXSZ1(I,J,1)+EXSZ1(I,J-1,1)
     $   +EXSZ1(I,J+1,2)-2.*EXSZ1(I,J,2)+EXSZ1(I,J-1,2))
         EXS(I,J,NZ)=-EXSZ2(I,J,3)+CZD*(EXS(I,J,NZ1)+EXSZ2(I,J,4))
     $   +CZZ*(EXSZ1(I,J,4)+EXSZ1(I,J,3))
     $   +CZFXD*(EXSZ1(I+1,J,4)-2.*EXSZ1(I,J,4)+EXSZ1(I-1,J,4)
     $   +EXSZ1(I+1,J,3)-2.*EXSZ1(I,J,3)+EXSZ1(I-1,J,3))
     $   +CZFYD*(EXSZ1(I,J+1,4)-2.*EXSZ1(I,J,4)+EXSZ1(I,J-1,4)
     $   +EXSZ1(I,J+1,3)-2.*EXSZ1(I,J,3)+EXSZ1(I,J-1,3))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 J=2,NY1
        DO 50 I=1,NX1
          EXSZ2(I,J,1)=EXSZ1(I,J,1)
          EXSZ2(I,J,2)=EXSZ1(I,J,2)
          EXSZ2(I,J,3)=EXSZ1(I,J,3)
          EXSZ2(I,J,4)=EXSZ1(I,J,4)
          EXSZ1(I,J,1)=EXS(I,J,1)
          EXSZ1(I,J,2)=EXS(I,J,2)
          EXSZ1(I,J,3)=EXS(I,J,NZ1)
          EXSZ1(I,J,4)=EXS(I,J,NZ)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE RADEYZ
      INCLUDE 'F-BIRCS.PAR'
C
      DO 10 J=1,NY1
        I=2
        EYS(I,J,1)=EYSZ1(I,J,2)+CZD*(EYS(I,J,2)-EYSZ1(I,J,1))
        EYS(I,J,NZ)=EYSZ1(I,J,3)+CZD*(EYS(I,J,NZ1)-EYSZ1(I,J,4))
        I=NX1
        EYS(I,J,1)=EYSZ1(I,J,2)+CZD*(EYS(I,J,2)-EYSZ1(I,J,1))
        EYS(I,J,NZ)=EYSZ1(I,J,3)+CZD*(EYS(I,J,NZ1)-EYSZ1(I,J,4))
 10   CONTINUE
C
      DO 20 I=3,NX1-1
        J=1
        EYS(I,J,1)=EYSZ1(I,J,2)+CZD*(EYS(I,J,2)-EYSZ1(I,J,1))
        EYS(I,J,NZ)=EYSZ1(I,J,3)+CZD*(EYS(I,J,NZ1)-EYSZ1(I,J,4))
        J=NY1
        EYS(I,J,1)=EYSZ1(I,J,2)+CZD*(EYS(I,J,2)-EYSZ1(I,J,1))
        EYS(I,J,NZ)=EYSZ1(I,J,3)+CZD*(EYS(I,J,NZ1)-EYSZ1(I,J,4))
 20   CONTINUE
C
C     NOW DO 2ND ORDER ORBC ON REMAINING PORTIONS OF FACES
C
      DO 40 J=2,NY1-1
        DO 30 I=3,NX1-1
          EYS(I,J,1)=-EYSZ2(I,J,2)+CZD*(EYS(I,J,2)+EYSZ2(I,J,1))
     $    +CZZ*(EYSZ1(I,J,1)+EYSZ1(I,J,2))
     $    +CZFXD*(EYSZ1(I+1,J,1)-2.*EYSZ1(I,J,1)+EYSZ1(I-1,J,1)
     $    +EYSZ1(I+1,J,2)-2.*EYSZ1(I,J,2)+EYSZ1(I-1,J,2))
     $    +CZFYD*(EYSZ1(I,J+1,1)-2.*EYSZ1(I,J,1)+EYSZ1(I,J-1,1)
     $    +EYSZ1(I,J+1,2)-2.*EYSZ1(I,J,2)+EYSZ1(I,J-1,2))
          EYS(I,J,NZ)=-EYSZ2(I,J,3)+CZD*(EYS(I,J,NZ1)+EYSZ2(I,J,4))
     $    +CZZ*(EYSZ1(I,J,4)+EYSZ1(I,J,3))
     $    +CZFXD*(EYSZ1(I+1,J,4)-2.*EYSZ1(I,J,4)+EYSZ1(I-1,J,4)
     $    +EYSZ1(I+1,J,3)-2.*EYSZ1(I,J,3)+EYSZ1(I-1,J,3))
     $    +CZFYD*(EYSZ1(I,J+1,4)-2.*EYSZ1(I,J,4)+EYSZ1(I,J-1,4)
     $    +EYSZ1(I,J+1,3)-2.*EYSZ1(I,J,3)+EYSZ1(I,J-1,3))
 30     CONTINUE
 40   CONTINUE
C
C     NOW SAVE PAST VALUES
C
      DO 60 J=1,NY1
        DO 50 I=2,NX1
          EYSZ2(I,J,1)=EYSZ1(I,J,1)
          EYSZ2(I,J,2)=EYSZ1(I,J,2)
          EYSZ2(I,J,3)=EYSZ1(I,J,3)
          EYSZ2(I,J,4)=EYSZ1(I,J,4)
          EYSZ1(I,J,1)=EYS(I,J,1)
          EYSZ1(I,J,2)=EYS(I,J,2)
          EYSZ1(I,J,3)=EYS(I,J,NZ1)
          EYSZ1(I,J,4)=EYS(I,J,NZ)
 50     CONTINUE
 60   CONTINUE
C
      RETURN
      END
C     *****************************************************************
      SUBROUTINE HXSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=1,NZ1
        DO 20 J=1,NY1
          DO 10 I=2,NX1
            HXS(I,J,K)=HXS(I,J,K)-(EZS(I,J+1,K)-EZS(I,J,K))*DTMDY
     $                           +(EYS(I,J,K+1)-EYS(I,J,K))*DTMDZ
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HYSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=1,NZ1
        DO 20 J=2,NY1
          DO 10 I=1,NX1
            HYS(I,J,K)=HYS(I,J,K)-(EXS(I,J,K+1)-EXS(I,J,K))*DTMDZ
     $                           +(EZS(I+1,J,K)-EZS(I,J,K))*DTMDX
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HZSFLD
      INCLUDE 'F-BIRCS.PAR'
C
      DO 30 K=2,NZ1
        DO 20 J=1,NY1
          DO 10 I=1,NX1
            HZS(I,J,K)=HZS(I,J,K)-(EYS(I+1,J,K)-EYS(I,J,K))*DTMDX
     $                           +(EXS(I,J+1,K)-EXS(I,J,K))*DTMDY
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END

C     ******************************************************************
      FUNCTION EXI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     THIS FUNCTION COMPUTES THE X COMPONENT OF THE INCIDENT
C     ELECTRIC FIELD
C
      DIST=((I-1)*DELX+0.5*DELX*OFF)*XDISP+((J-1)*DELY)*YDISP+
     $((K-1)*DELZ)*ZDISP + DELAY
      EXI = AMPX*SOURCE(DIST)
      RETURN
      END
C     ***************************************************************
      FUNCTION EYI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     THIS FUNCTION COMPUTES THE Y COMPONENT OF THE INCIDENT
C     ELECTRIC FIELD
C
      DIST=((I-1)*DELX)*XDISP+((J-1)*DELY+0.5*DELY*OFF)*
     $YDISP+((K-1)*DELZ)*ZDISP + DELAY
      EYI = AMPY*SOURCE(DIST)
      RETURN
      END
C     ************************************************************
      FUNCTION EZI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     THIS FUNCTION COMPUTES THE Z COMPONENT OF THE INCIDENT
C     ELECTRIC FIELD
C
      DIST=((I-1)*DELX)*XDISP+((J-1)*DELY)*YDISP+((K-1)*DELZ+
     $0.5*DELZ*OFF)*ZDISP + DELAY
      EZI = AMPZ*SOURCE(DIST)
      RETURN
      END
C     ********************************************************
      FUNCTION SOURCE(DIST)
      INCLUDE 'F-BIRCS.PAR'
C
C     THIS FUNCTION DEFINES THE FUNCTIONAL FORM OF THE INCIDENT
C     FIELD
C
      SOURCE = 0.0
C
C     EXECUTE FOLLOWING FOR INCIDENT E FIELD
C
      TAU=T-DIST/C
c      ALPHA=3.3E18*BAND*BAND     ! BAND in GHz (gauss icin)
      ALPHA=2.74E18*BAND*BAND     ! BAND in GHz (gauss turevi icin)

      SHIFT=4.0/SQRT(ALPHA)
C

c      SOURCE=EXP(-ALPHA*(TAU-SHIFT)**2)
      SOURCE=-2.3*SQRT(ALPHA)*(TAU-SHIFT)*EXP(-ALPHA*(TAU-SHIFT)**2)
C        SOURCE=SIN(2.*PI*FO*1.E9*TAU)
C
C
 10   RETURN
      END
C     ***************************************************************
      FUNCTION DEXI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     TIME DERIVATIVE OF INCIDENT EX FIELD FOR DIELECTRICS
C
      DIST=((I-1)*DELX+0.5*DELX*OFF)*XDISP+((J-1)*DELY)*YDISP+
     $((K-1)*DELZ)*ZDISP + DELAY
      DEXI=AMPX*DSRCE(DIST)
C
      RETURN
      END
C     ***************************************************************
      FUNCTION DEYI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     TIME DERIVATIVE OF INCIDENT EY FIELD FOR DIELECTRICS
C
      DIST=((I-1)*DELX)*XDISP+((J-1)*DELY+0.5*DELY*OFF)*YDISP+
     $((K-1)*DELZ)*ZDISP + DELAY
      DEYI=AMPY*DSRCE(DIST)
      RETURN
      END
C     ********************************************************
      FUNCTION DEZI(I,J,K)
      INCLUDE 'F-BIRCS.PAR'
C
C     TIME DERIVATIVE OF INCIDENT EZ FIELD FOR DIELECTRICS
C
      DIST=((I-1)*DELX)*XDISP+((J-1)*DELY)*YDISP+((K-1)*DELZ+
     $0.5*DELZ*OFF)*ZDISP + DELAY
      DEZI=AMPZ*DSRCE(DIST)
      RETURN
      END
C     ***************************************************************
      FUNCTION DSRCE(DIST)
      INCLUDE 'F-BIRCS.PAR'
C
C     TIME DERIVATIVE OF INCIDENT FIELD FOR DIELECTRICS
      DSRCE=0.0

C*****************************************
C       GAUSSIAN SOURCE FUNCTION         *
C*****************************************
      TAU=T-DIST/C
      IF(TAU.LT.0.0) GO TO 10
      IF (TAU.GT.PERIOD) GO TO 10
      DSRCE=EXP(-ALPHA*((TAU-BETADT)**2))*(-2.*ALPHA*(TAU-BETADT))

 10   RETURN
      END
C     ***************************************************************
      SUBROUTINE ZERO
      INCLUDE 'F-BIRCS.PAR'
      
C
      T=0.0
      DO 30 K=1,NZ
	DO 20 J=1,NY
	  DO 10 I=1,NX
	    EXS(I,J,K)=0.0
	    EYS(I,J,K)=0.0
	    EZS(I,J,K)=0.0
	    HXS(I,J,K)=0.0
	    HYS(I,J,K)=0.0
	    HZS(I,J,K)=0.0
	    IDONE(I,J,K)=0
	    IDTWO(I,J,K)=0
	    IDTHRE(I,J,K)=0
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
      DO 60 K=1,NZ1
	DO 50 J=1,NY1
	  DO 40 I=1,4
	    EYSX1(I,J,K)=0.0
	    EYSX2(I,J,K)=0.0
	    EZSX1(I,J,K)=0.0
	    EZSX2(I,J,K)=0.0
 40       CONTINUE
 50     CONTINUE
 60   CONTINUE
      DO 90 K=1,NZ1
	DO 80 J=1,4
	  DO 70 I=1,NX1
	    EXSY1(I,J,K)=0.0
	    EXSY2(I,J,K)=0.0
	    EZSY1(I,J,K)=0.0
	    EZSY2(I,J,K)=0.0
 70       CONTINUE
 80     CONTINUE
 90   CONTINUE
      DO 120 K=1,4
	DO 110 J=1,NY1
	  DO 100 I=1,NX1
	    EXSZ1(I,J,K)=0.0
	    EXSZ2(I,J,K)=0.0
	    EYSZ1(I,J,K)=0.0
	    EYSZ2(I,J,K)=0.0
 100      CONTINUE
 110    CONTINUE
 120  CONTINUE
      DO 130 L=1,9
	ESCTC(L)=0.0
	EINCC(L)=0.0
	EDEVCN(L)=0.0
	ECRLX(L)=0.0
	ECRLY(L)=0.0
	ECRLZ(L)=0.0
 130  CONTINUE

      DO L=1,MAX+1
       DO I=-MM,NSTOP+MM
	 UX(L,I)=0.0
	 UY(L,I)=0.0
	 UZ(L,I)=0.0
	 WX(L,I)=0.0
	 WY(L,I)=0.0
	 WZ(L,I)=0.0
       END DO
      END DO

      RETURN
      END

C*****************************************************************
      SUBROUTINE YAKUZ
      INCLUDE 'F-BIRCS.PAR'
C******************************************************************
C  NEAR TO FAR FIELD TRANSFORMATION FOR FDTD                      *
C-----------------------------------------------------------------*
C    IC,JC,KC   : CENTRAL COORDINATES                             *
C    NXS1, NXS2 : Xmin AND Xmax POINTS OF THE SURFACE             *
C    NYS1, NYS2 : Ymin AND Ymax POINTS OF THE SURFACE             *
C    NZS1, NZS2 : Zmin AND Zmax POINTS OF THE SURFACE             *
C    RRP        : LENGTH FROM ORIGIN TO THE POINT ON THE SURFACE  *
C    N          : FDTD TIME STEP                                  *
C    M          : TIME OF CONTRIBUTION TO FAR FIELD               *
C    E0,H0      : ELECTRIC AND MAGNETIC FLUX CONTRIBUTION         *
C    TETA       : ANGLE FROM Z AXIS                               *
C    PHI        : ANGLE FROM X AXIS AT Z=0                        *
C******************************************************************

C**********************
C   PLANE LOOP        *
C**********************

      DO L=1,MAX+1
         IF(NPLANE.EQ.0) THEN
	   PHI =(L-1)*DELTA
	   SPHI=SIN(PHI)
	   CPHI=COS(PHI)
	   STETA=SIN(TETA)
	   CTETA=COS(TETA)
	 ELSE
	   TETA=(L-1)*DELTA
	   SPHI=SIN(PHI)
	   CPHI=COS(PHI)
	   STETA=SIN(TETA)
	   CTETA=COS(TETA)
	 ENDIF
C*********************************************
C (1) X=Xmax         (Ey AND Ez) (Hy AND z)  *
C*********************************************
      I=NXS2
      ALAN=DELY*DELZ
        DO K=NZS1,NZS2
          DO J=NYS1,NYS2
	    RRP=DELX*(I+.5-IC)*STETA*CPHI+
     *          DELY*(J-JC)*STETA*SPHI+
     *          DELZ*(K-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=-.5*(EYS(I+1,J,K)+EYS(I+1,J,K+1))
	    UZ(L,M-1)= UZ(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UZ(L,M)  = UZ(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UZ(L,M+1)= UZ(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=.5*(EZS(I+1,J,K)+EZS(I+1,J+1,K))
	    UY(L,M-1)= UY(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UY(L,M)  = UY(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UY(L,M+1)= UY(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    H0=.25*(HYS(I,J,K)+HYS(I,J+1,K)
     *             +HYS(I+1,J,K)+HYS(I+1,J+1,K))
	    WZ(L,M-1) = WZ(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WZ(L,M)   = WZ(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WZ(L,M+1) = WZ(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=-.25*(HZS(I,J,K)+HZS(I,J,K+1)
     *             +HZS(I+1,J,K)+HZS(I+1,J,K+1))
	    WY(L,M-1) = WY(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WY(L,M)   = WY(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WY(L,M+1) = WY(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	  ENDDO
	ENDDO

C********************************************
C (2) X=Xmin        (Ey AND Ez) (Hy AND Hz) *
C********************************************
      I=NXS1
      ALAN=DELY*DELZ
       DO K=NZS1,NZS2
         DO J=NYS1,NYS2
	    RRP=DELX*(I-.5-IC)*STETA*CPHI+
     *          DELY*(J-JC)*STETA*SPHI+
     *          DELZ*(K-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=.5*(EYS(I,J,K)+EYS(I,J,K+1))
	    UZ(L,M-1)= UZ(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UZ(L,M)  = UZ(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UZ(L,M+1)= UZ(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=-.5*(EZS(I,J,K)+EZS(I,J+1,K))
	    UY(L,M-1)= UY(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UY(L,M)  = UY(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UY(L,M+1)= UY(L,M+1)-ALAN*E0*(.5+TC*DT1-M)


	    H0=-.25*(HYS(I,J,K)+HYS(I,J+1,K)
     *             +HYS(I-1,J,K)+HYS(I-1,J+1,K))
	    WZ(L,M-1) = WZ(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WZ(L,M)   = WZ(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WZ(L,M+1) = WZ(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=.25*(HZS(I,J,K)+HZS(I,J,K+1)
     *             +HZS(I-1,J,K)+HZS(I-1,J,K+1))
	    WY(L,M-1) = WY(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WY(L,M)   = WY(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WY(L,M+1) = WY(L,M+1)-ALAN*H0*(.5+TC*DT1-M)


	  ENDDO
	ENDDO

C********************************************
C (3) y=Ymax        (Ex AND Ez) (Hx AND Hz) *
C********************************************
      J=NYS2
      ALAN=DELX*DELZ
       DO K=NZS1,NZS2
         DO I=NXS1,NXS2
	    RRP=DELX*(I-IC)*STETA*CPHI+
     *          DELY*(J+.5-JC)*STETA*SPHI+
     *          DELZ*(K-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=.5*(EXS(I,J+1,K)+EXS(I,J+1,K+1))
	    UZ(L,M-1)= UZ(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UZ(L,M)  = UZ(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UZ(L,M+1)= UZ(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=-.5*(EZS(I,J+1,K)+EZS(I+1,J+1,K))
	    UX(L,M-1)= UX(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UX(L,M)  = UX(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UX(L,M+1)= UX(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    H0=-.25*(HXS(I,J,K)+HXS(I+1,J+1,K)
     *             +HXS(I+1,J,K)+HXS(I,J+1,K))
	    WZ(L,M-1) = WZ(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WZ(L,M)   = WZ(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WZ(L,M+1) = WZ(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=.25*(HZS(I,J,K)+HZS(I,J+1,K)
     *             +HZS(I,J+1,K+1)+HZS(I,J,K+1))
	    WX(L,M-1) = WX(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WX(L,M)   = WX(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WX(L,M+1) = WX(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	  ENDDO
	ENDDO

C********************************************
C (4) y=Ymin        (Ex AND Ez) (Hx AND Hz) *
C********************************************

      J=NYS1
      ALAN=DELX*DELZ
	  DO K=NZS1,NZS2
	DO I=NXS1,NXS2
	    RRP=DELX*(I-IC)*STETA*CPHI+
     *          DELY*(J-.5-JC)*STETA*SPHI+
     *          DELZ*(K-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=-.5*(EXS(I,J,K)+EXS(I,J,K+1))
	    UZ(L,M-1) = UZ(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UZ(L,M)   = UZ(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UZ(L,M+1) = UZ(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=.5*(EZS(I,J,K)+EZS(I+1,J,K))
	    UX(L,M-1) = UX(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UX(L,M)   = UX(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UX(L,M+1) = UX(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    H0=.25*(HXS(I,J,K)+HXS(I+1,J,K)
     *             +HXS(I+1,J-1,K)+HXS(I,J-1,K))
	    WZ(L,M-1) = WZ(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WZ(L,M)   = WZ(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WZ(L,M+1) = WZ(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=-.25*(HZS(I,J,K)+HZS(I,J,K+1)
     *             +HZS(I,J-1,K)+HZS(I,J-1,K+1))
	    WX(L,M-1) = WX(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WX(L,M)   = WX(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WX(L,M+1) = WX(L,M+1)-ALAN*H0*(.5+TC*DT1-M)


	  ENDDO
	ENDDO

C********************************************
C (5) z=Zmax        (Ex AND Ey) (Hx AND Hy) *
C********************************************

      K=NZS2
      ALAN=DELX*DELY
	  DO J=NYS1,NYS2
	DO I=NXS1,NXS2
	    RRP=DELX*(I-IC)*STETA*CPHI+
     *          DELY*(J-JC)*STETA*SPHI+
     *          DELZ*(K+.5-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=-.5*(EXS(I,J,K+1)+EXS(I,J+1,K+1))
	    UY(L,M-1) = UY(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UY(L,M)   = UY(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UY(L,M+1) = UY(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=.5*(EYS(I+1,J,K+1)+EYS(I,J,K+1))
	    UX(L,M-1) = UX(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UX(L,M)   = UX(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UX(L,M+1) = UX(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    H0=.25*(HXS(I,J,K)+HXS(I+1,J,K)
     *             +HXS(I,J,K+1)+HXS(I+1,J,K+1))
	    WY(L,M-1) = WY(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WY(L,M)   = WY(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WY(L,M+1) = WY(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=-.25*(HYS(I,J,K)+HYS(I,J+1,K)
     *             +HYS(I,J,K+1)+HYS(I,J+1,K+1))
	    WX(L,M-1) = WX(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WX(L,M)   = WX(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WX(L,M+1) = WX(L,M+1)-ALAN*H0*(.5+TC*DT1-M)


	  ENDDO
	ENDDO

C********************************************
C (6) z=Zmin        (Ex AND Ey) (Hx AND Hy) *
C********************************************

      K=NZS1
      ALAN=DELX*DELY
	  DO J=NYS1,NYS2
	DO I=NXS1,NXS2
	    RRP=DELX*(I-IC)*STETA*CPHI+
     *          DELY*(J-JC)*STETA*SPHI+
     *          DELZ*(K-.5-KC)*CTETA
            TC=N*DT-RRP*CC1
	    M=INT(TC*DT1+.5)

	    E0=.5*(EXS(I,J,K)+EXS(I,J+1,K))
	    UY(L,M-1) = UY(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UY(L,M)   = UY(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UY(L,M+1) = UY(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    E0=-.5*(EYS(I,J,K)+EYS(I+1,J,K))
	    UX(L,M-1) = UX(L,M-1)+ALAN*E0*(.5-TC*DT1+M)
	    UX(L,M)   = UX(L,M)+ALAN*E0*2*(TC*DT1-M)
	    UX(L,M+1) = UX(L,M+1)-ALAN*E0*(.5+TC*DT1-M)

	    H0=-.25*(HXS(I,J,K)+HXS(I+1,J,K)
     *             +HXS(I,J,K-1)+HXS(I+1,J,K-1))
	    WY(L,M-1) = WY(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WY(L,M)   = WY(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WY(L,M+1) = WY(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	    H0=.25*(HYS(I,J,K)+HYS(I,J+1,K)
     *             +HYS(I,J,K-1)+HYS(I,J+1,K-1))
	    WX(L,M-1) = WX(L,M-1)+ALAN*H0*(.5-TC*DT1+M)
	    WX(L,M)   = WX(L,M)+ALAN*H0*2*(TC*DT1-M)
	    WX(L,M+1) = WX(L,M+1)-ALAN*H0*(.5+TC*DT1-M)

	  ENDDO
	ENDDO
C*************************
C  END OF PLANE LOOP     *
C*************************
      ENDDO

	RETURN
	END
