C*******************************************************
C     PROGRAM     : FDSE.FOR                           *
C     AUTHOR      : FUNDA AKLEMAN & ORHAN OZYALXIN     *
C     PURPOSE     : SE SIMULATIONS WITH FDTD+PML       *
C     DATE        : MARCH 2002                         *
C*******************************************************
      PROGRAM FDSE
      INCLUDE 'FDSE.PAR'

      CALL ZERO
      OPEN(44,FILE='Source.dat')

      OPEN(15,FILE='SE.INP')
C**********************************************
C   READ INPUT PARAMETERS FROM INPUT FILE     *
C**********************************************
       READ (15,*) N1,N2,N3,N4
       IF(N1.GT.2.OR.N2.GT.2.OR.N3.GT.2.OR.N4.GT.2) THEN
        WRITE(*,*) 'WRONG PARAMETER IN INPUT FILE'
        STOP
       END IF
       READ (15,*) NSOURCE
       IF(NSOURCE.GT.3) THEN
        WRITE(*,*) 'WRONG PARAMETER IN INPUT FILE'
        STOP
       END IF
       READ (15,*) NOBS
       DO I=1,NOBS
        READ (15,*) NOX(I),NOY(I),NOZ(I), NOUT(I)
       ENDDO
       READ (15,*) NSTOP(2)
       READ (15,*) NHAN
       CLOSE (15)
C**************************************************

      ESIGMAM=-3.*EPS0*3.E8*ALOG(R_TEO)/(2.* NPML*DELX)
      HSIGMAM=ESIGMAM*XMU0/EPS0
C
      CALL SETUP
      CALL ANTENNA
C**********************
C  PML SUBROTINES     *
C**********************
      CALL PMLCUBE
      CALL PMLSETUP

C*************************************
C  FREE SPACE AND SHIELD LOOPS       *
C*************************************
      DO NC=1,2
        IF(NC.EQ.1) THEN
          OPEN(12,FILE='F-FREE.DAT')
        ELSE
          CALL ZERO
          CALL ANTENNA
          CALL STRUCTURE
          OPEN(12,FILE='F-EUT.DAT')
        ENDIF
C**********************
C  MAIN FDTD LOOP     *
C**********************
      T=0.0
      DO 100 N=1,NSTOP(NC)
       WRITE (*,*) N

C********************
C ELECTRIC FILEDS   *
C********************
	CALL EXFLD
        CALL EYFLD
	CALL EZFLD

	T=T+DT/2.
           
C**********************
C MAGNETIC FILEDS     *
C**********************
	CALL HXFLD
        CALL HYFLD
        CALL HZFLD
        
	T=T+DT/2.
        CALL OUTPUT
100   CONTINUE
C**************************
C  END OF TIME LOOP       *
C**************************
        CLOSE(12)
      ENDDO
      
      STOP
      END

C*************************************************
      SUBROUTINE OUTPUT
      INCLUDE 'FDSE.PAR'

       DO L=1,NOBS
        I=NOX(L)
        J=NOY(L)
        K=NOZ(L)
        IF(NOUT(L).EQ.1) THEN
            EAV(L)=0.25*(EX(I,J,K)+EX(I,J+1,K)
     >              +EX(I,J+1,K+1)+EX(I+1,J,K+1))
        ELSEIF(NOUT(L).EQ.2) THEN
            EAV(L)=0.25*(EY(I,J,K)+EY(I+1,J,K)
     >              +EY(I,J+1,K+1)+EY(I+1,J,K+1))
        ELSEIF(NOUT(L).EQ.3) THEN
            EAV(L)=0.25*(EZ(I,J,K)+EZ(I+1,J,K)
     >              +EZ(I,J+1,K)+EZ(I+1,J+1,K))
       END IF
       ENDDO

       WRITE(12,300) T*1.E9,(EAV(L),L=1,NOBS)
300    FORMAT(F13.9,1X,4(F13.10,1X))
      RETURN
      END

C*************************************************
      SUBROUTINE ANTENNA
      INCLUDE 'FDSE.PAR'
        IF(NSOURCE.EQ.1) THEN   ! X-DIRECTED DIPOLE
           CALL DCUBE(NKX-NANT,NKY,NKZ,NANT,1,1,1)
           CALL DCUBE(NKX+1,NKY,NKZ,NANT,1,1,1)
        ELSEIF(NSOURCE.EQ.2) THEN   ! Y-DIRECTED DIPOLE
           CALL DCUBE(NKX,NKY-NANT,NKZ,1,NANT,1,1)
           CALL DCUBE(NKX,NKY+1,NKZ,1,NANT,1,1)
        ELSEIF(NSOURCE.EQ.3) THEN   ! Z-DIRECTED DIPOLE
           CALL DCUBE(NKX,NKY,NKZ-NANT,1,1,NANT,1)
           CALL DCUBE(NKX,NKY,NKZ+1,1,1,NANT,1)
        ENDIF
      RETURN
      END

C*************************************************
      SUBROUTINE STRUCTURE
      INCLUDE 'FDSE.PAR'

        CALL DCUBE(21,21,21,10,10,0,1) ! BOTTOM PLANE
        CALL DCUBE(21,21,31,10,10,0,1) ! TOP PLANE

C**********************************
C   FIRST PANEL (X=A)             *
C**********************************
        II=31
        IF(N1.EQ.0) THEN
          CALL DCUBE(II,21,21,0,10,10,1)  ! NO APERTURE 
        ELSEIF(N1.EQ.1) THEN
          CALL DCUBE(II,21,21,0,2,10,1)   ! SINGLE H APERTURE (2*6)
          CALL DCUBE(II,28,21,0,2,10,1)
          CALL DCUBE(II,21,21,0,10,4,1)
          CALL DCUBE(II,21,26,0,10,4,1)
        ELSEIF(N1.EQ.2) THEN
          CALL DCUBE(II,21,21,0,10,4,1)   ! SINGLE V APERTURE (2*6)
          CALL DCUBE(II,26,21,0,4,10,1)
          CALL DCUBE(II,21,21,0,10,2,1)
          CALL DCUBE(II,21,28,0,10,2,1)
       ENDIF

C**********************************
C   THIRD PANEL (X=0)             *
C**********************************
        II=21
        IF(N3.EQ.0) THEN
          CALL DCUBE(II,21,21,0,10,10,1)  ! NO APERTURE 
        ELSEIF(N3.EQ.1) THEN
          CALL DCUBE(II,21,21,0,2,10,1)   ! SINGLE H APERTURE (2*6)
          CALL DCUBE(II,28,21,0,2,10,1)
          CALL DCUBE(II,21,21,0,10,4,1)
          CALL DCUBE(II,21,26,0,10,4,1)
        ELSEIF(N3.EQ.2) THEN
          CALL DCUBE(II,21,21,0,10,4,1)   ! SINGLE V APERTURE (2*6)
          CALL DCUBE(II,26,21,0,4,10,1)
          CALL DCUBE(II,21,21,0,10,2,1)
          CALL DCUBE(II,21,28,0,10,2,1)
       ENDIF

C**********************************
C   SECOND PANEL (Y=B)            *
C**********************************
        JJ=31
        IF(N2.EQ.0) THEN
          CALL DCUBE(21,JJ,21,0,10,10,1)  ! NO APERTURE 
        ELSEIF(N2.EQ.1) THEN
          CALL DCUBE(21,JJ,21,0,2,10,1)   ! SINGLE H APERTURE (2*6)
          CALL DCUBE(28,JJ,21,0,2,10,1)
          CALL DCUBE(21,JJ,21,0,10,4,1)
          CALL DCUBE(21,JJ,26,0,10,4,1)
        ELSEIF(N2.EQ.2) THEN
          CALL DCUBE(21,JJ,21,0,10,4,1)   ! SINGLE V APERTURE (2*6)
          CALL DCUBE(26,JJ,21,0,4,10,1)
          CALL DCUBE(21,JJ,21,0,10,2,1)
          CALL DCUBE(21,JJ,28,0,10,2,1)
       ENDIF

C**********************************
C   FOURTH PANEL (Y=0)            *
C**********************************
        JJ=21
        IF(N4.EQ.0) THEN
          CALL DCUBE(21,JJ,21,0,10,10,1)  ! NO APERTURE 
        ELSEIF(N4.EQ.1) THEN
          CALL DCUBE(21,JJ,21,0,2,10,1)   ! SINGLE H APERTURE (2*6)
          CALL DCUBE(28,JJ,21,0,2,10,1)
          CALL DCUBE(21,JJ,21,0,10,4,1)
          CALL DCUBE(21,JJ,26,0,10,4,1)
        ELSEIF(N4.EQ.2) THEN
          CALL DCUBE(21,JJ,21,0,10,4,1)   ! SINGLE V APERTURE (2*6)
          CALL DCUBE(26,JJ,21,0,4,10,1)
          CALL DCUBE(21,JJ,21,0,10,2,1)
          CALL DCUBE(21,JJ,28,0,10,2,1)
       ENDIF

      RETURN
      END

C********************************************************
      SUBROUTINE DCUBE (ISTART,JSTART,KSTART,NXWIDE,NYWIDE,NZWIDE,MTYPE)
      INCLUDE 'FDSE.PAR'
C
      IMAX=ISTART+NXWIDE-1
      JMAX=JSTART+NYWIDE-1
      KMAX=KSTART+NZWIDE-1
C
      IF (NXWIDE.EQ.0) THEN
	DO 20 K=KSTART,KMAX
	  DO 10 J=JSTART,JMAX
	    IDTWO(ISTART,J,K)=MTYPE
	    IDTWO(ISTART,J,K+1)=MTYPE
	    IDTHRE(ISTART,J,K)=MTYPE
	    IDTHRE(ISTART,J+1,K)=MTYPE
 10       CONTINUE
 20     CONTINUE
      ELSEIF (NYWIDE.EQ.0) THEN
	DO 40 K=KSTART,KMAX
	  DO 30 I=ISTART,IMAX
	    IDONE(I,JSTART,K)=MTYPE
	    IDONE(I,JSTART,K+1)=MTYPE
	    IDTHRE(I,JSTART,K)=MTYPE
	    IDTHRE(I+1,JSTART,K)=MTYPE
 30       CONTINUE
 40     CONTINUE
      ELSEIF (NZWIDE.EQ.0) THEN
	DO 60 J=JSTART,JMAX
	  DO 50 I=ISTART,IMAX
	    IDONE(I,J,KSTART)=MTYPE
	    IDONE(I,J+1,KSTART)=MTYPE
	    IDTWO(I,J,KSTART)=MTYPE
	    IDTWO(I+1,J,KSTART)=MTYPE
 50       CONTINUE
 60     CONTINUE
      ELSE
	DO 90 K=KSTART,KMAX
	  DO 80 J=JSTART,JMAX
	    DO 70 I=ISTART,IMAX
	      IDONE(I,J,K)=MTYPE
	      IDONE(I,J,K+1)=MTYPE
	      IDONE(I,J+1,K+1)=MTYPE
	      IDONE(I,J+1,K)=MTYPE
	      IDTWO(I,J,K)=MTYPE
	      IDTWO(I+1,J,K)=MTYPE
	      IDTWO(I+1,J,K+1)=MTYPE
	      IDTWO(I,J,K+1)=MTYPE
	      IDTHRE(I,J,K)=MTYPE
	      IDTHRE(I+1,J,K)=MTYPE
	      IDTHRE(I+1,J+1,K)=MTYPE
	      IDTHRE(I,J+1,K)=MTYPE
 70         CONTINUE
 80       CONTINUE
 90     CONTINUE
      ENDIF
      RETURN
      END
C********************************************************
      SUBROUTINE PMLCUBE 
      INCLUDE 'FDSE.PAR'
C
        DO K=1,NZ
        DO J=1,NY
        DO I=1,NX
C
C         PML KOSELERI
C
         IF (I.LE.NXB.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=1
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=2
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=3
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=4
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=5
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=6
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=7
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=8
C
C         PML KENARLARI
C
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=9
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=10
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=11
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=12
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=13
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=14
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=15
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=16
C
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=17
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=18
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=19
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=20
C
C         PML YUZEYLERI (GENEL)
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.LE.NZB) THEN
          IDPML(I,J,K)=21
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.GE.NZS) THEN
          IDPML(I,J,K)=22
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=23
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=24
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=25
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=26
         ELSE
           IDPML(I,J,K)=27
         ENDIF
        ENDDO
        ENDDO
        ENDDO

      RETURN
      END

C*******************************************************
      SUBROUTINE SETUP
      INCLUDE 'FDSE.PAR'

      NSTOP(1)=600
      C=1.0/SQRT(XMU0*EPS0)

      DTXI=C/DELX
      DTYI=C/DELY
      DTZI=C/DELZ
      DT=DELX/(2.*C) !TLM ILE UYGUNLUK OLSUN DIYE

C     PARAMETER ALPHA IS THE DECAY RATE DETERMINED BY BETA.
       ALPHA=2.74E18*BAND**2
       SHIFT=4.0/SQRT(ALPHA)

C***********************************
C   DEFINE MEDIUM PARAMETERS       *
C***********************************
      DO I=1,9
	EPS(I)=EPS0
	SIGMA(I)=0.0
	EPSP(I)=EPS0
      END DO 
C
C     FREE SPACE CONSTANTS 
C
      DTEDX=DT/(EPS0*DELX)
      DTEDY=DT/(EPS0*DELY)
      DTEDZ=DT/(EPS0*DELZ)
      DTMDX=DT/(XMU0*DELX)
      DTMDY=DT/(XMU0*DELY)
      DTMDZ=DT/(XMU0*DELZ)
C
C     LOSSY DIELECTRICS
C
      DO 20 I=2,9
	ESB(I)=(2.*EPS(I)-SIGMA(I)*DT)/(2.*EPS(I)+SIGMA(I) *DT)
	ECRLX(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELX)
	ECRLY(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELY)
	ECRLZ(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELZ)
 20   CONTINUE
      
      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLSETUP
      INCLUDE 'FDSE.PAR'
       
       SIGMAXE(1)=ESIGMAM
       SIGMAXE(2)=ESIGMAM
       SIGMAXE(3)=ESIGMAM
       SIGMAXE(4)=ESIGMAM
       SIGMAXE(5)=ESIGMAM
       SIGMAXE(6)=ESIGMAM
       SIGMAXE(7)=ESIGMAM
       SIGMAXE(8)=ESIGMAM
       SIGMAXE(9)=ESIGMAM
       SIGMAXE(10)=ESIGMAM
       SIGMAXE(11)=ESIGMAM
       SIGMAXE(12)=ESIGMAM
C
       SIGMAXE(13)=0.
       SIGMAXE(14)=0.
       SIGMAXE(15)=0.
       SIGMAXE(16)=0.
C
       SIGMAXE(17)=ESIGMAM
       SIGMAXE(18)=ESIGMAM
       SIGMAXE(19)=ESIGMAM
       SIGMAXE(20)=ESIGMAM
C
       SIGMAXE(21)=0.
       SIGMAXE(22)=0.
       SIGMAXE(23)=ESIGMAM
       SIGMAXE(24)=ESIGMAM
       SIGMAXE(25)=0.
       SIGMAXE(26)=0.
       SIGMAXE(27)=0.
       
C
       SIGMAYE(1)=ESIGMAM
       SIGMAYE(2)=ESIGMAM
       SIGMAYE(3)=ESIGMAM
       SIGMAYE(4)=ESIGMAM
       SIGMAYE(5)=ESIGMAM
       SIGMAYE(6)=ESIGMAM
       SIGMAYE(7)=ESIGMAM
       SIGMAYE(8)=ESIGMAM
C            
       SIGMAYE(9)=0.
       SIGMAYE(10)=0.
       SIGMAYE(11)=0.
       SIGMAYE(12)=0.
C
       SIGMAYE(13)=ESIGMAM
       SIGMAYE(14)=ESIGMAM
       SIGMAYE(15)=ESIGMAM
       SIGMAYE(16)=ESIGMAM
C
       SIGMAYE(17)=ESIGMAM
       SIGMAYE(18)=ESIGMAM
       SIGMAYE(19)=ESIGMAM
       SIGMAYE(20)=ESIGMAM
C
       SIGMAYE(21)=0.
       SIGMAYE(22)=0.
       SIGMAYE(23)=0.
       SIGMAYE(24)=0.
       SIGMAYE(25)=ESIGMAM
       SIGMAYE(26)=ESIGMAM
       SIGMAYE(27)=0.
       
C
       SIGMAZE(1)=ESIGMAM
       SIGMAZE(2)=ESIGMAM
       SIGMAZE(3)=ESIGMAM
       SIGMAZE(4)=ESIGMAM
       SIGMAZE(5)=ESIGMAM
       SIGMAZE(6)=ESIGMAM
       SIGMAZE(7)=ESIGMAM
       SIGMAZE(8)=ESIGMAM
C            
       SIGMAZE(9)=ESIGMAM
       SIGMAZE(10)=ESIGMAM
       SIGMAZE(11)=ESIGMAM
       SIGMAZE(12)=ESIGMAM
C
       SIGMAZE(13)=ESIGMAM
       SIGMAZE(14)=ESIGMAM
       SIGMAZE(15)=ESIGMAM
       SIGMAZE(16)=ESIGMAM
C
       SIGMAZE(17)=0.
       SIGMAZE(18)=0.
       SIGMAZE(19)=0.
       SIGMAZE(20)=0.
C
       SIGMAZE(21)=ESIGMAM
       SIGMAZE(22)=ESIGMAM
       SIGMAZE(23)=0.
       SIGMAZE(24)=0.
       SIGMAZE(25)=0.
       SIGMAZE(26)=0.
       SIGMAZE(27)=0.
C
       SIGMAXH(1)=HSIGMAM
       SIGMAXH(2)=HSIGMAM
       SIGMAXH(3)=HSIGMAM
       SIGMAXH(4)=HSIGMAM
       SIGMAXH(5)=HSIGMAM
       SIGMAXH(6)=HSIGMAM
       SIGMAXH(7)=HSIGMAM
       SIGMAXH(8)=HSIGMAM
C
       SIGMAXH(9)=HSIGMAM
       SIGMAXH(10)=HSIGMAM
       SIGMAXH(11)=HSIGMAM
       SIGMAXH(12)=HSIGMAM
C
       SIGMAXH(13)=0.
       SIGMAXH(14)=0.
       SIGMAXH(15)=0.
       SIGMAXH(16)=0.
C
       SIGMAXH(17)=HSIGMAM
       SIGMAXH(18)=HSIGMAM
       SIGMAXH(19)=HSIGMAM
       SIGMAXH(20)=HSIGMAM
C
       SIGMAXH(21)=0.
       SIGMAXH(22)=0.
       SIGMAXH(23)=HSIGMAM
       SIGMAXH(24)=HSIGMAM
       SIGMAXH(25)=0.
       SIGMAXH(26)=0.
       SIGMAXH(27)=0.
C
       SIGMAYH(1)=HSIGMAM
       SIGMAYH(2)=HSIGMAM
       SIGMAYH(3)=HSIGMAM
       SIGMAYH(4)=HSIGMAM
       SIGMAYH(5)=HSIGMAM
       SIGMAYH(6)=HSIGMAM
       SIGMAYH(7)=HSIGMAM
       SIGMAYH(8)=HSIGMAM
C            
       SIGMAYH(9)=0.
       SIGMAYH(10)=0.
       SIGMAYH(11)=0.
       SIGMAYH(12)=0.
C      
       SIGMAYH(13)=HSIGMAM
       SIGMAYH(14)=HSIGMAM
       SIGMAYH(15)=HSIGMAM
       SIGMAYH(16)=HSIGMAM
C
       SIGMAYH(17)=HSIGMAM
       SIGMAYH(18)=HSIGMAM
       SIGMAYH(19)=HSIGMAM
       SIGMAYH(20)=HSIGMAM
C      
       SIGMAYH(21)=0.
       SIGMAYH(22)=0.
       SIGMAYH(23)=0.
       SIGMAYH(24)=0.
       SIGMAYH(25)=HSIGMAM
       SIGMAYH(26)=HSIGMAM
       SIGMAYH(27)=0.
C
       SIGMAZH(1)=HSIGMAM
       SIGMAZH(2)=HSIGMAM
       SIGMAZH(3)=HSIGMAM
       SIGMAZH(4)=HSIGMAM
       SIGMAZH(5)=HSIGMAM
       SIGMAZH(6)=HSIGMAM
       SIGMAZH(7)=HSIGMAM
       SIGMAZH(8)=HSIGMAM
C            
       SIGMAZH(9)=HSIGMAM
       SIGMAZH(10)=HSIGMAM
       SIGMAZH(11)=HSIGMAM
       SIGMAZH(12)=HSIGMAM
C      
       SIGMAZH(13)=HSIGMAM
       SIGMAZH(14)=HSIGMAM
       SIGMAZH(15)=HSIGMAM
       SIGMAZH(16)=HSIGMAM
C
       SIGMAZH(17)=0.
       SIGMAZH(18)=0.
       SIGMAZH(19)=0.
       SIGMAZH(20)=0.
C      
       SIGMAZH(21)=HSIGMAM
       SIGMAZH(22)=HSIGMAM
       SIGMAZH(23)=0.
       SIGMAZH(24)=0.
       SIGMAZH(25)=0.
       SIGMAZH(26)=0.
       SIGMAZH(27)=0.

      RETURN
      END

C     ********************************************************
      SUBROUTINE EXFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=1,NX1
          II=I
C
C         FDTD BOLGESI 
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS.
     $                              AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

C           DETERMINE MATERIAL TYPE
	    IF(IDONE(I,J,K).EQ.0) GO TO 100
	    IF(IDONE(I,J,K).EQ.1) GO TO 200
	    GO TO 300

C           FREE SPACE
 100        EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)-HZ(I,J-1,K))*DTEDY
     $                           -(HY(I,J,K)-HY(I,J,K-1))*DTEDZ
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EX(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZ(I,J-1,K))*ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HY(I,J,K-1))*ECRLZ(IDONE(I,J,K))

          ELSE
C
C         PML BOLGESI 
C
            CALL PMLEX(II,JJ,KK)
            
          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE

      IF(NSOURCE.EQ.1) THEN
         SOURCE=-2.3*SQRT(ALPHA)*(T-SHIFT)*EXP(-ALPHA*(T-SHIFT)**2)
         EX(NKX,NKY,NKZ)=EX(NKX,NKY,NKZ)+SOURCE/4.
         EX(NKX,NKY+1,NKZ)=EX(NKX,NKY+1,NKZ)+SOURCE/4.
         EX(NKX,NKY,NKZ+1)=EX(NKX,NKY,NKZ+1)+SOURCE/4.
         EX(NKX,NKY+1,NKZ+1)=EX(NKX,NKY+1,NKZ+1)+SOURCE/4.
         IF(T.LE.NSTOP(1)*DT) THEN
	      WRITE(44,*) T*1.E9, SOURCE
	   ENDIF
      END IF

      RETURN
      END

C     ********************************************************
      SUBROUTINE EYFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I	  
C
C         FDTD BOLGESI
C
          IF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS.
     $                              AND.K.GT.NZBP1.AND.K.LT.NZS) THEN


C           DETERMINE MATERIAL TYPE
	    IF(IDTWO(I,J,K).EQ.0) GO TO 100
	    IF(IDTWO(I,J,K).EQ.1) GO TO 200
	    GO TO 300

C           FREE SPACE
 100        EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)-HX(I,J,K-1))*DTEDZ
     $                         -(HZ(I,J,K)-HZ(I-1,J,K))*DTEDX
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EY(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HX(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZ(I-1,J,K))*ECRLX(IDTWO(I,J,K))

          ELSE
C
C         PML BOLGESI 
C
            CALL PMLEY(II,JJ,KK)
	  ENDIF
  
  10      CONTINUE
  20    CONTINUE
  30  CONTINUE

      IF(NSOURCE.EQ.2) THEN
         SOURCE=-2.3*SQRT(ALPHA)*(T-SHIFT)*EXP(-ALPHA*(T-SHIFT)**2)
         EY(NKX,NKY,NKZ)=EY(NKX,NKY,NKZ)+SOURCE/4.
         EY(NKX+1,NKY,NKZ)=EY(NKX+1,NKY+1,NKZ)+SOURCE/4.
         EY(NKX,NKY,NKZ+1)=EY(NKX,NKY,NKZ+1)+SOURCE/4.
         EY(NKX+1,NKY,NKZ+1)=EY(NKX+1,NKY,NKZ+1)+SOURCE/4.
         IF(T.LE.NSTOP(1)*DT) THEN
	     WRITE(44,*) T*1.E9, SOURCE
	   END IF
      END IF

      RETURN
      END

C     ********************************************************
      SUBROUTINE EZFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I
C
C         FDTD BOLGESI
C
          IF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS.
     $                              AND.K.GT.NZB.AND.K.LT.NZS) THEN


C           DETERMINE MATERIAL TYPE
	    IF(IDTHRE(I,J,K).EQ.0) GO TO 100
	    IF(IDTHRE(I,J,K).EQ.1) GO TO 200
	    GO TO 300
C
C           FREE SPACE
 100        EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)-HY(I-1,J,K))*DTEDX
     $                           -(HX(I,J,K)-HX(I,J-1,K))*DTEDY
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EZ(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HY(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HX(I,J-1,K))*ECRLY(IDTHRE(I,J,K))

          ELSE
C
C         PML BOLGESI 
C
            CALL PMLEZ(II,JJ,KK)

          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE

C ---- SOURCE EXCITATION -------

      IF(NSOURCE.EQ.3) THEN
         SOURCE=-2.3*SQRT(ALPHA)*(T-SHIFT)*EXP(-ALPHA*(T-SHIFT)**2)
         EZ(NKX,NKY,NKZ)=EZ(NKX,NKY,NKZ)+SOURCE/4.
         EZ(NKX,NKY+1,NKZ)=EZ(NKX,NKY+1,NKZ)+SOURCE/4.
         EZ(NKX+1,NKY,NKZ)=EZ(NKX+1,NKY,NKZ)+SOURCE/4.
         EZ(NKX+1,NKY+1,NKZ)=EZ(NKX+1,NKY+1,NKZ)+SOURCE/4.
         IF(T.LE.NSTOP(1)*DT) THEN
           WRITE(44,*) T*1.E9, SOURCE
	   END IF
      END IF

      RETURN
      END

C     *****************************************************************
      SUBROUTINE HXFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I
C
C         FDTD BOLGESI
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYSM1.
     $                              AND.K.GT.NZB.AND.K.LT.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)-(EZ(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                           +(EY(I,J,K+1)-EY(I,J,K))*DTMDZ
          ELSE
C
C         PML BOLGESI 
C
            CALL PMLHX(II,JJ,KK)

	  ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HYFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=1,NX1
            II=I
C
C         FDTD BOLGESI
C
          IF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS.
     $                              AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	    HY(I,J,K)=HY(I,J,K)-(EX(I,J,K+1)-EX(I,J,K))*DTMDZ
     $                           +(EZ(I+1,J,K)-EZ(I,J,K))*DTMDX
          ELSE
C
C         PML BOLGESI 
C
            CALL PMLHY(II,JJ,KK)

          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HZFLD
      INCLUDE 'FDSE.PAR'
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=1,NX1
            II=I
C
C         FDTD BOLGESI
C
          IF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYSM1.
     $                              AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    HZ(I,J,K)=HZ(I,J,K)-(EY(I+1,J,K)-EY(I,J,K))*DTMDX
     $                           +(EX(I,J+1,K)-EX(I,J,K))*DTMDY
          ELSE
C
C         PML BOLGESI 
C
            CALL PMLHZ(II,JJ,KK)

	  ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ***************************************************************
      SUBROUTINE ZERO
      INCLUDE 'FDSE.PAR'
C
      T=0.0
      DO 30 K=1,NZ
	DO 20 J=1,NY
	  DO 10 I=1,NX
	    EX(I,J,K)=0.0
	    EY(I,J,K)=0.0
	    EZ(I,J,K)=0.0
	    HX(I,J,K)=0.0
	    HY(I,J,K)=0.0
	    HZ(I,J,K)=0.0
	    EXY(I,J,K)=0.0
	    EYX(I,J,K)=0.0
	    EZX(I,J,K)=0.0
	    EXZ(I,J,K)=0.0
	    EYZ(I,J,K)=0.0
	    EZY(I,J,K)=0.0
	    HXY(I,J,K)=0.0
	    HYX(I,J,K)=0.0
	    HZX(I,J,K)=0.0
	    HXZ(I,J,K)=0.0
	    HYZ(I,J,K)=0.0
	    HZY(I,J,K)=0.0
	    IDONE(I,J,K)=0.0
	    IDTWO(I,J,K)=0.0
	    IDTHRE(I,J,K)=0.0
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE

      DO K=1,4
        NOX(K)=0
        NOY(K)=0
        NOZ(K)=0
        EAV(K)=0
      END DO

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLEX(I,J,K)
      INCLUDE 'FDSE.PAR'
      
      XJ=1*J
      XK=1*K
      EPSF=EPSP(IDONE(I,J,K)+1)/EPS0
      IF(J.LE.NYB) THEN
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XNPML-XJ+1.)/XNPML)**2.
      ELSE
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XJ-YN+XNPML)/XNPML)**2.
      ENDIF
      
      IF(K.LE.NZB) THEN
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XNPML-XK+1.)/XNPML)**2.
      ELSE
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XK-ZN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAY.LE.0.) THEN
       DTESY=1.
       DTESDY=DT/(DELY*EPSP(IDONE(I,J,K)+1))
      ELSE
       DTESY=EXP(-(ESIGMAY*DT)/EPSP(IDONE(I,J,K)+1))
       DTESDY=(1.-DTESY)/(ESIGMAY*DELY)
      ENDIF

      IF (ESIGMAZ.LE.0.) THEN
       DTESZ=1.
       DTESDZ=DT/(DELZ*EPSP(IDONE(I,J,K)+1))
      ELSE
       DTESZ=EXP(-(ESIGMAZ*DT)/EPSP(IDONE(I,J,K)+1))
       DTESDZ=(1.-DTESZ)/(ESIGMAZ*DELZ)
      ENDIF
C
C         PML INTERFACE
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

            IF (IDONE(I,J,K).EQ.0) THEN
	    
             EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)
     $               -HZX(I,J-1,K)-HZY(I,J-1,K))*DTEDY
     $               -(HY(I,J,K)-HY(I,J,K-1))*DTEDZ
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZX(I,J-1,K)-HZY(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HY(I,J,K-1))*ECRLZ(IDONE(I,J,K))

	    ENDIF
	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS
     $                                         .AND.K.EQ.NZBP1) THEN

	    IF (IDONE(I,J,K).EQ.0) THEN
	    
	     EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)-HZ(I,J-1,K))*DTEDY
     $               -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*DTEDZ
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZ(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*ECRLZ(IDONE(I,J,K))
	    ENDIF


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
                 
	   IF(IDONE(I,J,K).EQ.1) THEN
	    EXY(I,J,K)=0.0
	    EXZ(I,J,K)=0.0
	   ELSE
	    EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZ(I,J-1,K))*DTESDY
	    EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HYX(I,J,K-1)-HYZ(I,J,K-1))*DTESDZ
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                         .AND.K.EQ.NZS) THEN
	   IF(IDONE(I,J,K).EQ.1) THEN
	    EXY(I,J,K)=0.0
	    EXZ(I,J,K)=0.0

	   ELSE
	   
	    EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HY(I,J,K-1))*DTESDZ
	    EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZX(I,J-1,K)-HZY(I,J-1,K))*DTESDY
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.EQ.NZBP1) THEN
	    IF (IDONE(I,J,K).EQ.0) THEN
	    
	     EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)
     $     -HZX(I,J-1,K)-HZY(I,J-1,K))*DTEDY
     $    -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*DTEDZ
	    
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZX(I,J-1,K)-HZY(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*ECRLZ(IDONE(I,J,K))

	    ENDIF
C
C         PML
C
          ELSE
      
            IF(IDONE(I,J,K).EQ.1) THEN
              EXY(I,J,K)=0.0
              EXZ(I,J,K)=0.0

            ELSE
              EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $              -HZX(I,J-1,K)-HZY(I,J-1,K))*DTESDY
              EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $              -HYX(I,J,K-1)-HYZ(I,J,K-1))*DTESDZ
      
            ENDIF
C            WRITE(*,*) 'TAMAM'
          ENDIF
      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLEY(I,J,K)
      INCLUDE 'FDSE.PAR'
      
      XI=1*I
      XK=1*K

      EPSF=EPSP(IDTWO(I,J,K)+1)/EPS0

      IF(I.LE.NXB) THEN
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XNPML-XI+1.)/XNPML)**2.
      ELSE
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XI-XN+XNPML)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XNPML-XK+1.)/XNPML)**2.
      ELSE
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XK-ZN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAZ.LE.0.) THEN
       DTESZ=1.
       DTESDZ=DT/(DELZ*EPSP(IDTWO(I,J,K)+1))
      ELSE
       DTESZ=EXP(-(ESIGMAZ*DT)/EPSP(IDTWO(I,J,K)+1))
       DTESDZ=(1.-DTESZ)/(ESIGMAZ*DELZ)
      ENDIF

      IF (ESIGMAX.LE.0.) THEN
       DTESX=1.
       DTESDX=DT/(DELX*EPSP(IDTWO(I,J,K)+1))
      ELSE
       DTESX=EXP(-(ESIGMAX*DT)/EPSP(IDTWO(I,J,K)+1))
       DTESDX=(1.-DTESX)/(ESIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXBP1.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)-HX(I,J,K-1))*DTEDZ
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*DTEDX
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HX(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*ECRLX(IDTWO(I,J,K))
	    ENDIF


	  ELSEIF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZBP1) THEN


	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)
     $              -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTEDZ
     $            -(HZ(I,J,K)-HZ(I-1,J,K))*DTEDX
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HXY(I,J,K-1)-HXZ(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZ(I-1,J,K))*ECRLX(IDTWO(I,J,K))
	    ENDIF


	  ELSEIF (I.EQ.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   IF(IDTWO(I,J,K).EQ.1) THEN
	    EYX(I,J,K)=0.0
	    EYZ(I,J,K)=0.0

	   ELSE

	    EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZ(I-1,J,K))*DTESDX
	    EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTESDZ
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZS) THEN
	   
	   IF(IDTWO(I,J,K).EQ.1) THEN
	    EYX(I,J,K)=0.0
	    EYZ(I,J,K)=0.0

	   ELSE
	   
	    EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HX(I,J,K-1))*DTESDZ
	    EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZX(I-1,J,K)-HZY(I-1,J,K))*DTESDX
	   ENDIF

	  ELSEIF (I.EQ.NXBP1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZBP1) THEN

	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)
     $     -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTEDZ
     $     -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*DTEDX
	    
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HXY(I,J,K-1)-HXZ(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*ECRLX(IDTWO(I,J,K))
	    ENDIF
C
C         PML
C
          ELSE
          
            IF(IDTWO(I,J,K).EQ.1) THEN
              EYX(I,J,K)=0.0
              EYZ(I,J,K)=0.0

            ELSE

              EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $              -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTESDZ
              EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $              -HZX(I-1,J,K)-HZY(I-1,J,K))*DTESDX

           ENDIF
          ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLEZ(I,J,K)
      INCLUDE 'FDSE.PAR'

      XJ=1*J
      XI=1*I
     
      EPSF=EPSP(IDTHRE(I,J,K)+1)/EPS0
      IF(J.LE.NYB) THEN
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XNPML-XJ+1.)/XNPML)**2.
      ELSE
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XJ-YN+XNPML)/XNPML)**2.
      ENDIF

      IF(I.LE.NXB) THEN
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XNPML-XI+1.)/XNPML)**2.
      ELSE
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XI-XN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAY.LE.0.) THEN
       DTESY=1.
       DTESDY=DT/(DELY*EPSP(IDTHRE(I,J,K)+1))
      ELSE
       DTESY=EXP(-(ESIGMAY*DT)/EPSP(IDTHRE(I,J,K)+1))
       DTESDY=(1.-DTESY)/(ESIGMAY*DELY)
      ENDIF

      IF (ESIGMAX.LE.0.) THEN
       DTESX=1.
       DTESDX=DT/(DELX*EPSP(IDTHRE(I,J,K)+1))
      ELSE
       DTESX=EXP(-(ESIGMAX*DT)/EPSP(IDTHRE(I,J,K)+1))
       DTESDX=(1.-DTESX)/(ESIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
         IF (I.EQ.NXBP1.AND.J.GT.NYBP1.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    IF (IDTHRE(I,J,K).EQ.0) THEN
	     EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)-HYX(I-1,J,K)
     $               -HYZ(I-1,J,K))*DTEDX
     $               -(HX(I,J,K)-HX(I,J-1,K))*DTEDY
	    
	    ELSEIF (IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HX(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF


	  ELSEIF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    IF (IDTHRE(I,J,K).EQ.0) THEN
	     EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)
     $                -HY(I-1,J,K))*DTEDX
     $                -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))*DTEDY
	    
	    ELSEIF (IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HY(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF
	    
	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   IF(IDTHRE(I,J,K).EQ.1) THEN
	    EZX(I,J,K)=0.0
	    EZY(I,J,K)=0.0

	   ELSE
		    
	    EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HX(I,J-1,K))*DTESDY
	    EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $                -HYX(I-1,J,K)-HYZ(I-1,J,K))*DTESDX
	   ENDIF


	  ELSEIF (I.EQ.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   
	   IF(IDTHRE(I,J,K).EQ.1) THEN
	    EZX(I,J,K)=0.0
	    EZY(I,J,K)=0.0

	   ELSE

	    EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HY(I-1,J,K))*DTESDX
	    EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $                -HXY(I,J-1,K)-HXZ(I,J-1,K))*DTESDY
	   ENDIF

	  ELSEIF (I.EQ.NXBP1.AND.J.EQ.NYBP1
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   IF(IDTHRE(I,J,K).EQ.0) THEN
	    EZ(I,J,K)=EZ(I,J,K)+
     $     (HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*DTEDX
     $     -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))*DTEDY
	    
	    ELSEIF(IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF
C
C         PML
C
	   ELSE
            
            IF(IDTHRE(I,J,K).EQ.1) THEN
              EZX(I,J,K)=0.0
              EZY(I,J,K)=0.0

            ELSE
	  
              EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $              -HYX(I-1,J,K)-HYZ(I-1,J,K))*DTESDX
              EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $              -HXY(I,J-1,K)-HXZ(I,J-1,K))*DTESDY

            ENDIF
           ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHX(I,J,K)
      INCLUDE 'FDSE.PAR'
      
      XJ=1*J
      XK=1*K

      IF(J.LE.NYB) THEN
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XNPML-XJ+0.5)/XNPML)**2.
      ELSE
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XJ-YN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XNPML-XK+0.5)/XNPML)**2.
      ELSE
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XK-ZN+XNPML+0.5)/XNPML)**2.
      ENDIF

      IF (HSIGMAZ.LE.0.) THEN
       DTMSZ=1.
       DTMSDZ=DT/(DELZ*XMU0)
      ELSE
       DTMSZ=EXP(-(HSIGMAZ*DT)/XMU0)
       DTMSDZ=(1.-DTMSZ)/(HSIGMAZ*DELZ)
      ENDIF

      IF (HSIGMAY.LE.0.) THEN
       DTMSY=1.
       DTMSDY=DT/(DELY*XMU0)
      ELSE
       DTMSY=EXP(-(HSIGMAY*DT)/XMU0)
       DTMSDY=(1.-DTMSY)/(HSIGMAY*DELY)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	    HX(I,J,K)=HX(I,J,K)-(EZX(I,J+1,K)+EZY(I,J+1,K)
     $                -EZ(I,J,K))*DTMDY+(EY(I,J,K+1)-EY(I,J,K))*DTMDZ

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYSM1
     $                                          .AND.K.EQ.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)-(EZ(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                +(EYX(I,J,K+1)+EYZ(I,J,K+1)-EY(I,J,K))*DTMDZ

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HXY(I,J,K)=HXY(I,J,K)*DTMSY-(EZ(I,J+1,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDY
	   HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ+(EYX(I,J,K+1)+EYZ(I,J,K+1)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZB) THEN

           HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ+(EY(I,J,K+1)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ
	   HXY(I,J,K)=HXY(I,J,K)*DTMSY-(EZX(I,J+1,K)+EZY(I,J+1,K)
     $       -EZX(I,J,K)-EZY(I,J,K))*DTMSDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYSM1
     $                                          .AND.K.EQ.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)
     $       -(EZX(I,J+1,K)+EZY(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                +(EYX(I,J,K+1)+EYZ(I,J,K+1)-EY(I,J,K))*DTMDZ
C
C         PML
C
          ELSE

           HXY(I,J,K)=HXY(I,J,K)*DTMSY
     $              -(EZX(I,J+1,K)+EZY(I,J+1,K)
     $              -EZX(I,J,K)-EZY(I,J,K))*DTMSDY
           HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ
     $              +(EYX(I,J,K+1)+EYZ(I,J,K+1)
     $              -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ

          ENDIF
      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHY(I,J,K)
      INCLUDE 'FDSE.PAR'

      XI=1*I
      XK=1*K

      IF(I.LE.NXB) THEN
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XNPML-XI+0.5)/XNPML)**2.
      ELSE
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XI-XN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XNPML-XK+0.5)/XNPML)**2.
      ELSE
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XK-ZN+XNPML+0.5)/XNPML)**2.
      ENDIF
      

      IF (HSIGMAZ.LE.0.) THEN
       DTMSZ=1.
       DTMSDZ=DT/(DELZ*XMU0)
      ELSE
       DTMSZ=EXP(-(HSIGMAZ*DT)/XMU0)
       DTMSDZ=(1.-DTMSZ)/(HSIGMAZ*DELZ)
      ENDIF

      IF (HSIGMAX.LE.0.) THEN
       DTMSX=1.
       DTMSDX=DT/(DELX*XMU0)
      ELSE
       DTMSX=EXP(-(HSIGMAX*DT)/XMU0)
       DTMSDX=(1.-DTMSX)/(HSIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	   HY(I,J,K)=HY(I,J,K)-(EX(I,J,K+1)-EX(I,J,K))*DTMDZ
     $               +(EZX(I+1,J,K)+EZY(I+1,J,K)-EZ(I,J,K))*DTMDX

	  ELSEIF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZSM1) THEN

	   HY(I,J,K)=HY(I,J,K)-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $               -EX(I,J,K))*DTMDZ+(EZ(I+1,J,K)-EZ(I,J,K))*DTMDX
     
     	  ELSEIF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZSM1) THEN

	   HY(I,J,K)=HY(I,J,K)-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $              -EX(I,J,K))*DTMDZ
     $              +(EZX(I+1,J,K)+EZY(I+1,J,K)-EZ(I,J,K))*DTMDX


	  ELSEIF (I.EQ.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HYX(I,J,K)=HYX(I,J,K)*DTMSX+(EZ(I+1,J,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDX
	   HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZB) THEN
           HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ-(EX(I,J,K+1)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ
	   HYX(I,J,K)=HYX(I,J,K)*DTMSX+(EZX(I+1,J,K)+EZY(I+1,J,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDX

C
C         PML
C
	  ELSE

           HYX(I,J,K)=HYX(I,J,K)*DTMSX
     $              +(EZX(I+1,J,K)+EZY(I+1,J,K)
     $              -EZX(I,J,K)-EZY(I,J,K))*DTMSDX
           HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ
     $              -(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $              -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ

          ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHZ(I,J,K)
      INCLUDE 'FDSE.PAR'

      XJ=1*J
      XI=1*I

      IF(I.LE.NXB) THEN
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XNPML-XI+0.5)/XNPML)**2.
      ELSE
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XI-XN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(J.LE.NYB) THEN
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XNPML-XJ+0.5)/XNPML)**2.
      ELSE
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XJ-YN+XNPML+0.5)/XNPML)**2.
      ENDIF

      IF (HSIGMAY.LE.0.) THEN
       DTMSY=1.
       DTMSDY=DT/(DELY*XMU0)
      ELSE
       DTMSY=EXP(-(HSIGMAY*DT)/XMU0)
       DTMSDY=(1.-DTMSY)/(HSIGMAY*DELY)
      ENDIF

      IF (HSIGMAX.LE.0.) THEN
       DTMSX=1.
       DTMSDX=DT/(DELX*XMU0)
      ELSE
       DTMSX=EXP(-(HSIGMAX*DT)/XMU0)
       DTMSDX=(1.-DTMSX)/(HSIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $               -EY(I,J,K))*DTMDX+(EX(I,J+1,K)-EX(I,J,K))*DTMDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.EQ.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EY(I+1,J,K)-EY(I,J,K))*DTMDX
     $               +(EXY(I,J+1,K)+EXZ(I,J+1,K)-EX(I,J,K))*DTMDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HZY(I,J,K)=HZY(I,J,K)*DTMSY+(EX(I,J+1,K)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY
	   HZX(I,J,K)=HZX(I,J,K)*DTMSX-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX


	  ELSEIF (I.EQ.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZX(I,J,K)=HZX(I,J,K)*DTMSX-(EY(I+1,J,K)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX
	   HZY(I,J,K)=HZY(I,J,K)*DTMSY+(EXY(I,J+1,K)+EXZ(I,J+1,K)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY


	  ELSEIF (I.EQ.NXSM1.AND.J.EQ.NYSM1
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $               -EY(I,J,K))*DTMDX
     $               +(EXY(I,J+1,K)+EXZ(I,J+1,K)-EX(I,J,K))*DTMDY

C
C         PML
C
	  ELSE

           HZX(I,J,K)=HZX(I,J,K)*DTMSX
     $           -(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX
           HZY(I,J,K)=HZY(I,J,K)*DTMSY
     $           +(EXY(I,J+1,K)+EXZ(I,J+1,K)
     $           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY
          ENDIF

      RETURN
      END

C**********************   END **************************
