c*********************************************************************
C    PROGRAM        : TLMMS.FOR                                      *
C    AUTHOR         : M. ORHAN OZYALCIN                              *
C    DATE           : OCTOBER 2000                                   *
C    PURPOSE        : TLM SIMULATION OF A MICROSTRIP LINE FOR        *
C                     S PARAMETER CALCULATIONS                       *            
C                     REQUIRES SPART PACKAGE                         *
C*********************************************************************

      PROGRAM TLMPML
      INCLUDE 'TLMMS.PAR'

      OPEN(11,FILE='MSTRIP.INP')

      OPEN(30,FILE='MST-LINE.DAT') !only microstrip line

      OPEN(31,FILE='MST-STR.DAT') !microstrip circuit

      OPEN(32,FILE='MST-FIELD.DAT')

      OPEN(33,FILE='T-SOURCE.DAT')

      READ(11,*) NSEN !
      READ(11,*) H !STRIP HEIGHT [mm]
      READ(11,*) ND  ! NUMBER OF CELLS (ANGLE ONLY FOR SCENARIO 5)
      READ(11,*) ITRATE


      DELX=H*1.E-3/MSH
      DELY=DELX
      DELZ=DELX
      
	DEL=DELX

      DT=DEL/(2.*CISIK*HDB)
	DT1=1./DT   !!  DT in sec.

      BAND=1.E-9*CISIK/(20.*DELZ) !MIN. DALGABOYU 40 HUCRE ILE MODELLENIYOR
	
      ASBT=1./(DEL*ZO)

	REFTH=0.06
      GM=1.9
	ESIGMAM=-LOG(REFTH)*EPS0*CISIK*(GM-1.)/(2.*DEL*(GM**XNPML-1.)) 	
	HSIGMAM=ESIGMAM*XMU0/EPS0


C************************************
         CALL ZERO  
         CALL PMLCUBE
         CALL PMLSETUP
         CALL PARAMET  

C ---------MICOSTRIP LINE AND CIRCUIT LOOPS --------------------------
      DO NC=1,2

      IF (NC.EQ.1) THEN
       CALL BUILD(0) !MICROSTRIP LINE
       
      ELSE
       CALL ZERO
       CALL BUILD(NSEN) !MICROSTRIP CIRCUIT
       
      ENDIF


C*********************************
C  TIME ITERATION STATRTS        *
C*********************************
       DO 60 ITIME = 1,ITRATE
       TT=REAL(ITIME)*DT

          DO  K = 1,NZE
            DO  J = 1,NYE
              DO  I = 1,NXE

              CALL EXCITE(K,J,I)
                  
             V1 = V(K,J,I,1)
             V2 = V(K,J,I,2)
             V3 = V(K,J,I,3)
             V4 = V(K,J,I,4)
             V5 = V(K,J,I,5)
             V6 = V(K,J,I,6)
             V7 = V(K,J,I,7)
             V8 = V(K,J,I,8)
             V9 = V(K,J,I,9)
             V10 = V(K,J,I,10)
             V11 = V(K,J,I,11)
             V12 = V(K,J,I,12)

C*** Vr = S * Vi CALCULATE REFLECTED VOLTAGE PULSE HESAPLA ***
       IF(IDPML(K,J,I).EQ.27) THEN
        IF(IDCELL(K,J,I) .EQ. 1) THEN
             CALL SCAT_PEC(K,J,I)
        ELSE
             CALL SCAT(K,J,I)
        END IF
       ELSE
            CALL SCAT_PGM(K,J,I)
       ENDIF
       END DO
       END DO
       END DO

C*** USE BOUNDARY CONDITIONS AND CONNECT NODES ***
        CALL CONNECTION

       IF(NC.EQ.1) CALL OUTPUT(32)

       WRITE(33,*) TT*1E9,EXSOUR  !TIME VARIATION OF THE SOURCE

       CALL OBSERVE(NOZ1,GV1)
       CALL OBSERVE(NOZ2,GV2)

       IF(NC.EQ.1) WRITE (30,*) TT*1E9,GV1,GV2
       IF(NC.EQ.2) WRITE (31,*) TT*1E9,GV1,GV2


        WRITE(*,*) ITIME


C******************************
C   END OF TIME LOOP          *
C******************************

 60   CONTINUE

      ENDDO
      STOP
      END

C**************************************************

        SUBROUTINE ZERO
        INCLUDE 'TLMMS.PAR'

       DO 11 K = 1,NZE
       DO 11 J = 1,NYE
       DO 11 I = 1,NXE
       DO 11 L = 1,NT2      
           V(K,J,I,L)=0.0
11     CONTINUE

      DO L = 0,9
          ER(L) = 1.0
          SIGMA(L) = 0.0
          YNR(L) = 4.E-7     
          GNR(L) = 0.  
          RV(L) = 0.5/HDB 
          RI(L) = 0.5/HDB
      END DO
         
C****************************
       DO 15 K = 1,NZE
       DO 15 J = 1,NYE
       DO 15 I = 1,NXE
          IDCELL(K,J,I) = 0  
15     CONTINUE

      RETURN
      END

C******************************************************

        SUBROUTINE BUILD(NSEN)
        INCLUDE 'TLMMS.PAR'

C***************************************************
         ER(2)=2.2
C***************************************************

        DO I = 2,9
           YNR(I) = 4.*(ER(I)-1.)
           GNR(I) = SIGMA(I)*DEL*ZO
           RV(I) = 2./(4.+YNR(I)+GNR(I))
        END DO

c  **** DIELECTRIC ****

         CALL DCUBE (1,1,1,NZE,MSH,NXE,2)

      IF(NSEN.EQ.0) THEN

C **** TOP CONDUCTOR STRIP ****       
        CALL DCUBE (1,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5) !ZERO THICKNESS PEC

C--------- SCENARIO - 1 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.1) THEN
        CALL DCUBE (1,MSH,MSXB,NZE/2-ND/2,MSH,MSXB+MSW-1,5)
        CALL DCUBE (NZE/2+ND/2,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)

C--------- SCENARIO - 2 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.2) THEN
      CALL DCUBE (1,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)
      CALL DCUBE (NZE/2-ND/2,MSH,MSXB-2-MSW,NZE/2+ND/2-1,MSH,MSXB-3,5)
      CALL DCUBE (NZE/2-ND/2,MSH,MSXB+MSW+2,NZE/2+ND/2-1,MSH,
     >                           MSXB+2*MSW+1,5)

C--------- SCENARIO - 3 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.3) THEN
      CALL DCUBE (1,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)
      CALL DCUBE (NZE/2-ND/2,MSH,MSXB-MSW,NZE/2+ND/2-1,MSH,
     >                           MSXB+2*MSW-1,5)

C--------- SCENARIO - 4 ( 1 < ND < 21) --------------
      ELSEIF (NSEN.EQ.4) THEN
      CALL DCUBE (1,MSH,MSXB,NZE/2-ND/2-2,MSH,MSXB+MSW-1,5)
      CALL DCUBE (NZE/2+ND/2+2,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)
      CALL DCUBE (NZE/2-ND/2,MSH,MSXB-MSW,NZE/2+ND/2-1,MSH,
     >                           MSXB+2*MSW-1,5)

C--------- SCENARIO - 5 ( 0 <= ND <= 90) --------------
      ELSEIF (NSEN.EQ.5) THEN

C      HALF RADIAL ANGLE  0 <= ND <= 90
       
       SR = 1.5*MSW*DELX !RADIAL LENGTH
       RTETA = ND*PI/180.

C **** TOP CONDUCTOR STRIP ****
      CALL DCUBE (1,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)

C **** BUTTERFLY 
      IM=NXE/2
      KM=NZE/2

      KRD=SR*SIN(RTETA)/DELZ
      IRD=SR*COS(RTETA)/DELX
         
      DO K=KM-KRD,KM+KRD
         DO I=IM-IRD,IM+IRD
          XR=(I-IM)*DELX
          ZR=(K-KM)*DELZ

C          DIST=SQRT(XR**2+ZR**2)

          TETA=ABS(ATAN(ZR/XR))

C          IF(DIST.LE.SR.AND.TETA.LE.RTETA) THEN

          IF(TETA.LE.RTETA) THEN
             CALL DCUBE (K,MSH,I,K,MSH,I,5)
          ENDIF
        END DO
      END DO

C--------- SCENARIO - 6 ( 0 <= ND <= 180) --------------
      ELSEIF (NSEN.EQ.6) THEN
      CALL DCUBE (1,MSH,MSXB,NZE,MSH,MSXB+MSW-1,5)
      CALL DCUBE (NZE/2-3*MSW/2,MSH,MSXB-MSW,NZE/2-MSW/2-1,MSH,
     >                           MSXB+2*MSW-1,5)
      CALL DCUBE (NZE/2+MSW/2,MSH,MSXB-MSW,NZE/2+3*MSW/2-1,MSH,
     >                           MSXB+2*MSW-1,5)

       ENDIF

        RETURN
        END

C***********************************************

        SUBROUTINE PARAMET
        INCLUDE 'TLMMS.PAR'

       ALPHA = 3.3E18*BAND*BAND     ! BAND in GHz
       SHIFT= 4.0/SQRT(ALPHA)
C       AMP2 = -2.3*AMP*SQRT(ALPHA)

      RETURN
      END

C*************************************************************

         SUBROUTINE DCUBE(KSTRT,JSTRT,ISTRT,KEND,JEND,IEND,MTYPE)
         INCLUDE 'TLMMS.PAR'

        DO K = KSTRT,KEND
          DO J = JSTRT,JEND
            DO I = ISTRT,IEND
               IDCELL(K,J,I) = MTYPE !
            ENDDO
          ENDDO
        ENDDO

        RETURN
        END

C*************************************************************

        SUBROUTINE SCAT(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'
	

             V13 = V(KS,JS,IS,13)
             V14 = V(KS,JS,IS,14)
             V15 = V(KS,JS,IS,15)
             V16 = V(KS,JS,IS,16)
             V17 = V(KS,JS,IS,17)
             V18 = V(KS,JS,IS,18)


      VX=RV(IDCELL(KS,JS,IS))*(V1+V2+V9+V12+YNR(IDCELL(KS,JS,IS))*V13)
      VY=RV(IDCELL(KS,JS,IS))*(V3+V4+V8+V11+YNR(IDCELL(KS,JS,IS))*V14)
      VZ=RV(IDCELL(KS,JS,IS))*(V5+V6+V7+V10+YNR(IDCELL(KS,JS,IS))*V15)
      RIX=RI(IDCELL(KS,JS,IS))*(V5-V7+V8-V4)
      RIY=RI(IDCELL(KS,JS,IS))*(V2-V9+V10-V6)
      RIZ=RI(IDCELL(KS,JS,IS))*(V3-V11+V12-V1)
 
       V(KS,JS,IS,1) = VX + RIZ - V12
       V(KS,JS,IS,2) = VX - RIY - V9 
       V(KS,JS,IS,3) = VY - RIZ - V11
       V(KS,JS,IS,4) = VY + RIX - V8 
       V(KS,JS,IS,5) = VZ - RIX - V7 
       V(KS,JS,IS,6) = VZ + RIY - V10
       V(KS,JS,IS,7) = VZ + RIX - V5 
       V(KS,JS,IS,8) = VY - RIX - V4 
       V(KS,JS,IS,9) = VX + RIY - V2 
       V(KS,JS,IS,10)= VZ - RIY - V6 
       V(KS,JS,IS,11)= VY + RIZ - V3 
       V(KS,JS,IS,12)= VX - RIZ - V1 

       V(KS,JS,IS,13)= VX - V13
       V(KS,JS,IS,14)= VY - V14
       V(KS,JS,IS,15)= VZ - V15

        RETURN
        END

c*************************************
        SUBROUTINE SCAT_PGM(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'

             V13 = V(KS,JS,IS,13)
             V14 = V(KS,JS,IS,14)
             V15 = V(KS,JS,IS,15)
             V16 = V(KS,JS,IS,16)
             V17 = V(KS,JS,IS,17)
             V18 = V(KS,JS,IS,18)

       V19 = V(KS,JS,IS,19) ! SHORT CIRCUIT STUB
       V20 = V(KS,JS,IS,20) ! SHORT CIRCUIT STUB
       V21 = V(KS,JS,IS,21) ! SHORT CIRCUIT STUB
       V22 = V(KS,JS,IS,22) ! SHORT CIRCUIT STUB
       V23 = V(KS,JS,IS,23) ! SHORT CIRCUIT STUB
       V24 = V(KS,JS,IS,24) ! SHORT CIRCUIT STUB

      XI=1.*IS
      XJ=1.*JS
      XK=1.*KS                 
      
	IF(IS.LE.NXB) THEN
      L=1.*NXB-XI
      ESIGMAX=SIGMAXE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ELSE
      L=XI-1.*NXS
      ESIGMAX=SIGMAXE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ENDIF

      IF(JS.LE.NYB) THEN
      L=1.*NYB-XJ
      ESIGMAY=SIGMAYE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ELSEIF(JS.GE.NYS) THEN !!
C      ELSE !!
      L=XJ-1.*NYS
      ESIGMAY=SIGMAYE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ELSE
      ESIGMAY=0. !!!
      ENDIF
      
      IF(KS.LE.NZB) THEN
      L=1.*NZB-XK
      ESIGMAZ=SIGMAZE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ELSE
      L=XK-1.*NZS
      ESIGMAZ=SIGMAZE(IDPML(KS,JS,IS))*ER(IDCELL(KS,JS,IS))*GM**(L)
      ENDIF

      IF(IS.LE.NXB) THEN
        L=1.*NXB-XI
        HSIGMAX=SIGMAXH(IDPML(KS,JS,IS))*GM**(L)
      ELSE
        L=XI-1.*NXS
        HSIGMAX=SIGMAXH(IDPML(KS,JS,IS))*GM**(L)
      ENDIF

      IF(JS.LE.NYB) THEN
        L=1.*NYB-XJ
        HSIGMAY=SIGMAYH(IDPML(KS,JS,IS))*GM**(L)
      ELSEIF(JS.GE.NYS) THEN !!
C      ELSE !!
        L=XJ-1.*NYS
        HSIGMAY=SIGMAYH(IDPML(KS,JS,IS))*GM**(L)
      ELSE !!
        HSIGMAY=0. !!
      ENDIF

      IF(KS.LE.NZB) THEN
        L=1.*NZB-XK
        HSIGMAZ=SIGMAZH(IDPML(KS,JS,IS))*GM**(L)
      ELSE
        L=XK-1.*NZS
        HSIGMAZ=SIGMAZH(IDPML(KS,JS,IS))*GM**(L)
      ENDIF

C*******************************************
        GXY = ESIGMAY*DEL*ZO
        GXZ = ESIGMAZ*DEL*ZO
        GYX = ESIGMAX*DEL*ZO
        GYZ = ESIGMAZ*DEL*ZO
        GZX = ESIGMAX*DEL*ZO
        GZY = ESIGMAY*DEL*ZO

        RXY = HSIGMAY*DEL/ZO
        RXZ = HSIGMAZ*DEL/ZO
        RYX = HSIGMAX*DEL/ZO
        RYZ = HSIGMAZ*DEL/ZO
        RZX = HSIGMAX*DEL/ZO
        RZY = HSIGMAY*DEL/ZO

        YX = 2.*ER(IDCELL(KS,JS,IS))*DEL/(CISIK*DT)-4.
        YY = 2.*ER(IDCELL(KS,JS,IS))*DEL/(CISIK*DT)-4.
        YZ = 2.*ER(IDCELL(KS,JS,IS))*DEL/(CISIK*DT)-4.

        ZX = 2.*MUPMLR*DEL/(CISIK*DT)-4.
        ZY = 2.*MUPMLR*DEL/(CISIK*DT)-4.
        ZZ = 2.*MUPMLR*DEL/(CISIK*DT)-4.

        RVXY = 2./(4.+YX+GXY)
        RVXZ = 2./(4.+YX+GXZ)
        RVYX = 2./(4.+YY+GYX)
        RVYZ = 2./(4.+YY+GYZ)
        RVZX = 2./(4.+YZ+GZX)
        RVZY = 2./(4.+YZ+GZY)

        RIXY = 2./(4.+ZX+RXY)
        RIXZ = 2./(4.+ZX+RXZ)
        RIYX = 2./(4.+ZY+RYX)
        RIYZ = 2./(4.+ZY+RYZ)
        RIZX = 2./(4.+ZZ+RZX)
        RIZY = 2./(4.+ZZ+RZY)

      VXY=RVXY*(V1+V12+(YX+2.)*V13-2.*V14)
      VXZ=RVXZ*(V2+V9+(YX+2.)*V14-2.*V13)
      VYX=RVYX*(V3+V11+(YY+2.)*V15-2.*V16)
      VYZ=RVYZ*(V4+V8+(YY+2.)*V16-2.*V15)
      VZX=RVZX*(V6+V10+(YZ+2.)*V17-2.*V18)
      VZY=RVZY*(V5+V7+(YZ+2.)*V18-2.*V17)
      AXY=RIXY*(V5-V7+(1.+2./ZX)*V19-(2./ZX)*V20)
      AXZ=RIXZ*(V8-V4+(1.+2./ZX)*V20-(2./ZX)*V19)
      AYX=RIYX*(V10-V6+(1.+2./ZY)*V21-(2./ZY)*V22)
      AYZ=RIYZ*(V2-V9+(1.+2./ZY)*V22-(2./ZY)*V21)
      AZX=RIZX*(V3-V11+(1.+2./ZZ)*V23-(2./ZZ)*V24)
      AZY=RIZY*(V12-V1+(1.+2./ZZ)*V24-(2./ZZ)*V23) 

       V(KS,JS,IS,1) = VXY + VXZ + AZX + AZY - V12
       V(KS,JS,IS,2) = VXY + VXZ - (AYX + AYZ) - V9 
       V(KS,JS,IS,3) = VYX + VYZ - (AZX + AZY) - V11
       V(KS,JS,IS,4) = VYX + VYZ + AXY + AXZ - V8 
       V(KS,JS,IS,5) = VZX + VZY - (AXY + AXZ) - V7 
       V(KS,JS,IS,6) = VZX + VZY + AYX + AYZ - V10
       V(KS,JS,IS,7) = VZX + VZY + AXY + AXZ - V5 
       V(KS,JS,IS,8) = VYX + VYZ - (AXY + AXZ) - V4 
       V(KS,JS,IS,9) = VXY + VXZ + AYX + AYZ - V2 
       V(KS,JS,IS,10)= VZX + VZY - (AYX + AYZ) - V6 
       V(KS,JS,IS,11)= VYX + VYZ + AZX + AZY - V3 
       V(KS,JS,IS,12)= VXY + VXZ - (AZX + AZY) - V1 

       V(KS,JS,IS,13)= VXY - V13
       V(KS,JS,IS,14)= VXZ - V14
       V(KS,JS,IS,15)= VYX - V15
       V(KS,JS,IS,16)= VYZ - V16
       V(KS,JS,IS,17)= VZX - V17
       V(KS,JS,IS,18)= VZY - V18

       V(KS,JS,IS,19)= -V19 +ZX*AXY
       V(KS,JS,IS,20)= -V20 +ZX*AXZ
       V(KS,JS,IS,21)= -V21 +ZY*AYX
       V(KS,JS,IS,22)= -V22 +ZY*AYZ
       V(KS,JS,IS,23)= -V23 +ZZ*AZX
       V(KS,JS,IS,24)= -V24 +ZZ*AZY
        RETURN
        END

C******************************************************

        SUBROUTINE SCAT_PEC(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'
             V(KS,JS,IS,1) = -V1
             V(KS,JS,IS,2) = -V2
             V(KS,JS,IS,3) = -V3
             V(KS,JS,IS,4) = -V4
             V(KS,JS,IS,5) = -V5
             V(KS,JS,IS,6) = -V6
             V(KS,JS,IS,7) = -V7
             V(KS,JS,IS,8) = -V8
             V(KS,JS,IS,9) = -V9
             V(KS,JS,IS,10) = -V10
             V(KS,JS,IS,11) = -V11
             V(KS,JS,IS,12) = -V12
        RETURN
        END

C*******************************************************

        SUBROUTINE CONNECTION
        INCLUDE 'TLMMS.PAR'

       DO KS = 1,NZE
          K1=KS+1
       DO JS = 1,NYE
          J1=JS+1
       DO IS = 1,NXE
          I1=IS+1

       IF(IS.LT.NXE) THEN
         V0 = V(KS,JS,IS,10)
         V(KS,JS,IS,10) = V(KS,JS,I1,6)
         V(KS,JS,I1,6) = V0
         V0 = V(KS,JS,IS,11)
         V(KS,JS,IS,11) = V(KS,JS,I1,3)
         V(KS,JS,I1,3) = V0
       ENDIF

       IF(JS.LT.NYE .AND. IDCELL(KS,JS,IS).NE.5) THEN
         V0 = V(KS,JS,IS,7)
         V(KS,JS,IS,7) = V(KS,J1,IS,5)
         V(KS,J1,IS,5) = V0
         V0 = V(KS,JS,IS,12)
         V(KS,JS,IS,12) = V(KS,J1,IS,1)
         V(KS,J1,IS,1) = V0
       ENDIF

       IF(KS.LT.NZE) THEN
         V0 = V(KS,JS,IS,8)
         V(KS,JS,IS,8) = V(K1,JS,IS,4)
         V(K1,JS,IS,4) = V0
         V0 = V(KS,JS,IS,9)
         V(KS,JS,IS,9) = V(K1,JS,IS,2)
         V(K1,JS,IS,2) = V0
       END IF

       CALL BOUN_MS(KS,JS,IS)

        END DO
        END DO
        END DO

        RETURN
        END

C***************************************************

        SUBROUTINE EXCITE(K,J,I)
        INCLUDE 'TLMMS.PAR'

       IF(I.GE.MSXB.AND.I.LT.(MSXB+MSW)
     $             .AND.J.LE.MSH.AND.K.EQ.NKZ) THEN

       AMP3 = -(0.5+GNR(IDCELL(K,J,I))/8.)
       SOURCE=AMP*EXP(-ALPHA*(TT-SHIFT)**2)
                     
       V(K,J,I,3)=V(K,J,I,3)+AMP3*SOURCE
       V(K,J,I,4)=V(K,J,I,4)+AMP3*SOURCE
       V(K,J,I,8)=V(K,J,I,8)+AMP3*SOURCE
       V(K,J,I,11)=V(K,J,I,11)+AMP3*SOURCE

       IF(IDCELL(K,J,I).NE.0) V(K,J,I,14)=V(K,J,I,14)-0.5*SOURCE
       ENDIF

       EXSOUR=AMP*EXP(-ALPHA*(TT-SHIFT)**2)

        RETURN
        END


C**********************************************

        SUBROUTINE OUTPUT(MDSY)
        INCLUDE 'TLMMS.PAR'
        I=NXE/2
        J= MSH-2

         K=KL1
         EYSG1 =-RV(IDCELL(K,J,I))*(V(K,J,I,3)+V(K,J,I,4)+
     >   V(K,J,I,8)+V(K,J,I,11)+YNR(IDCELL(K,J,I))*V(K,J,I,14))

         K=KL2

         EYSG2 =-RV(IDCELL(K,J,I))*(V(K,J,I,3)+V(K,J,I,4)+
     >   V(K,J,I,8)+V(K,J,I,11)+YNR(IDCELL(K,J,I))*V(K,J,I,14))

         K=KL3

         EYSG3 =-RV(IDCELL(K,J,I))*(V(K,J,I,3)+V(K,J,I,4)+
     >   V(K,J,I,8)+V(K,J,I,11)+YNR(IDCELL(K,J,I))*V(K,J,I,14))

         K=KL4

         EYSG4 =-RV(IDCELL(K,J,I))*(V(K,J,I,3)+V(K,J,I,4)+
     >   V(K,J,I,8)+V(K,J,I,11)+YNR(IDCELL(K,J,I))*V(K,J,I,14))

         WRITE(MDSY,205) TT*1.E9,EYSG1,EYSG2,EYSG3,EYSG4
  205    FORMAT(F10.5,1X,4(F14.10,1X))


        RETURN
        END

C ****************************************************

        SUBROUTINE BOUN_MS(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'

      IF (IS.EQ.1) THEN
            V(KS,JS,IS,3) = 0.
            V(KS,JS,IS,6) = 0.
	END IF

      IF(IS.EQ.NXE) THEN
            V(KS,JS,IS,10) = 0.
            V(KS,JS,IS,11) = 0.
	END IF
	     
      IF (JS.EQ.1) THEN
             V(KS,JS,IS,1) = -V(KS,JS,IS,1)
             V(KS,JS,IS,5) = -V(KS,JS,IS,5)
	END IF

      IF(JS.EQ.NYE) THEN
            V(KS,JS,IS,7)  = 0.
            V(KS,JS,IS,12) = 0.
	END IF

      IF (IDCELL(KS,JS,IS).EQ.5) THEN
            V(KS,JS,IS,7) = -V(KS,JS,IS,7)  
            V(KS,JS,IS,12) = -V(KS,JS,IS,12)
            V(KS,JS+1,IS,1) = -V(KS,JS+1,IS,1)
            V(KS,JS+1,IS,5) = -V(KS,JS+1,IS,5)
      END IF
	
	IF(KS.EQ.1) THEN
            V(KS,JS,IS,2) = 0. 
            V(KS,JS,IS,4) = 0.  ! PEC
	END IF

      IF (KS.EQ.NZE) THEN
            V(KS,JS,IS,8) = 0.  
            V(KS,JS,IS,9) = 0.  ! PEC
	END IF


        RETURN
        END

C***************************************

        SUBROUTINE BOUN_PEC(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'

        IF (IS.EQ.1) THEN
            V(KS,JS,IS,3) = -V(KS,JS,IS,3) 
            V(KS,JS,IS,6) = -V(KS,JS,IS,6) 
	END IF

        IF(IS.EQ.NXE) THEN
            V(KS,JS,IS,10) = -V(KS,JS,IS,10)  
            V(KS,JS,IS,11) = -V(KS,JS,IS,11)  
	END IF
	     
        IF (JS.EQ.1) THEN
             V(KS,JS,IS,1) = -V(KS,JS,IS,1)
             V(KS,JS,IS,5) = -V(KS,JS,IS,5)
	END IF

        IF(JS.EQ.NYE) THEN
            V(KS,JS,IS,7)  = -V(KS,JS,IS,7)       
            V(KS,JS,IS,12) = -V(KS,JS,IS,12)      
	END IF

        IF(KS.EQ.1) THEN
            V(KS,JS,IS,2) = -V(KS,JS,IS,2)  
            V(KS,JS,IS,4) = -V(KS,JS,IS,4)  
	END IF

        IF (KS.EQ.NZE) THEN
            V(KS,JS,IS,8) = -V(KS,JS,IS,8)  
            V(KS,JS,IS,9) = -V(KS,JS,IS,9)  
	END IF

        RETURN
        END

C***************************************

        SUBROUTINE BOUN_ZERO(KS,JS,IS)
        INCLUDE 'TLMMS.PAR'

	IF (IS.EQ.1) THEN
            V(KS,JS,IS,3) = 0. 
            V(KS,JS,IS,6) = 0.
	END IF

	IF(IS.EQ.NXE) THEN
            V(KS,JS,IS,10) = 0.
            V(KS,JS,IS,11) = 0.
	END IF
	     
	IF (JS.EQ.1) THEN
             V(KS,JS,IS,1) = 0.
             V(KS,JS,IS,5) = 0.
	END IF

	IF(JS.EQ.NYE) THEN
            V(KS,JS,IS,7)  = 0.
            V(KS,JS,IS,12) = 0.
	END IF

	IF(KS.EQ.1) THEN
            V(KS,JS,IS,2) = 0.
            V(KS,JS,IS,4) = 0.
	END IF

	IF (KS.EQ.NZE) THEN
            V(KS,JS,IS,8) = 0.
            V(KS,JS,IS,9) = 0.
	END IF

        RETURN
        END

C********************************************************
      SUBROUTINE PMLCUBE 
      INCLUDE 'TLMMS.PAR'
C
        DO K=1,NZE
        DO J=1,NYE
        DO I=1,NXE
C
C         PML CORNERS
C
         IF (I.LE.NXB.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=1
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=2
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=3
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=4
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=5
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=6
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=7
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=8
C
C         PML EDGES
C
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=9
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=10
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=11
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=12
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=13
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=14
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(K,J,I)=15
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(K,J,I)=16
C
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(K,J,I)=17
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(K,J,I)=18
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(K,J,I)=19
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(K,J,I)=20
C
C         PML SURFACES
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.LE.NZB) THEN
          IDPML(K,J,I)=21
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.GE.NZS) THEN
          IDPML(K,J,I)=22
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(K,J,I)=23
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(K,J,I)=24
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(K,J,I)=25
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(K,J,I)=26
         ELSE
           IDPML(K,J,I)=27
         ENDIF
        ENDDO
        ENDDO
        ENDDO

      RETURN
      END

C*******************************************************
      SUBROUTINE PMLSETUP
      INCLUDE 'TLMMS.PAR'
       
       SIGMAXE(1)=ESIGMAM
       SIGMAXE(2)=ESIGMAM
       SIGMAXE(3)=ESIGMAM
       SIGMAXE(4)=ESIGMAM
       SIGMAXE(5)=ESIGMAM
       SIGMAXE(6)=ESIGMAM
       SIGMAXE(7)=ESIGMAM
       SIGMAXE(8)=ESIGMAM
       SIGMAXE(9)=ESIGMAM
       SIGMAXE(10)=ESIGMAM
       SIGMAXE(11)=ESIGMAM
       SIGMAXE(12)=ESIGMAM
C
       SIGMAXE(13)=0.
       SIGMAXE(14)=0.
       SIGMAXE(15)=0.
       SIGMAXE(16)=0.
C
       SIGMAXE(17)=ESIGMAM
       SIGMAXE(18)=ESIGMAM
       SIGMAXE(19)=ESIGMAM
       SIGMAXE(20)=ESIGMAM
C
       SIGMAXE(21)=0.
       SIGMAXE(22)=0.
       SIGMAXE(23)=ESIGMAM
       SIGMAXE(24)=ESIGMAM
       SIGMAXE(25)=0.
       SIGMAXE(26)=0.
       SIGMAXE(27)=0.
       
C
       SIGMAYE(1)=ESIGMAM
       SIGMAYE(2)=ESIGMAM
       SIGMAYE(3)=ESIGMAM
       SIGMAYE(4)=ESIGMAM
       SIGMAYE(5)=ESIGMAM
       SIGMAYE(6)=ESIGMAM
       SIGMAYE(7)=ESIGMAM
       SIGMAYE(8)=ESIGMAM
C            
       SIGMAYE(9)=0.
       SIGMAYE(10)=0.
       SIGMAYE(11)=0.
       SIGMAYE(12)=0.
C
       SIGMAYE(13)=ESIGMAM
       SIGMAYE(14)=ESIGMAM
       SIGMAYE(15)=ESIGMAM
       SIGMAYE(16)=ESIGMAM
C
       SIGMAYE(17)=ESIGMAM
       SIGMAYE(18)=ESIGMAM
       SIGMAYE(19)=ESIGMAM
       SIGMAYE(20)=ESIGMAM
C
       SIGMAYE(21)=0.
       SIGMAYE(22)=0.
       SIGMAYE(23)=0.
       SIGMAYE(24)=0.
       SIGMAYE(25)=ESIGMAM
       SIGMAYE(26)=ESIGMAM
       SIGMAYE(27)=0.
       
C
       SIGMAZE(1)=ESIGMAM
       SIGMAZE(2)=ESIGMAM
       SIGMAZE(3)=ESIGMAM
       SIGMAZE(4)=ESIGMAM
       SIGMAZE(5)=ESIGMAM
       SIGMAZE(6)=ESIGMAM
       SIGMAZE(7)=ESIGMAM
       SIGMAZE(8)=ESIGMAM
C            
       SIGMAZE(9)=ESIGMAM
       SIGMAZE(10)=ESIGMAM
       SIGMAZE(11)=ESIGMAM
       SIGMAZE(12)=ESIGMAM
C
       SIGMAZE(13)=ESIGMAM
       SIGMAZE(14)=ESIGMAM
       SIGMAZE(15)=ESIGMAM
       SIGMAZE(16)=ESIGMAM
C
       SIGMAZE(17)=0.
       SIGMAZE(18)=0.
       SIGMAZE(19)=0.
       SIGMAZE(20)=0.
C
       SIGMAZE(21)=ESIGMAM
       SIGMAZE(22)=ESIGMAM
       SIGMAZE(23)=0.
       SIGMAZE(24)=0.
       SIGMAZE(25)=0.
       SIGMAZE(26)=0.
       SIGMAZE(27)=0.
C
       SIGMAXH(1)=HSIGMAM
       SIGMAXH(2)=HSIGMAM
       SIGMAXH(3)=HSIGMAM
       SIGMAXH(4)=HSIGMAM
       SIGMAXH(5)=HSIGMAM
       SIGMAXH(6)=HSIGMAM
       SIGMAXH(7)=HSIGMAM
       SIGMAXH(8)=HSIGMAM
C
       SIGMAXH(9)=HSIGMAM
       SIGMAXH(10)=HSIGMAM
       SIGMAXH(11)=HSIGMAM
       SIGMAXH(12)=HSIGMAM
C
       SIGMAXH(13)=0.
       SIGMAXH(14)=0.
       SIGMAXH(15)=0.
       SIGMAXH(16)=0.
C
       SIGMAXH(17)=HSIGMAM
       SIGMAXH(18)=HSIGMAM
       SIGMAXH(19)=HSIGMAM
       SIGMAXH(20)=HSIGMAM
C
       SIGMAXH(21)=0.
       SIGMAXH(22)=0.
       SIGMAXH(23)=HSIGMAM
       SIGMAXH(24)=HSIGMAM
       SIGMAXH(25)=0.
       SIGMAXH(26)=0.
       SIGMAXH(27)=0.
C
       SIGMAYH(1)=HSIGMAM
       SIGMAYH(2)=HSIGMAM
       SIGMAYH(3)=HSIGMAM
       SIGMAYH(4)=HSIGMAM
       SIGMAYH(5)=HSIGMAM
       SIGMAYH(6)=HSIGMAM
       SIGMAYH(7)=HSIGMAM
       SIGMAYH(8)=HSIGMAM
C            
       SIGMAYH(9)=0.
       SIGMAYH(10)=0.
       SIGMAYH(11)=0.
       SIGMAYH(12)=0.
C      
       SIGMAYH(13)=HSIGMAM
       SIGMAYH(14)=HSIGMAM
       SIGMAYH(15)=HSIGMAM
       SIGMAYH(16)=HSIGMAM
C
       SIGMAYH(17)=HSIGMAM
       SIGMAYH(18)=HSIGMAM
       SIGMAYH(19)=HSIGMAM
       SIGMAYH(20)=HSIGMAM
C      
       SIGMAYH(21)=0.
       SIGMAYH(22)=0.
       SIGMAYH(23)=0.
       SIGMAYH(24)=0.
       SIGMAYH(25)=HSIGMAM
       SIGMAYH(26)=HSIGMAM
       SIGMAYH(27)=0.
C
       SIGMAZH(1)=HSIGMAM
       SIGMAZH(2)=HSIGMAM
       SIGMAZH(3)=HSIGMAM
       SIGMAZH(4)=HSIGMAM
       SIGMAZH(5)=HSIGMAM
       SIGMAZH(6)=HSIGMAM
       SIGMAZH(7)=HSIGMAM
       SIGMAZH(8)=HSIGMAM
C            
       SIGMAZH(9)=HSIGMAM
       SIGMAZH(10)=HSIGMAM
       SIGMAZH(11)=HSIGMAM
       SIGMAZH(12)=HSIGMAM
C      
       SIGMAZH(13)=HSIGMAM
       SIGMAZH(14)=HSIGMAM
       SIGMAZH(15)=HSIGMAM
       SIGMAZH(16)=HSIGMAM
C
       SIGMAZH(17)=0.
       SIGMAZH(18)=0.
       SIGMAZH(19)=0.
       SIGMAZH(20)=0.
C      
       SIGMAZH(21)=HSIGMAM
       SIGMAZH(22)=HSIGMAM
       SIGMAZH(23)=0.
       SIGMAZH(24)=0.
       SIGMAZH(25)=0.
       SIGMAZH(26)=0.
       SIGMAZH(27)=0.

      RETURN
      END
      
C     ********************************************************

      SUBROUTINE OBSERVE(NOZ,VOLT)
      INCLUDE 'TLMMS.PAR' 
      EY0B=0.0
      EY0E=0.0

      VOLT=0.

      K=NOZ
      I=NXE/2

        DO J=1,MSH
         EYSGD =-RV(IDCELL(K,J,I))*(V(K,J,I,3)+V(K,J,I,4)+
     >   V(K,J,I,8)+V(K,J,I,11)+YNR(IDCELL(K,J,I))*V(K,J,I,14))

         VOLT=VOLT-EYSGD*DELY

        END DO


      RETURN
      END


