C*******************************************************
C     PROGRAM     : F-STRIP.FOR                        *
C     DATE        : MARCH 2002                         *
c     AUTHORS     : FUNDA AKLEMAN & LEVENT SEVGI       *
C     PURPOSE     : SIMPLE MICROTRIP LINE ANALYSIS     * 
C                   WITH FDTD+PML PACKAGE              *
C                   SPAR CODE IS REQUIRED FOR OFF-LINE *
C                   DFT ANALYSIS OF S PARAMETERS       *
C*******************************************************
      PROGRAM PMLMOD
      INCLUDE 'F-STRIP.PAR'

      OPEN(11,FILE='MSTRIP.INP')
      OPEN(30,FILE='MS-LINE.DAT') !only microstrip line

      OPEN(31,FILE='MS-STR.DAT') !microstrip circuit

      OPEN(32,FILE='MSFIELD.DAT')

      OPEN(33,FILE='F-SOURCE.DAT')


      READ(11,*) NSEN !
      READ(11,*) H !STRIP HEIGHT [mm]
      READ(11,*) ND  ! NUMBER OF CELLS (ANGLE ONLY FOR SCENARIO 5)
      READ(11,*) NSTOP


      DELX=H*1.E-3/MSH
      DELY=DELX
      DELZ=DELX!2.*DELX

      ESIGMAM=3.*EPS0*3.E8*ALOG(1./5.E-3)/(2.* NPML *DELX)
      HSIGMAM=ESIGMAM*XMU0/EPS0

      CALL ZERO
      CALL SETUP
C**********************
C  PML SUBROTINES     *
C**********************

      CALL PMLCUBE
      CALL PMLSETUP

C ---------MICOSTRIP LINE AND CIRCUIT LOOPS --------------------------
      DO NC=1,2

      IF (NC.EQ.1) THEN
       CALL BUILD(0) !MICROSTRIP LINE
       
      ELSE
       CALL ZERO
       CALL BUILD(NSEN) !MICROSTRIP CIRCUIT
       
      ENDIF

C****************************
C  MAIN FDTD LOOP  STARTS   *
C****************************

      T=0.0
      AKIE=0.0
      
      DO 100 N=1,NSTOP
                                      
      WRITE (*,*) N

 203   FORMAT(I4,(F14.10)) 
C********************
C ELECTRIC FIELDS   *
C********************
	CALL EXFLD
      CALL EYFLD
	CALL EZFLD

C***************************
C (PEC) BOUNDARIES         *
C***************************
	CALL PEC
	T=T+DT/2.
           
C***********************
C MAGNETIC FIELDS      *
C***********************
	CALL HXFLD
      CALL HYFLD
      CALL HZFLD
        
	T=T+DT/2.
      
      IF(NC.EQ.1) THEN !MICROSERIT ALTINDA ALANLAR
         WRITE(32,205) N*DT*1.E9,EY(NX/2,MSH-2,KL1),EY(NX/2,MSH-2,KL2),
     *                 EY(NX/2,MSH-2,KL3),EY(NX/2,MSH-2,KL4)
      END IF
      WRITE(33,*) N*DT*1E9,SOURCE(NX/2,MSH-1,NKZ)
205   FORMAT(F10.5,1X,4(F14.10,1X))

       CALL OBSERVE(NOZ1,V1,C1)
       CALL OBSERVE(NOZ2,V2,C2)

       IF(NC.EQ.1) WRITE (30,*) N*DT*1E9,V1,C1,V2,C2
       IF(NC.EQ.2) WRITE (31,*) N*DT*1E9,V1,C1,V2,C2

 100  CONTINUE
C**************************
C   END OF TIME LOOP      *
C**************************

      ENDDO

      CLOSE(31)
      STOP
      END

C*************************************************
      SUBROUTINE BUILD(NSEN)
      INCLUDE 'F-STRIP.PAR'

c  **** DIELECTRIC ****
       MTYPE=2
       CALL DCUBE (1,1,1,NX-1,MSH,NZ-1,MTYPE)

C **** Y'DE DIELECTRIC - AIR INTERFACE ****
       MTYPE=3
       CALL DCUBE (1,MSH+1,1,NX-1,0,NZ-1,MTYPE)


      IF(NSEN.EQ.0) THEN

C **** TOP CONDUCTOR STRIP ****       
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ-1,1)

C--------- SCENARIO - 1 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.1) THEN
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ/2-ND/2,1)
      CALL DCUBE (MSXB,MSH+1,NZ/2+ND/2,MSW,0,NZ/2-ND/2,1)

C--------- SCENARIO - 2 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.2) THEN
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ-1,1)
      CALL DCUBE (MSXB-2-MSW,MSH+1,NZ/2-ND/2,MSW,0,ND,1)
      CALL DCUBE (MSXB+MSW+2,MSH+1,NZ/2-ND/2,MSW,0,ND,1)

C--------- SCENARIO - 3 ( 1 < ND < 25) --------------
      ELSEIF (NSEN.EQ.3) THEN
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ-1,1)
      CALL DCUBE (MSXB-MSW,MSH+1,NZ/2-ND/2,3*MSW,0,ND,1)

C--------- SCENARIO - 4 ( 1 < ND < 21) --------------
      ELSEIF (NSEN.EQ.4) THEN
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ/2-ND/2-2,1)
      CALL DCUBE (MSXB,MSH+1,NZ/2+ND/2+2,MSW,0,NZ/2-ND/2-2,1)
      CALL DCUBE (MSXB-MSW,MSH+1,NZ/2-ND/2,3*MSW,0,ND,1)


C--------- SCENARIO - 5 ( 0 <= ND <= 90) --------------
      ELSEIF (NSEN.EQ.5) THEN

C      HALF RADIAL ANGLE  0 <= ND <= 90
       
       SR = 1.5*MSW*DELX !RADIAL LENGTH
       RTETA = ND*PI/180.


C **** K=NZ'YE KADAR UST ILETKEN SERIT ****
      CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ-1,1)

C **** BUTTERFLY
      IM=NX/2
      KM=NZ/2

      KRD=SR*SIN(RTETA)/DELZ
      IRD=SR*COS(RTETA)/DELX

      DO K=KM-KRD,KM+KRD
         DO I=IM-IRD,IM+IRD
          XR=(I-IM)*DELX
          ZR=(K-KM)*DELZ
C          DIST=SQRT(XR**2+ZR**2)
          TETA=ABS(ATAN(ZR/XR))
C          IF(DIST.LE.SR.AND.TETA.LE.RTETA) THEN
          IF(TETA.LE.RTETA) THEN
             CALL DCUBE (I,MSH+1,K,1,0,1,1)
          ENDIF
        END DO
      END DO
C--------- SCENARIO - 6 ( 0 <= ND <= 180) --------------
      ELSEIF (NSEN.EQ.6) THEN
        CALL DCUBE (MSXB,MSH+1,1,MSW,0,NZ-1,1)
        CALL DCUBE (MSXB-MSW,MSH+1,NZ/2-3*MSW/2,3*MSW,0,MSW,1)
        CALL DCUBE (MSXB-MSW,MSH+1,NZ/2+MSW/2,3*MSW,0,MSW,1)

      ELSEIF (NSEN.EQ.7) THEN
        CALL DCUBE (MSXB,MSH+1,1,MSW,0,3*NZ/4,1)

      ELSEIF (NSEN.EQ.8) THEN
        CALL DCUBE (MSXB,MSH+1,1,MSW,0,70,1)
        CALL DCUBE (MSXB-MSW,MSH+1,56,3*MSW,0,14,1)
      ENDIF

      RETURN
      END

C********************************************************
      SUBROUTINE DCUBE (ISTART,JSTART,KSTART,NXWIDE,NYWIDE,NZWIDE,MTYPE)
      INCLUDE 'F-STRIP.PAR'
C
C     THIS SUBROUTINE SETS ALL TWELVE IDXXX COMPONENTS FOR ONE CUBE
C     TO THE SAME MATERIAL TYPE SPECIFIED BY MTYPE.  IF NXWIDE, NYWIDE,
C     OR NZWIDE=0, THEN ONLY 4 IDXXX ARRAY COMPONENTS WILL BE SET
C     CORRESPONDING TO AN INFINITELY THIN PLATE.  THE SUBROUTINE IS MOST
C     USEFUL CONSTRUCTING OBJECTS WITH MANY CELLS OF THE SAME MATERIAL
C     (I.E. CUBES, PEC PLATES, ETC.).  THIS SUBROUTINE DOES NOT
C     AUTOMATICALLY DO WIRES!
C
      IMAX=ISTART+NXWIDE-1
      JMAX=JSTART+NYWIDE-1
      KMAX=KSTART+NZWIDE-1
C
      IF (NXWIDE.EQ.0) THEN
	DO 20 K=KSTART,KMAX
	  DO 10 J=JSTART,JMAX
	    IDTWO(ISTART,J,K)=MTYPE
	    IDTWO(ISTART,J,K+1)=MTYPE
	    IDTHRE(ISTART,J,K)=MTYPE
	    IDTHRE(ISTART,J+1,K)=MTYPE
 10       CONTINUE
 20     CONTINUE
      ELSEIF (NYWIDE.EQ.0) THEN
	DO 40 K=KSTART,KMAX
	  DO 30 I=ISTART,IMAX
	    IDONE(I,JSTART,K)=MTYPE
	    IDONE(I,JSTART,K+1)=MTYPE
	    IDTHRE(I,JSTART,K)=MTYPE
	    IDTHRE(I+1,JSTART,K)=MTYPE
 30       CONTINUE
 40     CONTINUE
      ELSEIF (NZWIDE.EQ.0) THEN
	DO 60 J=JSTART,JMAX
	  DO 50 I=ISTART,IMAX
	    IDONE(I,J,KSTART)=MTYPE
	    IDONE(I,J+1,KSTART)=MTYPE
	    IDTWO(I,J,KSTART)=MTYPE
	    IDTWO(I+1,J,KSTART)=MTYPE
 50       CONTINUE
 60     CONTINUE
      ELSE
	DO 90 K=KSTART,KMAX
	  DO 80 J=JSTART,JMAX
	    DO 70 I=ISTART,IMAX
	      IDONE(I,J,K)=MTYPE
	      IDONE(I,J,K+1)=MTYPE
	      IDONE(I,J+1,K+1)=MTYPE
	      IDONE(I,J+1,K)=MTYPE
	      IDTWO(I,J,K)=MTYPE
	      IDTWO(I+1,J,K)=MTYPE
	      IDTWO(I+1,J,K+1)=MTYPE
	      IDTWO(I,J,K+1)=MTYPE
	      IDTHRE(I,J,K)=MTYPE
	      IDTHRE(I+1,J,K)=MTYPE
	      IDTHRE(I+1,J+1,K)=MTYPE
	      IDTHRE(I,J+1,K)=MTYPE
 70         CONTINUE
 80       CONTINUE
 90     CONTINUE
      ENDIF
      RETURN
      END
C********************************************************
      SUBROUTINE PMLCUBE 
      INCLUDE 'F-STRIP.PAR'
C
        DO K=1,NZ
        DO J=1,NY
        DO I=1,NX
C
C         PML CORNERS
C
         IF (I.LE.NXB.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=1
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=2
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=3
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=4
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=5
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=6
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=7
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=8
C
C         PML EDGES
C
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=9
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=10
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=11
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=12
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=13
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=14
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.LE.NZB) THEN
            IDPML(I,J,K)=15
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB.AND.K.GE.NZS) THEN
            IDPML(I,J,K)=16
C
         ELSEIF (I.LE.NXB.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=17
         ELSEIF (I.GE.NXS.AND.J.GE.NYS.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=18
         ELSEIF (I.LE.NXB.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=19
         ELSEIF (I.GE.NXS.AND.J.LE.NYB.AND.K.GT.NZB.AND.K.LT.NZS) THEN
            IDPML(I,J,K)=20
C
C         PML FACES 
C
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.LE.NZB) THEN
          IDPML(I,J,K)=21
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.GE.NZS) THEN
          IDPML(I,J,K)=22
         ELSEIF (I.LE.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=23
         ELSEIF (I.GE.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=24
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.LE.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=25
         ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GE.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
          IDPML(I,J,K)=26
         ELSE
           IDPML(I,J,K)=27
         ENDIF
        ENDDO
        ENDDO
        ENDDO

      RETURN
      END
C*******************************************************

      SUBROUTINE SETUP
      INCLUDE 'F-STRIP.PAR'
C
      C=1.0/SQRT(XMU0*EPS0)
C
      DTXI=C/DELX
      DTYI=C/DELY
      DTZI=C/DELZ
C
      DT=DELX/(2.*C) 

      BAND=1.E-9*C/(20.*DELZ) 

C     PARAMETER ALPHA IS THE DECAY RATE DETERMINED BY BETA.
       ALPHA=3.3E18*BAND**2
       SHIFT=4.0/SQRT(ALPHA)

C ******* CONSTITUTIVE PARAMETERS ************
      DO 10 I=1,9
	EPS(I)=EPS0
	SIGMA(I)=0.0
	EPSP(I)=EPS0
 10   CONTINUE
C
C     DEFINE EPS AND SIGMA FOR EACH MATERIAL HERE
C
      EPS(2)=EPSR*EPS0
      EPSP(3)=EPSR*EPS0
      SIGMA(2)=0.0
      EPS(3)=.5*(EPS(1)+EPS(2))
      EPSP(4)=.5*(EPSP(2)+EPSP(3))
      SIGMA(3)=0.0
C
C     FREE SPACE
C
      DTEDX=DT/(EPS0*DELX)
      DTEDY=DT/(EPS0*DELY)
      DTEDZ=DT/(EPS0*DELZ)
      DTMDX=DT/(XMU0*DELX)
      DTMDY=DT/(XMU0*DELY)
      DTMDZ=DT/(XMU0*DELZ)
C
C     LOSSY DIELECTRICS
C
      DO 20 I=2,9
	ESB(I)=(2.*EPS(I)-SIGMA(I)*DT)/(2.*EPS(I)+SIGMA(I) *DT)
	ECRLX(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELX)
	ECRLY(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELY)
	ECRLZ(I)=2.*DT/((2.*EPS(I)+SIGMA(I)*DT)*DELZ)
 20   CONTINUE
      RETURN
      END
C     ********************************************************
      SUBROUTINE PMLSETUP
      INCLUDE 'F-STRIP.PAR'
       
       SIGMAXE(1)=ESIGMAM
       SIGMAXE(2)=ESIGMAM
       SIGMAXE(3)=ESIGMAM
       SIGMAXE(4)=ESIGMAM
       SIGMAXE(5)=ESIGMAM
       SIGMAXE(6)=ESIGMAM
       SIGMAXE(7)=ESIGMAM
       SIGMAXE(8)=ESIGMAM
       SIGMAXE(9)=ESIGMAM
       SIGMAXE(10)=ESIGMAM
       SIGMAXE(11)=ESIGMAM
       SIGMAXE(12)=ESIGMAM
C
       SIGMAXE(13)=0.
       SIGMAXE(14)=0.
       SIGMAXE(15)=0.
       SIGMAXE(16)=0.
C
       SIGMAXE(17)=ESIGMAM
       SIGMAXE(18)=ESIGMAM
       SIGMAXE(19)=ESIGMAM
       SIGMAXE(20)=ESIGMAM
C
       SIGMAXE(21)=0.
       SIGMAXE(22)=0.
       SIGMAXE(23)=ESIGMAM
       SIGMAXE(24)=ESIGMAM
       SIGMAXE(25)=0.
       SIGMAXE(26)=0.
       SIGMAXE(27)=0.
       
C
       SIGMAYE(1)=ESIGMAM
       SIGMAYE(2)=ESIGMAM
       SIGMAYE(3)=ESIGMAM
       SIGMAYE(4)=ESIGMAM
       SIGMAYE(5)=ESIGMAM
       SIGMAYE(6)=ESIGMAM
       SIGMAYE(7)=ESIGMAM
       SIGMAYE(8)=ESIGMAM
C            
       SIGMAYE(9)=0.
       SIGMAYE(10)=0.
       SIGMAYE(11)=0.
       SIGMAYE(12)=0.
C
       SIGMAYE(13)=ESIGMAM
       SIGMAYE(14)=ESIGMAM
       SIGMAYE(15)=ESIGMAM
       SIGMAYE(16)=ESIGMAM
C
       SIGMAYE(17)=ESIGMAM
       SIGMAYE(18)=ESIGMAM
       SIGMAYE(19)=ESIGMAM
       SIGMAYE(20)=ESIGMAM
C
       SIGMAYE(21)=0.
       SIGMAYE(22)=0.
       SIGMAYE(23)=0.
       SIGMAYE(24)=0.
       SIGMAYE(25)=ESIGMAM
       SIGMAYE(26)=ESIGMAM
       SIGMAYE(27)=0.
       
C
       SIGMAZE(1)=ESIGMAM
       SIGMAZE(2)=ESIGMAM
       SIGMAZE(3)=ESIGMAM
       SIGMAZE(4)=ESIGMAM
       SIGMAZE(5)=ESIGMAM
       SIGMAZE(6)=ESIGMAM
       SIGMAZE(7)=ESIGMAM
       SIGMAZE(8)=ESIGMAM
C            
       SIGMAZE(9)=ESIGMAM
       SIGMAZE(10)=ESIGMAM
       SIGMAZE(11)=ESIGMAM
       SIGMAZE(12)=ESIGMAM
C
       SIGMAZE(13)=ESIGMAM
       SIGMAZE(14)=ESIGMAM
       SIGMAZE(15)=ESIGMAM
       SIGMAZE(16)=ESIGMAM
C
       SIGMAZE(17)=0.
       SIGMAZE(18)=0.
       SIGMAZE(19)=0.
       SIGMAZE(20)=0.
C
       SIGMAZE(21)=ESIGMAM
       SIGMAZE(22)=ESIGMAM
       SIGMAZE(23)=0.
       SIGMAZE(24)=0.
       SIGMAZE(25)=0.
       SIGMAZE(26)=0.
       SIGMAZE(27)=0.
C
       SIGMAXH(1)=HSIGMAM
       SIGMAXH(2)=HSIGMAM
       SIGMAXH(3)=HSIGMAM
       SIGMAXH(4)=HSIGMAM
       SIGMAXH(5)=HSIGMAM
       SIGMAXH(6)=HSIGMAM
       SIGMAXH(7)=HSIGMAM
       SIGMAXH(8)=HSIGMAM
C
       SIGMAXH(9)=HSIGMAM
       SIGMAXH(10)=HSIGMAM
       SIGMAXH(11)=HSIGMAM
       SIGMAXH(12)=HSIGMAM
C
       SIGMAXH(13)=0.
       SIGMAXH(14)=0.
       SIGMAXH(15)=0.
       SIGMAXH(16)=0.
C
       SIGMAXH(17)=HSIGMAM
       SIGMAXH(18)=HSIGMAM
       SIGMAXH(19)=HSIGMAM
       SIGMAXH(20)=HSIGMAM
C
       SIGMAXH(21)=0.
       SIGMAXH(22)=0.
       SIGMAXH(23)=HSIGMAM
       SIGMAXH(24)=HSIGMAM
       SIGMAXH(25)=0.
       SIGMAXH(26)=0.
       SIGMAXH(27)=0.
C
       SIGMAYH(1)=HSIGMAM
       SIGMAYH(2)=HSIGMAM
       SIGMAYH(3)=HSIGMAM
       SIGMAYH(4)=HSIGMAM
       SIGMAYH(5)=HSIGMAM
       SIGMAYH(6)=HSIGMAM
       SIGMAYH(7)=HSIGMAM
       SIGMAYH(8)=HSIGMAM
C            
       SIGMAYH(9)=0.
       SIGMAYH(10)=0.
       SIGMAYH(11)=0.
       SIGMAYH(12)=0.
C      
       SIGMAYH(13)=HSIGMAM
       SIGMAYH(14)=HSIGMAM
       SIGMAYH(15)=HSIGMAM
       SIGMAYH(16)=HSIGMAM
C
       SIGMAYH(17)=HSIGMAM
       SIGMAYH(18)=HSIGMAM
       SIGMAYH(19)=HSIGMAM
       SIGMAYH(20)=HSIGMAM
C      
       SIGMAYH(21)=0.
       SIGMAYH(22)=0.
       SIGMAYH(23)=0.
       SIGMAYH(24)=0.
       SIGMAYH(25)=HSIGMAM
       SIGMAYH(26)=HSIGMAM
       SIGMAYH(27)=0.
C
       SIGMAZH(1)=HSIGMAM
       SIGMAZH(2)=HSIGMAM
       SIGMAZH(3)=HSIGMAM
       SIGMAZH(4)=HSIGMAM
       SIGMAZH(5)=HSIGMAM
       SIGMAZH(6)=HSIGMAM
       SIGMAZH(7)=HSIGMAM
       SIGMAZH(8)=HSIGMAM
C            
       SIGMAZH(9)=HSIGMAM
       SIGMAZH(10)=HSIGMAM
       SIGMAZH(11)=HSIGMAM
       SIGMAZH(12)=HSIGMAM
C      
       SIGMAZH(13)=HSIGMAM
       SIGMAZH(14)=HSIGMAM
       SIGMAZH(15)=HSIGMAM
       SIGMAZH(16)=HSIGMAM
C
       SIGMAZH(17)=0.
       SIGMAZH(18)=0.
       SIGMAZH(19)=0.
       SIGMAZH(20)=0.
C      
       SIGMAZH(21)=HSIGMAM
       SIGMAZH(22)=HSIGMAM
       SIGMAZH(23)=0.
       SIGMAZH(24)=0.
       SIGMAZH(25)=0.
       SIGMAZH(26)=0.
       SIGMAZH(27)=0.

      RETURN
      END
C     ********************************************************
   
      SUBROUTINE EXFLD
      INCLUDE 'F-STRIP.PAR'
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=1,NX1
          II=I
C
C         FDTD REGION 
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS.
     $                              AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

C           DETERMINE MATERIAL TYPE
	    IF(IDONE(I,J,K).EQ.0) GO TO 100
	    IF(IDONE(I,J,K).EQ.1) GO TO 200
	    GO TO 300

C           FREE SPACE
 100        EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)-HZ(I,J-1,K))*DTEDY
     $                           -(HY(I,J,K)-HY(I,J,K-1))*DTEDZ
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EX(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZ(I,J-1,K))*ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HY(I,J,K-1))*ECRLZ(IDONE(I,J,K))

          ELSE
C
C         PML REGION 
C
            CALL PMLEX(II,JJ,KK)
            
          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE

      RETURN
      END
C     ********************************************************
      SUBROUTINE EYFLD
      INCLUDE 'F-STRIP.PAR'
      REAL SOURCE
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I	  
C
C         FDTD REGION 
C
          IF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS.
     $                              AND.K.GT.NZBP1.AND.K.LT.NZS) THEN


C           DETERMINE MATERIAL TYPE
	    IF(IDTWO(I,J,K).EQ.0) GO TO 100
	    IF(IDTWO(I,J,K).EQ.1) GO TO 200
	    GO TO 300

C           FREE SPACE
 100        EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)-HX(I,J,K-1))*DTEDZ
     $                         -(HZ(I,J,K)-HZ(I-1,J,K))*DTEDX
     $                         +SOURCE(I,J,K)
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EY(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HX(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZ(I-1,J,K))*ECRLX(IDTWO(I,J,K))
     $      +SOURCE(I,J,K)

          ELSE
C
C         PML REGION
C
            CALL PMLEY(II,JJ,KK)
	  ENDIF
  
  10      CONTINUE
  20    CONTINUE
  30  CONTINUE

      RETURN
      END
C     ********************************************************
      SUBROUTINE EZFLD
      INCLUDE 'F-STRIP.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I
C
C         FDTD REGION
C
          IF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS.
     $                              AND.K.GT.NZB.AND.K.LT.NZS) THEN


C           DETERMINE MATERIAL TYPE
	    IF(IDTHRE(I,J,K).EQ.0) GO TO 100
	    IF(IDTHRE(I,J,K).EQ.1) GO TO 200
	    GO TO 300
C
C           FREE SPACE
 100        EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)-HY(I-1,J,K))*DTEDX
     $                           -(HX(I,J,K)-HX(I,J-1,K))*DTEDY
	    GO TO 10

C           PERFECT CONDUCTOR
 200        EZ(I,J,K)=0.0
	    GO TO 10

C           LOSSY DIELECTRIC
 300        EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HY(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HX(I,J-1,K))*ECRLY(IDTHRE(I,J,K))

          ELSE
C
C         PML REGION 
C
            CALL PMLEZ(II,JJ,KK)

          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END

C     *****************************************************************
      SUBROUTINE HXFLD
      INCLUDE 'F-STRIP.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=2,NX1
          II=I
C
C         FDTD REGION
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYSM1.
     $                              AND.K.GT.NZB.AND.K.LT.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)-(EZ(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                           +(EY(I,J,K+1)-EY(I,J,K))*DTMDZ
          ELSE
C
C         PML REGION 
C
            CALL PMLHX(II,JJ,KK)

	  ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HYFLD
      INCLUDE 'F-STRIP.PAR'
C
      DO 30 K=1,NZ1
        KK=K
	DO 20 J=2,NY1
          JJ=J
	  DO 10 I=1,NX1
            II=I
C
C         FDTD REGION
C
          IF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS.
     $                              AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	    HY(I,J,K)=HY(I,J,K)-(EX(I,J,K+1)-EX(I,J,K))*DTMDZ
     $                           +(EZ(I+1,J,K)-EZ(I,J,K))*DTMDX
          ELSE
C
C         PML REGION 
C
            CALL PMLHY(II,JJ,KK)

          ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ********************************************************
      SUBROUTINE HZFLD
      INCLUDE 'F-STRIP.PAR'
C
      DO 30 K=2,NZ1
        KK=K
	DO 20 J=1,NY1
          JJ=J
	  DO 10 I=1,NX1
            II=I
C
C         FDTD REGION
C
          IF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYSM1.
     $                              AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    HZ(I,J,K)=HZ(I,J,K)-(EY(I+1,J,K)-EY(I,J,K))*DTMDX
     $                           +(EX(I,J+1,K)-EX(I,J,K))*DTMDY
          ELSE
C
C         PML REGION 
C
            CALL PMLHZ(II,JJ,KK)

	  ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C     ***************************************************************
      SUBROUTINE ZERO
      INCLUDE 'F-STRIP.PAR'
C
      T=0.0
      DO 30 K=1,NZ
	DO 20 J=1,NY
	  DO 10 I=1,NX
	    EX(I,J,K)=0.0
	    EY(I,J,K)=0.0
	    EZ(I,J,K)=0.0
	    HX(I,J,K)=0.0
	    HY(I,J,K)=0.0
	    HZ(I,J,K)=0.0
	    EXY(I,J,K)=0.0
	    EYX(I,J,K)=0.0
	    EZX(I,J,K)=0.0
	    EXZ(I,J,K)=0.0
	    EYZ(I,J,K)=0.0
	    EZY(I,J,K)=0.0
	    HXY(I,J,K)=0.0
	    HYX(I,J,K)=0.0
	    HZX(I,J,K)=0.0
	    HXZ(I,J,K)=0.0
	    HYZ(I,J,K)=0.0
	    HZY(I,J,K)=0.0
	    IDONE(I,J,K)=0.0
	    IDTWO(I,J,K)=0.0
	    IDTHRE(I,J,K)=0.0
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE


      RETURN
      END

C----------------------------------
      REAL FUNCTION SOURCE(I,J,K)
      INCLUDE 'F-STRIP.PAR'
      
      SOURCE=0.0

       IF(I.GE.MSXB.AND.I.LT.(MSXB+MSW)
     $             .AND.J.LE.MSH.AND.K.EQ.NKZ) THEN

         SOURCE=AMP*EXP(-ALPHA*(T-SHIFT)**2)
     
	ENDIF


      RETURN
      END


*---------------------------------------------------------------------
      SUBROUTINE OBSERVE(NOZ,VOLT,CURRENT)
      INCLUDE 'F-STRIP.PAR' 
      EY0B=0.0
      EY0E=0.0

      VOLT=0.
C      DO I=10,50
        DO J=1,MSH
         EYOZ=0.25*(EY(NX/2,J,NOZ)+EY(NX/2+1,J,NOZ)
     >    +EY(NX/2+1,J,NOZ+1)+EY(NX/2,J,NOZ+1))

         VOLT=VOLT-EYOZ*DELY

        END DO
C      END DO

      CURRENT=0.0
      CUR=0.0
      
      DO I=INXB,INXS
       HXB=0.5*(HX(I,JNYB,NOZ)+HX(I+1,JNYB,NOZ))
       HXS=0.5*(HX(I,JNYS,NOZ)+HX(I+1,JNYS,NOZ))
       CUR=CUR+(HXB-HXS)*DELX
      END DO
      
      DO J=JNYB,JNYS
       HYB=0.5*(HY(INXB,J,NOZ)+HY(INXB,J+1,NOZ))
       HYS=0.5*(HY(INXS,J,NOZ)+HY(INXS,J+1,NOZ))
       CUR=CUR+(-HYB+HYS)*DELY
      END DO
      
      CURRENT=0.5*(CUR+CUR_OLD)
      CUR_OLD=CUR

      RETURN
      END

C     ********************************************************

      SUBROUTINE PMLEX(I,J,K)
      INCLUDE 'F-STRIP.PAR'
      
      XJ=1*J
      XK=1*K
      EPSF=EPSP(IDONE(I,J,K)+1)/EPS0
      IF(J.LE.NYB) THEN
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XNPML-XJ+1.)/XNPML)**2.
      ELSE
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XJ-YN+XNPML)/XNPML)**2.
      ENDIF
      
      IF(K.LE.NZB) THEN
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XNPML-XK+1.)/XNPML)**2.
      ELSE
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XK-ZN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAY.LE.0.) THEN
       DTESY=1.
       DTESDY=DT/(DELY*EPSP(IDONE(I,J,K)+1))
      ELSE
       DTESY=EXP(-(ESIGMAY*DT)/EPSP(IDONE(I,J,K)+1))
       DTESDY=(1.-DTESY)/(ESIGMAY*DELY)
      ENDIF

      IF (ESIGMAZ.LE.0.) THEN
       DTESZ=1.
       DTESDZ=DT/(DELZ*EPSP(IDONE(I,J,K)+1))
      ELSE
       DTESZ=EXP(-(ESIGMAZ*DT)/EPSP(IDONE(I,J,K)+1))
       DTESDZ=(1.-DTESZ)/(ESIGMAZ*DELZ)
      ENDIF
C
C         PML INTERFACE
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

            IF (IDONE(I,J,K).EQ.0) THEN
	    
             EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)
     $               -HZX(I,J-1,K)-HZY(I,J-1,K))*DTEDY
     $               -(HY(I,J,K)-HY(I,J,K-1))*DTEDZ
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZX(I,J-1,K)-HZY(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HY(I,J,K-1))*ECRLZ(IDONE(I,J,K))

	    ENDIF
	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYBP1.AND.J.LT.NYS
     $                                         .AND.K.EQ.NZBP1) THEN

	    IF (IDONE(I,J,K).EQ.0) THEN
	    
	     EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)-HZ(I,J-1,K))*DTEDY
     $               -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*DTEDZ
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZ(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*ECRLZ(IDONE(I,J,K))
	    ENDIF


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
                 
	   IF(IDONE(I,J,K).EQ.1) THEN
	    EXY(I,J,K)=0.0
	    EXZ(I,J,K)=0.0
	   ELSE
	    EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZ(I,J-1,K))*DTESDY
	    EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HYX(I,J,K-1)-HYZ(I,J,K-1))*DTESDZ
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                         .AND.K.EQ.NZS) THEN
	   IF(IDONE(I,J,K).EQ.1) THEN
	    EXY(I,J,K)=0.0
	    EXZ(I,J,K)=0.0

	   ELSE
	   
	    EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HY(I,J,K-1))*DTESDZ
	    EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZX(I,J-1,K)-HZY(I,J-1,K))*DTESDY
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.EQ.NZBP1) THEN
	    IF (IDONE(I,J,K).EQ.0) THEN
	    
	     EX(I,J,K)=EX(I,J,K)+(HZ(I,J,K)
     $     -HZX(I,J-1,K)-HZY(I,J-1,K))*DTEDY
     $    -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*DTEDZ
	    
	    ELSEIF (IDONE(I,J,K).EQ.1) THEN  
	     EX(I,J,K)=0.0
	    
	    ELSE
	     EX(I,J,K)=EX(I,J,K)*ESB(IDONE(I,J,K))
     $      +(HZ(I,J,K)-HZX(I,J-1,K)-HZY(I,J-1,K))
     $      *ECRLY(IDONE(I,J,K))
     $      -(HY(I,J,K)-HYX(I,J,K-1)-HYZ(I,J,K-1))*ECRLZ(IDONE(I,J,K))

	    ENDIF
C
C         PML
C
          ELSE
      
            IF(IDONE(I,J,K).EQ.1) THEN
              EXY(I,J,K)=0.0
              EXZ(I,J,K)=0.0

            ELSE
              EXY(I,J,K)=EXY(I,J,K)*DTESY+(HZX(I,J,K)+HZY(I,J,K)
     $              -HZX(I,J-1,K)-HZY(I,J-1,K))*DTESDY
              EXZ(I,J,K)=EXZ(I,J,K)*DTESZ-(HYX(I,J,K)+HYZ(I,J,K)
     $              -HYX(I,J,K-1)-HYZ(I,J,K-1))*DTESDZ
      
            ENDIF
          ENDIF
      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLEY(I,J,K)
      INCLUDE 'F-STRIP.PAR'
      REAL SOURCE
      
      XI=1*I
      XK=1*K

      EPSF=EPSP(IDTWO(I,J,K)+1)/EPS0

      IF(I.LE.NXB) THEN
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XNPML-XI+1.)/XNPML)**2.
      ELSE
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XI-XN+XNPML)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XNPML-XK+1.)/XNPML)**2.
      ELSE
        ESIGMAZ=SIGMAZE(IDPML(I,J,K))*EPSF*((XK-ZN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAZ.LE.0.) THEN
       DTESZ=1.
       DTESDZ=DT/(DELZ*EPSP(IDTWO(I,J,K)+1))
      ELSE
       DTESZ=EXP(-(ESIGMAZ*DT)/EPSP(IDTWO(I,J,K)+1))
       DTESDZ=(1.-DTESZ)/(ESIGMAZ*DELZ)
      ENDIF

      IF (ESIGMAX.LE.0.) THEN
       DTESX=1.
       DTESDX=DT/(DELX*EPSP(IDTWO(I,J,K)+1))
      ELSE
       DTESX=EXP(-(ESIGMAX*DT)/EPSP(IDTWO(I,J,K)+1))
       DTESDX=(1.-DTESX)/(ESIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXBP1.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZBP1.AND.K.LT.NZS) THEN

	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)-HX(I,J,K-1))*DTEDZ
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))
     $      *DTEDX+SOURCE(I,J,K)
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HX(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*ECRLX(IDTWO(I,J,K))
     $      +SOURCE(I,J,K)
	    ENDIF


	  ELSEIF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZBP1) THEN


	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)
     $              -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTEDZ
     $            -(HZ(I,J,K)-HZ(I-1,J,K))*DTEDX+SOURCE(I,J,K)
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HXY(I,J,K-1)-HXZ(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZ(I-1,J,K))*ECRLX(IDTWO(I,J,K))
     $      +SOURCE(I,J,K)
	    ENDIF


	  ELSEIF (I.EQ.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   IF(IDTWO(I,J,K).EQ.1) THEN
	    EYX(I,J,K)=0.0
	    EYZ(I,J,K)=0.0

	   ELSE

	    EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZ(I-1,J,K))*DTESDX
	    EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTESDZ
	   ENDIF

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZS) THEN
	   
	   IF(IDTWO(I,J,K).EQ.1) THEN
	    EYX(I,J,K)=0.0
	    EYZ(I,J,K)=0.0

	   ELSE
	   
	    EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HX(I,J,K-1))*DTESDZ
	    EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $                           -HZX(I-1,J,K)-HZY(I-1,J,K))*DTESDX
	   ENDIF

	  ELSEIF (I.EQ.NXBP1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZBP1) THEN

	    IF (IDTWO(I,J,K).EQ.0) THEN
	     EY(I,J,K)=EY(I,J,K)+(HX(I,J,K)
     $     -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTEDZ
     $     -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*DTEDX
     $     +SOURCE(I,J,K)
	    
	    ELSEIF (IDTWO(I,J,K).EQ.1) THEN
	     EY(I,J,K)=0.0
	    ELSE
	    
	    EY(I,J,K)=EY(I,J,K)*ESB(IDTWO(I,J,K))
     $      +(HX(I,J,K)-HXY(I,J,K-1)-HXZ(I,J,K-1))*ECRLZ(IDTWO(I,J,K))
     $      -(HZ(I,J,K)-HZX(I-1,J,K)-HZY(I-1,J,K))*ECRLX(IDTWO(I,J,K))
     $      +SOURCE(I,J,K)
	    ENDIF
C
C         PML
C
          ELSE
          
            IF(IDTWO(I,J,K).EQ.1) THEN
              EYX(I,J,K)=0.0
              EYZ(I,J,K)=0.0

            ELSE

              EYZ(I,J,K)=EYZ(I,J,K)*DTESZ+(HXY(I,J,K)+HXZ(I,J,K)
     $              -HXY(I,J,K-1)-HXZ(I,J,K-1))*DTESDZ
              EYX(I,J,K)=EYX(I,J,K)*DTESX-(HZX(I,J,K)+HZY(I,J,K)
     $              -HZX(I-1,J,K)-HZY(I-1,J,K))*DTESDX

           ENDIF
          ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLEZ(I,J,K)
      INCLUDE 'F-STRIP.PAR'

      XJ=1*J
      XI=1*I
     
      EPSF=EPSP(IDTHRE(I,J,K)+1)/EPS0
      IF(J.LE.NYB) THEN
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XNPML-XJ+1.)/XNPML)**2.
      ELSE
        ESIGMAY=SIGMAYE(IDPML(I,J,K))*EPSF*((XJ-YN+XNPML)/XNPML)**2.
      ENDIF

      IF(I.LE.NXB) THEN
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XNPML-XI+1.)/XNPML)**2.
      ELSE
        ESIGMAX=SIGMAXE(IDPML(I,J,K))*EPSF*((XI-XN+XNPML)/XNPML)**2.
      ENDIF

      IF (ESIGMAY.LE.0.) THEN
       DTESY=1.
       DTESDY=DT/(DELY*EPSP(IDTHRE(I,J,K)+1))
      ELSE
       DTESY=EXP(-(ESIGMAY*DT)/EPSP(IDTHRE(I,J,K)+1))
       DTESDY=(1.-DTESY)/(ESIGMAY*DELY)
      ENDIF

      IF (ESIGMAX.LE.0.) THEN
       DTESX=1.
       DTESDX=DT/(DELX*EPSP(IDTHRE(I,J,K)+1))
      ELSE
       DTESX=EXP(-(ESIGMAX*DT)/EPSP(IDTHRE(I,J,K)+1))
       DTESDX=(1.-DTESX)/(ESIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
         IF (I.EQ.NXBP1.AND.J.GT.NYBP1.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    IF (IDTHRE(I,J,K).EQ.0) THEN
	     EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)-HYX(I-1,J,K)
     $               -HYZ(I-1,J,K))*DTEDX
     $               -(HX(I,J,K)-HX(I,J-1,K))*DTEDY
	    
	    ELSEIF (IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HX(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF


	  ELSEIF (I.GT.NXBP1.AND.I.LT.NXS.AND.J.EQ.NYBP1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	    IF (IDTHRE(I,J,K).EQ.0) THEN
	     EZ(I,J,K)=EZ(I,J,K)+(HY(I,J,K)
     $                -HY(I-1,J,K))*DTEDX
     $                -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))*DTEDY
	    
	    ELSEIF (IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HY(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF
	    
	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   IF(IDTHRE(I,J,K).EQ.1) THEN
	    EZX(I,J,K)=0.0
	    EZY(I,J,K)=0.0

	   ELSE
		    
	    EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $                           -HX(I,J-1,K))*DTESDY
	    EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $                -HYX(I-1,J,K)-HYZ(I-1,J,K))*DTESDX
	   ENDIF


	  ELSEIF (I.EQ.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   
	   IF(IDTHRE(I,J,K).EQ.1) THEN
	    EZX(I,J,K)=0.0
	    EZY(I,J,K)=0.0

	   ELSE

	    EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $                           -HY(I-1,J,K))*DTESDX
	    EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $                -HXY(I,J-1,K)-HXZ(I,J-1,K))*DTESDY
	   ENDIF

	  ELSEIF (I.EQ.NXBP1.AND.J.EQ.NYBP1
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   IF(IDTHRE(I,J,K).EQ.0) THEN
	    EZ(I,J,K)=EZ(I,J,K)+
     $     (HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*DTEDX
     $     -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))*DTEDY
	    
	    ELSEIF(IDTHRE(I,J,K).EQ.1) THEN
	     EZ(I,J,K)=0.0
	    ELSE
	     EZ(I,J,K)=EZ(I,J,K)*ESB(IDTHRE(I,J,K))
     $      +(HY(I,J,K)-HYX(I-1,J,K)-HYZ(I-1,J,K))*ECRLX(IDTHRE(I,J,K))
     $      -(HX(I,J,K)-HXY(I,J-1,K)-HXZ(I,J-1,K))
     $      *ECRLY(IDTHRE(I,J,K))
	    ENDIF
C
C         PML
C
	   ELSE
            
            IF(IDTHRE(I,J,K).EQ.1) THEN
              EZX(I,J,K)=0.0
              EZY(I,J,K)=0.0

            ELSE
	  
              EZX(I,J,K)=EZX(I,J,K)*DTESX+(HYX(I,J,K)+HYZ(I,J,K)
     $              -HYX(I-1,J,K)-HYZ(I-1,J,K))*DTESDX
              EZY(I,J,K)=EZY(I,J,K)*DTESY-(HXY(I,J,K)+HXZ(I,J,K)
     $              -HXY(I,J-1,K)-HXZ(I,J-1,K))*DTESDY

            ENDIF
           ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHX(I,J,K)
      INCLUDE 'F-STRIP.PAR'
      
      XJ=1*J
      XK=1*K

      IF(J.LE.NYB) THEN
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XNPML-XJ+0.5)/XNPML)**2.
      ELSE
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XJ-YN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XNPML-XK+0.5)/XNPML)**2.
      ELSE
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XK-ZN+XNPML+0.5)/XNPML)**2.
      ENDIF

      IF (HSIGMAZ.LE.0.) THEN
       DTMSZ=1.
       DTMSDZ=DT/(DELZ*XMU0)
      ELSE
       DTMSZ=EXP(-(HSIGMAZ*DT)/XMU0)
       DTMSDZ=(1.-DTMSZ)/(HSIGMAZ*DELZ)
      ENDIF

      IF (HSIGMAY.LE.0.) THEN
       DTMSY=1.
       DTMSDY=DT/(DELY*XMU0)
      ELSE
       DTMSY=EXP(-(HSIGMAY*DT)/XMU0)
       DTMSDY=(1.-DTMSY)/(HSIGMAY*DELY)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	    HX(I,J,K)=HX(I,J,K)-(EZX(I,J+1,K)+EZY(I,J+1,K)
     $                -EZ(I,J,K))*DTMDY+(EY(I,J,K+1)-EY(I,J,K))*DTMDZ

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYSM1
     $                                          .AND.K.EQ.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)-(EZ(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                +(EYX(I,J,K+1)+EYZ(I,J,K+1)-EY(I,J,K))*DTMDZ

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HXY(I,J,K)=HXY(I,J,K)*DTMSY-(EZ(I,J+1,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDY
	   HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ+(EYX(I,J,K+1)+EYZ(I,J,K+1)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZB) THEN

           HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ+(EY(I,J,K+1)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ
	   HXY(I,J,K)=HXY(I,J,K)*DTMSY-(EZX(I,J+1,K)+EZY(I,J+1,K)
     $       -EZX(I,J,K)-EZY(I,J,K))*DTMSDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYSM1
     $                                          .AND.K.EQ.NZSM1) THEN

	    HX(I,J,K)=HX(I,J,K)
     $       -(EZX(I,J+1,K)+EZY(I,J+1,K)-EZ(I,J,K))*DTMDY
     $                +(EYX(I,J,K+1)+EYZ(I,J,K+1)-EY(I,J,K))*DTMDZ
C
C         PML
C
          ELSE

           HXY(I,J,K)=HXY(I,J,K)*DTMSY
     $              -(EZX(I,J+1,K)+EZY(I,J+1,K)
     $              -EZX(I,J,K)-EZY(I,J,K))*DTMSDY
           HXZ(I,J,K)=HXZ(I,J,K)*DTMSZ
     $              +(EYX(I,J,K+1)+EYZ(I,J,K+1)
     $              -EYX(I,J,K)-EYZ(I,J,K))*DTMSDZ

          ENDIF
      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHY(I,J,K)
      INCLUDE 'F-STRIP.PAR'

      XI=1*I
      XK=1*K

      IF(I.LE.NXB) THEN
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XNPML-XI+0.5)/XNPML)**2.
      ELSE
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XI-XN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(K.LE.NZB) THEN
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XNPML-XK+0.5)/XNPML)**2.
      ELSE
        HSIGMAZ=SIGMAZH(IDPML(I,J,K))*((XK-ZN+XNPML+0.5)/XNPML)**2.
      ENDIF
      

      IF (HSIGMAZ.LE.0.) THEN
       DTMSZ=1.
       DTMSDZ=DT/(DELZ*XMU0)
      ELSE
       DTMSZ=EXP(-(HSIGMAZ*DT)/XMU0)
       DTMSDZ=(1.-DTMSZ)/(HSIGMAZ*DELZ)
      ENDIF

      IF (HSIGMAX.LE.0.) THEN
       DTMSX=1.
       DTMSDX=DT/(DELX*XMU0)
      ELSE
       DTMSX=EXP(-(HSIGMAX*DT)/XMU0)
       DTMSDX=(1.-DTMSX)/(HSIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZSM1) THEN


	   HY(I,J,K)=HY(I,J,K)-(EX(I,J,K+1)-EX(I,J,K))*DTMDZ
     $               +(EZX(I+1,J,K)+EZY(I+1,J,K)-EZ(I,J,K))*DTMDX

	  ELSEIF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZSM1) THEN

	   HY(I,J,K)=HY(I,J,K)-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $               -EX(I,J,K))*DTMDZ+(EZ(I+1,J,K)-EZ(I,J,K))*DTMDX
     
     	  ELSEIF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZSM1) THEN

	   HY(I,J,K)=HY(I,J,K)-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $              -EX(I,J,K))*DTMDZ
     $              +(EZX(I+1,J,K)+EZY(I+1,J,K)-EZ(I,J,K))*DTMDX


	  ELSEIF (I.EQ.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HYX(I,J,K)=HYX(I,J,K)*DTMSX+(EZ(I+1,J,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDX
	   HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ-(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ


	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.GT.NYB.AND.J.LT.NYS
     $                                          .AND.K.EQ.NZB) THEN
           HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ-(EX(I,J,K+1)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ
	   HYX(I,J,K)=HYX(I,J,K)*DTMSX+(EZX(I+1,J,K)+EZY(I+1,J,K)
     $                           -EZX(I,J,K)-EZY(I,J,K))*DTMSDX

C
C         PML
C
	  ELSE

           HYX(I,J,K)=HYX(I,J,K)*DTMSX
     $              +(EZX(I+1,J,K)+EZY(I+1,J,K)
     $              -EZX(I,J,K)-EZY(I,J,K))*DTMSDX
           HYZ(I,J,K)=HYZ(I,J,K)*DTMSZ
     $              -(EXY(I,J,K+1)+EXZ(I,J,K+1)
     $              -EXY(I,J,K)-EXZ(I,J,K))*DTMSDZ

          ENDIF

      RETURN
      END

C     ********************************************************
      SUBROUTINE PMLHZ(I,J,K)
      INCLUDE 'F-STRIP.PAR'

      XJ=1*J
      XI=1*I

      IF(I.LE.NXB) THEN
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XNPML-XI+0.5)/XNPML)**2.
      ELSE
        HSIGMAX=SIGMAXH(IDPML(I,J,K))*((XI-XN+XNPML+0.5)/XNPML)**2.
      ENDIF
      IF(J.LE.NYB) THEN
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XNPML-XJ+0.5)/XNPML)**2.
      ELSE
        HSIGMAY=SIGMAYH(IDPML(I,J,K))*((XJ-YN+XNPML+0.5)/XNPML)**2.
      ENDIF

      IF (HSIGMAY.LE.0.) THEN
       DTMSY=1.
       DTMSDY=DT/(DELY*XMU0)
      ELSE
       DTMSY=EXP(-(HSIGMAY*DT)/XMU0)
       DTMSDY=(1.-DTMSY)/(HSIGMAY*DELY)
      ENDIF

      IF (HSIGMAX.LE.0.) THEN
       DTMSX=1.
       DTMSDX=DT/(DELX*XMU0)
      ELSE
       DTMSX=EXP(-(HSIGMAX*DT)/XMU0)
       DTMSDX=(1.-DTMSX)/(HSIGMAX*DELX)
      ENDIF
C
C          PML INTERFACE
C
          IF (I.EQ.NXSM1.AND.J.GT.NYB.AND.J.LT.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $               -EY(I,J,K))*DTMDX+(EX(I,J+1,K)-EX(I,J,K))*DTMDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXSM1.AND.J.EQ.NYSM1
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EY(I+1,J,K)-EY(I,J,K))*DTMDX
     $               +(EXY(I,J+1,K)+EXZ(I,J+1,K)-EX(I,J,K))*DTMDY

	  ELSEIF (I.GT.NXB.AND.I.LT.NXS.AND.J.EQ.NYB
     $                              .AND.K.GT.NZB.AND.K.LT.NZS) THEN
	   HZY(I,J,K)=HZY(I,J,K)*DTMSY+(EX(I,J+1,K)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY
	   HZX(I,J,K)=HZX(I,J,K)*DTMSX-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX


	  ELSEIF (I.EQ.NXB.AND.J.GT.NYB.AND.J.LT.NYS
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZX(I,J,K)=HZX(I,J,K)*DTMSX-(EY(I+1,J,K)
     $                           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX
	   HZY(I,J,K)=HZY(I,J,K)*DTMSY+(EXY(I,J+1,K)+EXZ(I,J+1,K)
     $                           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY


	  ELSEIF (I.EQ.NXSM1.AND.J.EQ.NYSM1
     $                               .AND.K.GT.NZB.AND.K.LT.NZS) THEN

	   HZ(I,J,K)=HZ(I,J,K)-(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $               -EY(I,J,K))*DTMDX
     $               +(EXY(I,J+1,K)+EXZ(I,J+1,K)-EX(I,J,K))*DTMDY

C
C         PML
C
	  ELSE

           HZX(I,J,K)=HZX(I,J,K)*DTMSX
     $           -(EYX(I+1,J,K)+EYZ(I+1,J,K)
     $           -EYX(I,J,K)-EYZ(I,J,K))*DTMSDX
           HZY(I,J,K)=HZY(I,J,K)*DTMSY
     $           +(EXY(I,J+1,K)+EXZ(I,J+1,K)
     $           -EXY(I,J,K)-EXZ(I,J,K))*DTMSDY
          ENDIF

      RETURN
      END

C     ********************************************************
      
      SUBROUTINE PEC
      INCLUDE 'F-STRIP.PAR'
       
      DO K=1,NZ
       DO J=1,NY
	 
	 I=1
	 EYX(I,J,K)=0.0
	 EYZ(I,J,K)=0.0
	 EZX(I,J,K)=0.0
	 EZY(I,J,K)=0.0
c         EY(I,J,K)=0.0
c         EZ(I,J,K)=0.0
	   
	 I=NX
	 EYX(I,J,K)=0.0
	 EYZ(I,J,K)=0.0
	 EZX(I,J,K)=0.0
	 EZY(I,J,K)=0.0
C         EY(I,J,K)=0.0
C         EZ(I,J,K)=0.0
	 
       ENDDO
      ENDDO
       
      DO K=1,NZ
       DO I=1,NX
	 
         J=1
         EXY(I,J,K)=0.0
         EXZ(I,J,K)=0.0
         EZX(I,J,K)=0.0
         EZY(I,J,K)=0.0
         EX(I,J,K)=0.0
         EZ(I,J,K)=0.0
	   
	 J=NY
	 EXY(I,J,K)=0.0
	 EXZ(I,J,K)=0.0
	 EZX(I,J,K)=0.0
	 EZY(I,J,K)=0.0
C         EX(I,J,K)=0.0
C         EZ(I,J,K)=0.0
	 
       ENDDO
      ENDDO

      DO J=1,NY
       DO I=1,NX
	 
	 K=1
	 EXY(I,J,K)=0.0
	 EXZ(I,J,K)=0.0
	 EYX(I,J,K)=0.0
	 EYZ(I,J,K)=0.0
C         EX(I,J,K)=0.0
C         EY(I,J,K)=0.0
	   
	 K=NZ
	 EXY(I,J,K)=0.0
	 EXZ(I,J,K)=0.0
	 EYX(I,J,K)=0.0
	 EYZ(I,J,K)=0.0
C         EX(I,J,K)=0.0
C         EY(I,J,K)=0.0

       ENDDO
      ENDDO

      RETURN
      END
C*****************************************************************
