C****************************************************************
C   PROGRAM     : SSPE.FOR                                      *
C   TERRAIN TYPE: STAIRCASE / DIRICHLET BC                      *
C   METHOD      : 1-WAY PARABOLIC EQUATION                      *
C   AUTHORS     : FUNDA AKLEMAN & LEVENT SEVG                  *
C   DATE        : JUNE 2002                                     *
C                                                               *
C   XMAX = 500M                                                 *
C   FREQUENCY (100 MHz - 1 GHz)                                 *
C   SIGMA = 2                                                   *
C   MIN SOURCE HEIGHT= 7 M                                      *
C   512 HEIGHT SAMPLES                                          *
C****************************************************************
      INTEGER IH(2)
      PARAMETER(N=1024,K=10,PI=3.1415927)
      CHARACTER CR      
      REAL WAVEKX(N),REFR(N),PI,A0,Z,Z2,ALFA,K0
      COMPLEX LFFT(N),CC,CUNIT,CSH,LFREET(N),CCFREE,CREFR,CREFREE 
      COMMON/AA/DELX,DELZ,HFUNC(1000),X(N),IG

      OPEN(UNIT=24,FILE='RANGE.DAT')
      OPEN(UNIT=25,FILE='HEIGHT.DAT')
      
C**********************
C     INPUT VALUES    *
C**********************
      OPEN(UNIT=15,FILE='SSPE.INP')
      READ(15,*) F0
      READ(15,*) HV
      READ(15,*) Z2
      READ(15,*) HOBS
      READ(15,*) NCON
      READ(15,*) NSEN
      IF(NSEN.EQ.1.OR.NSEN.EQ.0) THEN
        READ(15,*) HTER1,D1,W1,NTER1
      ELSEIF(NSEN.EQ.2) THEN
        READ(15,*) HTER1,D1,W1,NTER1
        READ(15,*) HTER2,D2,W2,NTER2
      ENDIF
      READ(15,*) NREF
      CLOSE(15)

      IF(NCON.EQ.1) THEN
         OPEN(UNIT=30, FILE='CONTOUR.DAT')
      END IF

C******** INITIAL PARAMETERS ********************
      K0=2*PI*F0*1.E6/3E8 !FREE SPACE WAVENUMBER
      SIGMA=2
      ALFA=0.0005
      XMAX=1000.
      XF=XMAX*0.5
      Z1=0.
      DELZ=Z2/1000.
      IG=N/2+(N*HOBS)/(2*XMAX) 

      IF(NSEN.LT.2) THEN
          CALL TERSCE1(D1,W1,HTER1,NTER1,DELZ,NSEN)
      ELSEIF(NSEN.EQ.2) THEN
          CALL TERSCE2(D1,W1,HTER1,NTER1,D2,W2,HTER2,NTER2,DELZ)
      END IF  

C***** SET INITIALS TO ZERO ********
      DO L=1,1000
       HFUNC(L)=0.
      ENDDO

      DO L=1,N
       X(L)=0.
       REFR(L)=0.
      ENDDO

C***** READ TERRAIN DATA FROM TER.DAT FILE ********

      IF(NSEN.EQ.1.OR.NSEN.EQ.2) THEN
        OPEN(35,FILE='TER.DAT')
        DO LZ=1,1000
          READ(35,*,END=100) DUMMY, HFUNC(LZ)
        ENDDO
100     CONTINUE
        CLOSE(35)
      END IF

      CALL SOURCE(LFFT,LFREET,HV,SIGMA,XMAX)
      CUNIT=(.0,1.)

C**********************************
C   ASSIGN WAVENUMBER VALUES      *
C**********************************
      DO L=1,N
         WAVEKX(L)=2.*PI*(-1/(2*DELX)+(L-1)/(N*DELX))
      END DO

      H=HFUNC(1)/DELX
      IH(2)=HFUNC(1)/DELX
      IF ((H-1.*IH(2)).GT.0.5) IH(2)=IH(2)+1 

C*******************************
C     RANGE LOOP BEGINS        *
C*******************************        
      DO LZ=1,1000
        WRITE(*,*) 'CALCULATING RANGE=',(LZ-1)*delz
        H=HFUNC(LZ)/DELX
        IH(1)=H/DELX
        IF ((H-1.*IH(1)).GT.0.5) IH(1)=IH(1)+1 

        IF(IH(1).LE.IH(2)) THEN
          DO KH=1,IH(2)
            LFFT(N/2+KH)=0.
          ENDDO
        ELSE
          DO KH=1,IH(1)
            LFFT(N/2+KH)=0.
          ENDDO
        ENDIF

        DO L=1,N/2
	 LFFT(L)=-LFFT(N-L+1)
        ENDDO
        NNEW=N/2-IH(1)

c********* LINEER PROFILE ***************************

       IF(NREF.EQ.1) THEN
         A0=2.*(1.17*1.E-7)  ! 117 MUnit/km
       ELSEIF(NREF.EQ.2) THEN
         A0=2.*(1.6*1.E-7)   ! 160 MUnit/km
       ELSEIF(NREF.EQ.3) THEN
         A0=2.*(0.50*1.E-7) !  50 MUnit/km
       ELSE
         A0=-2.*(0.3*1.E-7) ! -30 MUnit/km
       ENDIF

       DO L=N/2+1,N
          REFR(L)=1.0003**2+A0*X(L) 
       ENDDO 
	   
       DO L=1,N/2
          REFR((N/2-L+1))=REFR((N/2+L))
       ENDDO

C***********************************
C     GO TO WAVENUMBER SPACE       *
C***********************************

	CALL FFT(N,K,LFFT,1)
        CALL FFT(N,K,LFREET,1)

	DO L=1,N
          CC=(.0,.0)+CEXP(-CUNIT*DELZ*WAVEKX(L)**2/(2*K0))
          LFFT(L)=CC*LFFT(L)
          LFREET(L)=CC*LFREET(L)
	END DO 

C***********************************
C     GO BACK TO SPATIAL SPACE     *
C***********************************

	CALL FFT(N,K,LFFT,0)
        CALL FFT(N,K,LFREET,0)

C ****** APPLY ABSORBING BOUNDARY *************
	DO L=1,N
         IF(ABS(X(L)).GT.XF) THEN
           CREFR=(.0,.0)+CMPLX(REFR(L),ALFA*(ABS(X(L))-XF))
           CREFREE=(.0,.0)+CMPLX(1.,ALFA*(ABS(X(L))-XF))
         ELSE
           CREFR=(.0,.0)+REFR(L)
           CREFREE=(.0,.0)+(1.,0.)
         ENDIF

C ******** SET VALUES INSIDE PEC TERRAIN ZERO **********
           IF(L.LE.NNEW.OR.L.GT.(N-NNEW)) THEN
             CC=(.0,.0)+CEXP(CUNIT*DELZ*K0*(CREFR-1)/2)/N
           ELSE
             CC=(.0,.0) 
           ENDIF
           CCFREE=(.0,.0)+CEXP(CUNIT*DELZ*K0*(CREFREE-1)/2)/N
           LFFT(L)=LFFT(L)*CC
           LFREET(L)=LFREET(L)*CCFREE
	END DO

C****************************************
        PF=20.*ALOG10(CABS(LFFT(IG)/LFREET(IG)))
        WRITE(24,*) (LZ-1)*DELZ*1.E-3,PF,CABS(LFFT(IG)),CABS(LFREET(IG))

C****** CONTOUR DATA FILE ***************
        NSAY=NSAY+1
        IF(NCON.EQ.1.AND.MOD(NSAY,1).EQ.0) THEN
          DO L=N/2+1,N/2+150 
           PF=CABS(LFFT(L)/LFREET(L))
          WRITE(30,*) PF
          END DO
        END IF
C****************************************
        IH(2)=IH(1)

      END DO
C****************************
C   END OF RANGE LOOP       *
C****************************
      write(*,*) lz
C*********************************
C     WRITE HEIGHT PROFILE       *
C*********************************
      DO L=N/2+1,N
          IF(CABS(LFFT(L)).NE.0) THEN
            PF=20.*ALOG10(CABS(LFFT(L)/LFREET(L)))
          ELSE
            PF=0.
          ENDIF
          WRITE(25,*) X(L),PF,CABS(LFFT(L)),CABS(LFREET(L))
      ENDDO
    
222   STOP
      END
C**************************
C   END OF THE PROGRAM    *
C**************************
      SUBROUTINE SOURCE(LFFT,LFREET,HV,SIGMA,XMAX)
      PARAMETER(N=1024,K=10)
      PARAMETER(PI=3.1415927)
      COMPLEX J,CS,LFFT(N),LFREET(N)
      COMMON/AA/DELX,DELZ,HFUNC(1000),X(N),IG
      OPEN(UNIT=13,FILE='SOURCE.DAT')

      J=(0.,1.)
      DELX=2*XMAX/(N-1)

      DO L=1,N
       LFFT(N)=(0.,0.)
       LFREET(N)=(0.,0.)
      ENDDO

      DO L=N,1,-1
       X(L)=XMAX-(N-L)*DELX 
      ENDDO
      
      DO L=N,1,-1
C*************************************
C   ODD SYMMETRIC INPUT PROFILE      *
C*************************************
      IF(L.GT.N/2) THEN
        LFFT(L)=(.0,.0)+EXP(-(X(L)-HV)**2/(2*SIGMA**2))
        LFREET(L)=LFFT(L)
      ELSE
        LFFT(L)=-LFFT(N-L+1)
        LFREET(L)=0.
      ENDIF
      ENDDO

      DO L=1,N
       WRITE(13,*) X(L),cabs(LFFT(L)),cabs(LFREET(L))
      ENDDO
      CLOSE(13)
      RETURN
      END

C************************************************************
       SUBROUTINE TERSCE1(D1,W1,HTER,NTER,DELZ,NSEN)
       OPEN(35,FILE='TER.DAT')

       SLOPE=0.
       IF(HTER.GT.0.) SLOPE= 2.*HTER/W1

       DO LZ=1,1000 
        ZZ=(LZ-1)*DELZ
        IF(NTER.EQ.1) THEN
          IF(ZZ.GE.D1.AND.ZZ.LE.(D1+W1/2.)) THEN
            HT = SLOPE*(ZZ-D1)
          ELSEIF(ZZ.GE.(D1+W1/2).AND.ZZ.LT.(D1+W1)) THEN
            HT = HTER - SLOPE*(ZZ-D1-W1/2)
          ELSE
            HT=0.
          ENDIF
        ELSE
          IF(ZZ.GE.D1.AND.ZZ.LE.(D1+W1)) THEN
            HT = HTER
          ELSE
            HT=0.
          ENDIF
        END IF
        IF(NSEN.EQ.0) HT=0.
        WRITE(35,*) ZZ,HT
       ENDDO
       CLOSE(35)
       RETURN 
       END
C************************************************************
       SUBROUTINE TERSCE2(D1,W1,HTER1,NTER1,D2,W2,HTER2,NTER2,DELZ)
       OPEN(35,FILE='TER.DAT')

       SLOPE1=0.
       SLOPE2=0.

       IF(HTER1.GT.0.) SLOPE1= 2.*HTER1/W1
       IF(HTER2.GT.0.) SLOPE2= 2.*HTER2/W2

       ZBAS=D1+W1+DELZ
       NZBAS=ZBAS/DELZ

C****** FIRST TERRAIN **********************************
       DO LZ=1,NZBAS-1 
        ZZ=(LZ-1)*DELZ
        IF(NTER1.EQ.1) THEN
          IF(ZZ.GE.D1.AND.ZZ.LE.(D1+W1/2.)) THEN
            HT = SLOPE1*(ZZ-D1)
          ELSEIF(ZZ.GE.(D1+W1/2).AND.ZZ.LT.(D1+W1)) THEN
            HT = HTER1 - SLOPE1*(ZZ-D1-W1/2)
          ELSE
            HT=0.
          ENDIF
        ELSE
          IF(ZZ.GE.D1.AND.ZZ.LE.(D1+W1)) THEN
            HT = HTER1
          ELSE
            HT=0.
          ENDIF
        END IF
        WRITE(35,*) ZZ,HT
       ENDDO

C****** SECOND TERRAIN **********************************


       DO LZ=NZBAS,1000 
        ZZ=(LZ-1)*DELZ
        IF(NTER2.EQ.1) THEN
          IF(ZZ.GE.(ZBAS+D2).AND.ZZ.LE.(ZBAS+(D2+W2/2.))) THEN
            HT = SLOPE2*(ZZ-ZBAS-D2)
          ELSEIF(ZZ.GE.(ZBAS+D2+W2/2).AND.ZZ.LT.(ZBAS+D2+W2)) THEN
            HT = HTER2 - SLOPE2*(ZZ-ZBAS-D2-W2/2)
          ELSE
            HT=0.
          ENDIF
        ELSE
          IF(ZZ.GE.(ZBAS+D2).AND.ZZ.LE.(ZBAS+D2+W2)) THEN
            HT = HTER2
          ELSE
            HT=0.
          ENDIF
        END IF
        WRITE(35,*) ZZ,HT
       ENDDO
       CLOSE(35)
       RETURN 
       END
C*******************************************************       
      SUBROUTINE FFT(N,M,A,SIGN)
      INTEGER I,IP,J,K,L,M,N,N2,NM1,LE,LE2,SIGN
      COMPLEX A(1024),U,W,TEMP
      REAL PI,PIOLI
      PI=3.1415927
      CALL CEVIRME(N,A)
      N2=N/2
      NM1=N-1
      J=1
      DO 30 I=1,NM1
      IF(I.GE.J) GO TO 10
      TEMP=A(J)
      A(J)=A(I)
      A(I)=TEMP
   10 K=N2
   20 IF(K.GE.J) GO TO 30
      J=J-K
      K=K/2
      GO TO 20
   30 J=J+K
      DO 60 L=1,M
      LE=2**L
      LE2=LE/2
      U=CMPLX(1.,0.)
      PIOLI=PI/LE2
      IF(SIGN.EQ.1) THEN
	 W=CMPLX(COS(PIOLI),-SIN(PIOLI))
      ELSE
	 W=CMPLX(COS(PIOLI),SIN(PIOLI))
      ENDIF
      DO 50 J=1,LE2
      DO 40 I=J,N,LE
      IP=I+LE2
      TEMP=A(IP)*U
      A(IP)=A(I)-TEMP
      A(I)=A(I)+TEMP
   40 CONTINUE
      U=U*W
   50 CONTINUE
   60 CONTINUE
      CALL CEVIRME(N,A)
      RETURN
      END

C***************************
      SUBROUTINE CEVIRME(N,A)
      INTEGER N,I
      COMPLEX A(1024),B(1024)
      DO 10 I=1,N/2
      B(I)=A(N/2+I)
   10 B(N/2+I)=A(I)
      DO 20 I=1,N
   20 A(I)=B(I)
      RETURN
      END


